/*
 * da9052_adc.h: ADC driver header file for DA9052
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 *  (05/08/2009):	Initial version
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef _DA9052_ADC_H
#define _DA9052_ADC_H

/*--------------------------------------------------------------------------*/
/* System wide include files						    */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Module specific include files					    */
/*--------------------------------------------------------------------------*/
#include "da9052_gpio.h"
/*--------------------------------------------------------------------------*/
/* Type Definitions							    */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Constant Definitions							    */
/*--------------------------------------------------------------------------*/

#define DA9052_ADC_DEVICE_NAME				"da9052_adc"
#define DA9052_ADC_DEVICE_MAJOR_NO			246

/* To enable debug output, set this to 1 */
#define		DA9052_ADC_DEBUG			0
#undef DA9052_DEBUG
#if DA9052_ADC_DEBUG
#define DA9052_DEBUG( fmt, args... ) printk( KERN_CRIT "" fmt, ##args )
#else
#define DA9052_DEBUG( fmt, args... )
#endif

/* used for debugging non IOCTL functions */
#define DA9052_UNIT_TESTING_ADC				1 

/* IOCTL defines */
#define DA9052_ADC_IOCTL_SET_ADC_MODE			2
#define DA9052_ADC_IOCTL_SET_SAMPLING_INTERVAL		3
#define DA9052_ADC_IOCTL_CONFIGURE_THRESHOLDS		4
#define DA9052_ADC_IOCTL_SET_ADCIN5_COMPARATOR		5
#define DA9052_ADC_IOCTL_SET_TBAT_SOURCE_CURRENT	6
#define DA9052_ADC_IOCTL_SET_ADCIN4_SOURCE_CURRENT	7
#define DA9052_ADC_IOCTL_START_ADC			8
#define DA9052_ADC_IOCTL_STOP_ADC			9
#define DA9052_ADC_IOCTL_READ_ADC			10
#define DA9052_ADC_IOCTL_GET_EVENT			11
#define DA9052_ADC_IOCTL_REGISTER_EVENT			12
#define DA9052_ADC_IOCTL_UNREGISTER_EVENT		13

#if DA9052_ADC_DEBUG
#define DA9052_ADC_IOCTL_UT_VDDOUT			14
#define DA9052_ADC_IOCTL_UT_ICH				15
#define DA9052_ADC_IOCTL_UT_TBAT			16
#define DA9052_ADC_IOCTL_UT_VBAT			17
#define DA9050_ADC_IOCTL_UT_TJUNC			18
#define DA9052_ADC_IOCTL_UT_VBBAT			19
#endif

/* Channel Definations */
#define ADC_VDDOUT					0
#define ADC_ICH						1
#define ADC_TBAT					2
#define ADC_VBAT					3
#define ADC_ADCIN4					4
#define ADC_ADCIN5					5
#define ADC_ADCIN6					6
#define ADC_TSI						7
#define ADC_TJUNC					8
#define ADC_VBBAT					9
/* Mode selection Definations */
#define AUTO_MODE					0
#define MANUAL_MODE					1

/* Error codes */
#define ADC_INVALID_CHANNEL_ERROR			2
#define ADC_INVALID_MODE_ERROR				3
#define ADC_INVALID_TIME_ERROR				4
#define ADC_SSC_BUS_ERROR				5
#define ADC_INVALID_VALUE				6
#define ADC_CHANNEL_NOT_STARTED_ERROR			7
#define ADC_NO_IOCTL_CMD				8
#define ADC_INVALID_EVENT				9
#define ADC_EVENT_REGISTRATION_FAILED			10
#define ADC_EVENT_UNREGISTRATION_FAILED			11
#define ADC_MANCONV_TIMEOUT_ERROR			12
#define ADC_CH4_NOT_CONF				13
#define ADC_CH5_NOT_CONF				14
#define ADC_CH6_NOT_CONF				15

/* Parameters */
#define TBAT_50UA_ONE_SLOT				0
#define TBAT_50UA_PREMANENTLY				1

#define ADC_INTERVAL_10MS				0
#define ADC_INTERVAL_1MS				1

/*--------------------------------------------------------------------------*/
/* User Configurations                                                     */
/*--------------------------------------------------------------------------*/

/* Macros to configure the ADC port pins */
#if (DA9052_GPIO_PIN_0 == DA9052_GPIO_CONFIG_ADC)
#define ADC_CONF_ADC4					ENABLE
#else
#define ADC_CONF_ADC4					DISABLE
#endif 
#if (DA9052_GPIO_PIN_1 == DA9052_GPIO_CONFIG_ADC)
#define ADC_CONF_ADC5					ENABLE
#else
#define ADC_CONF_ADC5					DISABLE
#endif 
#if (DA9052_GPIO_PIN_2 == DA9052_GPIO_CONFIG_ADC)
#define ADC_CONF_ADC6					ENABLE
#else
#define ADC_CONF_ADC6					DISABLE
#endif 

/* Maximum retry count to check manual conversion over */
#define MAX_MANCONV_RETRY_COUNT				8
/*--------------------------------------------------------------------------*/
/* Structure Definitions						    */
/*--------------------------------------------------------------------------*/

/**
 * struct threshold_res - 
 *				Structure for configuring the thereshold values
 * @auto4_high	: HIGH threshold value for ADCIN4
 * @auto4_low	: LOW threshold value for ADCIN4
 * @auto5_high	: HIGH threshold value for ADCIN5
 * @auto5_low	: LOW threshold value for ADCIN5
 * @auto6_high	: HIGH threshold value for ADCIN6
 * @auto6_low	: LOW threshold value for ADCIN6
 *
 */
typedef struct {
	u8 auto4_high;
	u8 auto4_low;
	u8 auto5_high;
	u8 auto5_low;
	u8 auto6_high;
	u8 auto6_low;
}threshold_res;

/**
 * struct da9052_adc_data_t - 
 *				Structure to hold ADC module data
 * @channel	: Channel on which action to be done
 * @mode	: AUTO / MANUAL mode selection
 * @interval	: 1ms or 10ms timer selection
 * @*data	: Pointer where data to be stored
 * @threshold	: Threshold values of ADCIN4, ADCIN5 and ADCIN6
 * @action	: Enable or Disable action
 * @event_type	: For event registration
 *
 */ 
 typedef struct {
	u8 channel;
	u8 mode;
	u8 interval;
	u16 *data;
	threshold_res threshold;
	u8 action;
	u8 event_type;
}da9052_adc_data_t;

/**
 * struct adc_event_occured_status - This structure stores the status of the 
 * 		events that have been reported by the event handler through 
 *		callback. This is required for handling multiple events occured 
 *		same time.
 *		adc_adcin4_e_occ	: Status of the ADCIN4 event occurance.
 *		adc_adcin5_e_occ	: Status of the ADCIN5 event occurance.
 *		adc_adcin6_e_occ	: Status of the ADCIN6 event occurance.
 *		adc_comp1v2_e_occ	: Status of the COMP1V2 event occurance.
 *
*/
typedef union {
	u8 adc_event_status;
	struct {
		u8 adc_adcin4_e_occ 	: 1;
		u8 adc_adcin5_e_occ 	: 1;
		u8 adc_adcin6_e_occ 	: 1;
		u8 adc_comp1v2_e_occ	: 1;
		u8 dummy		: 4;
	} event_bits_t;
} adc_event_occ_status_t;

/*--------------------------------------------------------------------------*/
/* Global Variables							    */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Inline Functions							    */
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*/
/* External Functions							    */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Other Functions							    */
/*--------------------------------------------------------------------------*/
 
#endif /* _DA9052_ADC_H */
