/*
 * da9052_gpio.h: GPIO driver file for DA9052
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (05/05/2009)		Unit tested Code
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef _DA9052_GPIO_H
#define _DA9052_GPIO_H

/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Type Definitions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Constant Definitions                                                     */
/*--------------------------------------------------------------------------*/

/* String for the Identification */
#define DA9052_GPIO_DEVICE_NAME		"da9052_gpio"

/* Major Number of the device driver */
#define DA9052_GPIO_DEVICE_MAJOR_NO		(250)

/* Calculate the size of the variable */
#define len_array(arr) (sizeof(arr)/sizeof(arr[0]))

/*--------------------------------------------------------------------------*/
/* Error Codes */
/*--------------------------------------------------------------------------*/
/* Failed Copy from User */
#define COPY_FROM_USER_FAILED			(3)

/* Failed Copy to User */
#define COPY_TO_USER_FAILED			(4)

/* Invalid Ioctl */
#define DA9052_GPIO_INVALID_IOCTL		(5)

//#define INVALID_SIGNAL_LEVEL			(6)
/* Invalid functionality code for the 
	corresponding pin */
#define DA9052_GPIO_INVALID_FUNCTION		(7)

/* Invalid event for the corresponding pin */
#define INVALID_EVENT				(8)

/* Event Registration with EH failed */
#define EVENT_REGISTRATION_FAILED		(9)

/* Event Unregistration with EH failed */
#define EVENT_UNREGISTRATION_FAILED		(10)

/* Invalid Signal level in case of Input */
/* Invalid Voltage source in case of Output */
#define INVALID_TYPE				(11)

/* Invalid Debouncing in case of Input */
/* Invalid Signal level in case of Output */
#define INVALID_MODE				(12)

/* Invalid Port Number */
#define INVALID_PORTNUMBER			(13)

/* Corresponding Pin has Already 
	been registered with EH */
#define EVENT_ALREADY_REGISTERED		(14)

/* Corresponding Pin has Already 
	been Unregistered with EH */
#define EVENT_UNREGISTERED			(15)

/* --------------------------------------------------------------------------*/
/* IOCTL calls that are permitted to the /dev/gpio interface, if */
/* any of the drivers are enabled. */
/*--------------------------------------------------------------------------*/
/* Read GPIO port */
#define DA9052_GPIO_IOCTL_READ_PORT		(1)

/* Write GPIO port */
#define DA9052_GPIO_IOCTL_WRITE_PORT		(2)

/* Configure GPIO port */
#define DA9052_GPIO_IOCTL_CONFIGURE_PORT	(3)

/* Read multiple GPIO ports */
#define DA9052_GPIO_IOCTL_READ_ALL_PORTS	(4)

/* Get last event occured */
#define DA9052_GPIO_IOCTL_GET_EVENT		(5)

/* Register for an event */
#define DA9052_GPIO_IOCTL_REGISTER_EVENT	(6)

/* Unregister for an event */
#define DA9052_GPIO_IOCTL_UNREGISTER_EVENT	(7)


/*--------------------------------------------------------------------------*/
/* Defines for Various Pin Configurations of the Gpio pins */
/*--------------------------------------------------------------------------*/
/* ADC pin Configuration */
#define DA9052_GPIO_CONFIG_ADC			(1)

/* TSI pin Configuration */
#define DA9052_GPIO_CONFIG_TSI			(2)

/* Power Manager pin Configuration */
#define DA9052_GPIO_CONFIG_PM			(3)

/* USB ID detect pin Configuration */
#define DA9052_GPIO_CONFIG_ACC_ID_DET		(4)

/* General Purpose FB1 Configuration */
#define DA9052_GPIO_CONFIG_GP_FB1		(5)

/* VDD fault Configuration */
#define DA9052_GPIO_CONFIG_VDD_FAULT		(6)

/* I2C Pin configuration */
#define DA9052_GPIO_CONFIG_I2C			(7)	

/* Input/Output Pin Configuration */
#define DA9052_GPIO_CONFIG			(8)

/*--------------------------------------------------------------------------*/
/* Currently used defines for GPIO PINs  */
/*--------------------------------------------------------------------------*/

/* ADC pin Configuration */
#define DA9052_GPIO_PIN_0 	DA9052_GPIO_CONFIG_ADC
#define DA9052_GPIO_PIN_1 	DA9052_GPIO_CONFIG_ADC
#define DA9052_GPIO_PIN_2 	DA9052_GPIO_CONFIG_ADC

/* TSI pin Configuration */
#define DA9052_GPIO_PIN_3 	DA9052_GPIO_CONFIG_TSI
#define DA9052_GPIO_PIN_4 	DA9052_GPIO_CONFIG_TSI
#define DA9052_GPIO_PIN_5 	DA9052_GPIO_CONFIG_TSI
#define DA9052_GPIO_PIN_6 	DA9052_GPIO_CONFIG_TSI
#define DA9052_GPIO_PIN_7 	DA9052_GPIO_CONFIG_TSI

/* Input Pin Configuration */
#define DA9052_GPIO_PIN_8 	DA9052_GPIO_CONFIG
#define DA9052_GPIO_PIN_9 	DA9052_GPIO_CONFIG
#define DA9052_GPIO_PIN_10 	DA9052_GPIO_CONFIG
#define DA9052_GPIO_PIN_11 	DA9052_GPIO_CONFIG

/* Output Pin Configuration */
#define DA9052_GPIO_PIN_12 	DA9052_GPIO_CONFIG
#define DA9052_GPIO_PIN_13 	DA9052_GPIO_CONFIG

/* Configuring for High Speed I2C */
#define DA9052_GPIO_PIN_14 	DA9052_GPIO_CONFIG_I2C
#define DA9052_GPIO_PIN_15 	DA9052_GPIO_CONFIG_I2C

/*--------------------------------------------------------------------------*/
/* To enable debug output, set this to 1 */
/*--------------------------------------------------------------------------*/
#define	DA9052_GPIO_DEBUG			(0)

#undef DA9052_DEBUG
#if DA9052_GPIO_DEBUG
#define DA9052_DEBUG( fmt, args... ) printk( KERN_CRIT "" fmt, ##args )
#else
#define DA9052_DEBUG( fmt, args... )
#endif

/**
 * enum ip_op_type - GPIO pin functionality
 * @ALTERNATE_FUNCTIONALITY: configured as alternate functionality
 * @INPUT: configured as GPI
 * @OUTPUT_OPENDRAIN: configured as GPO open-drain
 * @OUTPUT_PUSHPULL: configured as GPO push-pull
 *
 */
enum ip_op_type {
	ALTERNATE_FUNCTIONALITY = 0,
	INPUT,
	OUTPUT_OPENDRAIN,
	OUTPUT_PUSHPULL
};


/**
 * enum ip_type - GPI type
 * @ACTIVE_LOW: active low
 * @ACTIVE_HIGH: active high
 *
 */
enum ip_type {
	ACTIVE_LOW = 0,
	ACTIVE_HIGH
};

/**
 * enum op_type - GPO type
 * @SUPPLY_VDD_IO1: supplied from VDDIO1/ internal pull-up
 * @SUPPLY_VDD_IO2: supplied from VDDIO2/ external pull-up
 *
 */
enum op_type {
	SUPPLY_VDD_IO1 = 0,
	SUPPLY_VDD_IO2
};

/**
 * enum op_mode - GPO mode
 * @OUTPUT_LOWLEVEL: low
 * @OUTPUT_HIGHLEVEL: high
 *
 */
enum op_mode {
	OUTPUT_LOWLEVEL = 0,
	OUTPUT_HIGHLEVEL
};

/**
 * enum ip_mode - GPI mode
 * @DEBOUNCING_OFF: debouncing off
 * @DEBOUNCING_ON: debouncing on
 *
 */
enum ip_mode {
	DEBOUNCING_OFF = 0,
	DEBOUNCING_ON
};

/**
 * enum da9052_gpi_event_type - GPI event types
 * @GPI_0_EVENT: Event for GPI port 0
 * @GPI_1_EVENT: Event for GPI port 1
 * ...
 * @GPI_15_EVENT: Event for GPI port 15
 *
 */
enum da9052_gpi_event_type {
	GPI_0_EVENT = (1<<0),
	GPI_1_EVENT = (1<<1),
	GPI_2_EVENT = (1<<2),
	GPI_3_EVENT = (1<<3),
	GPI_4_EVENT = (1<<4),
	GPI_5_EVENT = (1<<5),
	GPI_6_EVENT = (1<<6),
	GPI_7_EVENT = (1<<7),
	GPI_8_EVENT = (1<<8),
	GPI_9_EVENT = (1<<9),
	GPI_10_EVENT = (1<<10),
	GPI_11_EVENT = (1<<11),
	GPI_12_EVENT = (1<<12),
	GPI_13_EVENT = (1<<13),
	GPI_14_EVENT = (1<<14),
	GPI_15_EVENT = (1<<15),
};



/*--------------------------------------------------------------------------*/
/* Default used defines for GPIO PINs  */
/*--------------------------------------------------------------------------*/

/*DEFAULT CONFIG FOR GPIO 0*/

#if (DA9052_GPIO_PIN_0 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO0_FUNCTION	INPUT
#define DEFAULT_GPIO0_TYPE	ACTIVE_LOW
#define DEFAULT_GPIO0_MODE	DEBOUNCING_ON
#endif

/*DEFAULT CONFIG FOR GPIO 1*/
#if (DA9052_GPIO_PIN_1 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO1_FUNCTION	INPUT
#define DEFAULT_GPIO1_TYPE	ACTIVE_LOW
#define DEFAULT_GPIO1_MODE	DEBOUNCING_ON
#endif

/*DEFAULT CONFIG FOR GPIO 2*/
#if (DA9052_GPIO_PIN_2 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO2_FUNCTION	INPUT
#define DEFAULT_GPIO2_TYPE	ACTIVE_LOW
#define DEFAULT_GPIO2_MODE	DEBOUNCING_ON
#endif

/*DEFAULT CONFIG FOR GPIO 3*/
#if (DA9052_GPIO_PIN_3 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO3_FUNCTION	INPUT
#define DEFAULT_GPIO3_TYPE	ACTIVE_LOW
#define DEFAULT_GPIO3_MODE	DEBOUNCING_ON
#endif

/*DEFAULT CONFIG FOR GPIO 4*/
#if (DA9052_GPIO_PIN_4 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO4_FUNCTION	OUTPUT_PUSHPULL
#define DEFAULT_GPIO4_TYPE	SUPPLY_VDD_IO1
#define DEFAULT_GPIO4_MODE	OUTPUT_LOWLEVEL
#endif
/*DEFAULT CONFIG FOR GPIO 5*/
#if (DA9052_GPIO_PIN_5 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO5_FUNCTION	OUTPUT_PUSHPULL
#define DEFAULT_GPIO5_TYPE	SUPPLY_VDD_IO1
#define DEFAULT_GPIO5_MODE	OUTPUT_LOWLEVEL
#endif

/*DEFAULT CONFIG FOR GPIO 6*/
#if (DA9052_GPIO_PIN_6 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO6_FUNCTION	OUTPUT_PUSHPULL
#define DEFAULT_GPIO6_TYPE	SUPPLY_VDD_IO1
#define DEFAULT_GPIO6_MODE	OUTPUT_LOWLEVEL
#endif

/*DEFAULT CONFIG FOR GPIO 7*/
#if (DA9052_GPIO_PIN_7 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO7_FUNCTION	OUTPUT_PUSHPULL
#define DEFAULT_GPIO7_TYPE	SUPPLY_VDD_IO1
#define DEFAULT_GPIO7_MODE	OUTPUT_LOWLEVEL
#endif

/*DEFAULT CONFIG FOR GPIO 8*/
#if (DA9052_GPIO_PIN_8 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO8_FUNCTION	INPUT
#define DEFAULT_GPIO8_TYPE	ACTIVE_LOW
#define DEFAULT_GPIO8_MODE	DEBOUNCING_ON
#endif

/*DEFAULT CONFIG FOR GPIO 9*/
#if (DA9052_GPIO_PIN_9 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO9_FUNCTION	INPUT
#define DEFAULT_GPIO9_TYPE	ACTIVE_LOW
#define DEFAULT_GPIO9_MODE	DEBOUNCING_ON
#endif

/*DEFAULT CONFIG FOR GPIO 10 - for RTC blinking LED */
#if (DA9052_GPIO_PIN_10 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO10_FUNCTION	OUTPUT_PUSHPULL
#define DEFAULT_GPIO10_TYPE	SUPPLY_VDD_IO2
#define DEFAULT_GPIO10_MODE	OUTPUT_HIGHLEVEL
#endif

/*DEFAULT CONFIG FOR GPIO 11 - for RTC blinking LED */
#if (DA9052_GPIO_PIN_11 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO11_FUNCTION	OUTPUT_PUSHPULL
#define DEFAULT_GPIO11_TYPE	SUPPLY_VDD_IO2
#define DEFAULT_GPIO11_MODE	OUTPUT_HIGHLEVEL
#endif

/*DEFAULT CONFIG FOR GPIO 12*/
#if (DA9052_GPIO_PIN_12 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO12_FUNCTION	OUTPUT_PUSHPULL
#define DEFAULT_GPIO12_TYPE	SUPPLY_VDD_IO1
#define DEFAULT_GPIO12_MODE	OUTPUT_LOWLEVEL
#endif
/*DEFAULT CONFIG FOR GPIO 13*/
#if (DA9052_GPIO_PIN_13 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO13_FUNCTION	OUTPUT_PUSHPULL
#define DEFAULT_GPIO13_TYPE	SUPPLY_VDD_IO1
#define DEFAULT_GPIO13_MODE	OUTPUT_LOWLEVEL
#endif

/*DEFAULT CONFIG FOR GPIO 14 - for LED4 */
#if (DA9052_GPIO_PIN_14 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO14_FUNCTION	OUTPUT_OPENDRAIN
#define DEFAULT_GPIO14_TYPE	SUPPLY_VDD_IO1
#define DEFAULT_GPIO14_MODE	OUTPUT_HIGHLEVEL
#endif

/*DEFAULT CONFIG FOR GPIO 15 - for LED5 */
#if (DA9052_GPIO_PIN_15 == DA9052_GPIO_CONFIG)
#define DEFAULT_GPIO15_FUNCTION	OUTPUT_OPENDRAIN
#define DEFAULT_GPIO15_TYPE	SUPPLY_VDD_IO1
#define DEFAULT_GPIO15_MODE	OUTPUT_HIGHLEVEL
#endif


/* Maximum number of Gpio Ports */
#define MAX_PORTNUMBER				(16)

/* Maximum number of ports per register */
#define MAX_PORTS_PER_REGISTER			(8)

/* No of shift operations to performed */
#define SHIFT_COUNT(no)				((no)%8)

/* Mask of even numbered port */
#define EVEN_PORT_FUNCTIONALITY 		(0x03)

/* Mask of odd numbered port */
#define ODD_PORT_FUNCTIONALITY 			(0x30)

/* Mask Upper nibble */
#define MASK_UPPER_NIBBLE 			(0xF0)

/* Mask Lower nibble */
#define MASK_LOWER_NIBBLE 			(0x0F)

/* Nibble Shift */
#define NIBBLE_SHIFT 				(4)

/* Mode setting for Even Port number Write */
#define EVEN_PORT_WRITE_MODE 			(1 << 3)

/* Mode setting for ODD Port number Write */
#define ODD_PORT_WRITE_MODE 			(1 << 7)
/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*/
/* Structure Definitions                                                    */
/*--------------------------------------------------------------------------*/

/**
 * struct da9052_gpio_read_write_t - Contains the i/p or o/p value for a GPIO
 * @port_number: gpio ports
 * @read_write_value: gpio value
 *
 */
typedef struct {
	u8 port_number:4;
	u8 read_write_value:1;
} da9052_gpio_read_write_t;

/**
 * struct da9052_gpio_multiple_read_t - Contains the signal level at each port
 * @port_number: all gpio ports
 *
 */
typedef struct {
	u8 signal_value[16];
} da9052_gpio_multiple_read_t;


/**
 * struct da9052_gpi_config_t - GPI configuration
 * @type: GPI Type
 * @mode: GPI mode
 *
 */
typedef struct {
	enum ip_type type;
	enum ip_mode mode;
} da9052_gpi_config_t;

/**
 * struct da9052_gpo_config_t - GPO configuration
 * @type: GPO Type
 * @mode: GPO mode
 *
 */
typedef struct {
	enum op_type type;
	enum op_mode mode;
} da9052_gpo_config_t; 

/**
 * union da9052_gpio_config_t - GPIO configuration either input or output
 * @input: GPI configuration
 * @output: GPO configuration
 *
 */
typedef union {
	da9052_gpi_config_t input;
	da9052_gpo_config_t output;
} da9052_gpio_config_t;

/**
 * struct da9052_gpio_t - GPIO information
 * @gpio_config: GPIO configuration either input or output
 * @gpio_function: GPIO function
 * @port_number: GPIO port number
 *
 */
typedef struct {
	da9052_gpio_config_t gpio_config;
	enum ip_op_type gpio_function;
	u8 port_number:4;
} da9052_gpio_t;



/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Inline Functions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* External Functions                                                       */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Other Functions                                                          */
/*--------------------------------------------------------------------------*/
s32 da9052_gpio_read_port(da9052_gpio_read_write_t *read_port);
s32 da9052_gpio_multiple_read(da9052_gpio_multiple_read_t *multiple_port);
s32 da9052_gpio_write_port(da9052_gpio_read_write_t *write_port);
s32 da9052_gpio_configure_port(da9052_gpio_t *gpio_data);
s32 da9052_gpio_register_event(u8 event_type);
s32 da9052_gpio_unregister_event(u8 event_type);

#endif /* _DA9052_GPIO_H */
