/*
 * da9052_pm.h: Power Manager for DA9052
 * Header file contain the structure, prototype and defines needed for 
 * PM driver API's
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (07/05/2009):	First Release version
 *
 * (25/06/2009):	Added support for multiple events occurring at the same 
 *			time
 *			Added support to control functions in power down mode
 *
 * (29/06/2009):	Implemented review comments:
 *			Used macros, modified function names etc.
 *
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef _DA9052_PM_H
#define _DA9052_PM_H

/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "da9052_lib.h"
/*--------------------------------------------------------------------------*/
/* Type Definitions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Constant Definitions                                                     */
/*--------------------------------------------------------------------------*/

/* PM Device name and static Major number macros */
#define DA9052_PM_DEVICE_NAME			"da9052_pm"
#define DA9052_PM_DEVICE_MAJOR_NO		248
#define TRUE					1
#define FALSE					0


/* PM Device Error Codes */
#define INVALID_BUCK_NUM			2
#define INVALID_LDO_NUM				3
#define INVALID_PM_EVENT			4
#define INVALID_BUCKCORE_VOLT_VALUE		5
#define INVALID_BUCKPRO_VOLT_VALUE		6
#define INVALID_BUCKMEM_VOLT_VALUE		7
#define INVALID_BUCKPERI_VOLT_VALUE		8
#define INVALID_LDO1_VOLT_VALUE			9
#define INVALID_LDO2_VOLT_VALUE			10
#define INVALID_LDO3_VOLT_VALUE			11
#define INVALID_LDO4_VOLT_VALUE			12
#define INVALID_LDO5_VOLT_VALUE			13
#define INVALID_LDO6_VOLT_VALUE			14
#define INVALID_LDO7_VOLT_VALUE			15
#define INVALID_LDO8_VOLT_VALUE			16
#define INVALID_LDO9_VOLT_VALUE			17
#define INVALID_LDO10_VOLT_VALUE		18
#define INVALID_PM_IOCTL			19
#define INVALID_OPERATION_ON_MERGED_BUCK	20
#define INVALID_PD_FUNCTION			21

/* Buck Config Validation Macros */
#define BUCK_CORE_PRO_VOLT_UPPER		2075
#define BUCK_CORE_PRO_VOLT_LOWER		500
#define BUCK_MEM_VOLT_UPPER			2500
#define BUCK_MEM_VOLT_LOWER			925
#define BUCK_PERI_VOLT_UPPER			3600
#define BUCK_PERI_VOLT_LOWER			1800
#define LDO1_VOLT_UPPER				1800
#define LDO1_VOLT_LOWER				600
#define LDO2_VOLT_UPPER				1800
#define LDO2_VOLT_LOWER				600
#define LDO34_VOLT_UPPER			3300
#define LDO34_VOLT_LOWER			1725
#define LDO567810_VOLT_UPPER			3600
#define LDO567810_VOLT_LOWER			1200
#define LDO9_VOLT_UPPER				3650
#define LDO9_VOLT_LOWER				1250

/* PM Debug Macro. To enable the Debug set 1 to the macr0 */
#define DA9052_PM_DEBUG 			0

#undef DA9052_DEBUG
#if DA9052_PM_DEBUG
	#define DA9052_DEBUG( fmt, args... ) printk( KERN_CRIT "" fmt, ##args )
#else
	#define DA9052_DEBUG( fmt, args... )
#endif

/* Enum definitions */
/**
 * enum buck_mode_enum -	Enum to represent the BUCK Mode
 *
 * @SLEEP_MODE: 		Buck Sleep Mode = 0
 * @AUOTMATIC_MODE: 		Buck Automatic mode = 1
 * @SYNCHRONOUS_MODE: 		Buck Synchronous Mode = 2
 * @AUTO_SYNCHRONOUS_MODE: 	BuCk Auto-Synchronous Mode = 3 
 */
enum buck_mode_enum {
	SLEEP_MODE = 0,
	AUOTMATIC_MODE,
	SYNCHRONOUS_MODE,
	AUTO_SYNCHRONOUS_MODE
};

/**
 * enum buck_cur_lim_enum -	Enum to represent the BUCK Current limit
 *
 * @CURRENT_LIMIT_700mA: 	Buck Current limit = 700mA
 * @CURRENT_LIMIT_800mA: 	Buck Current limit = 800mA
 * @CURRENT_LIMIT_1000mA: 	Buck Current limit = 1000mA
 * @CURRENT_LIMIT_1200mA: 	BuCk Current limit = 1200mA
 */
enum buck_cur_lim_enum {
	CURRENT_LIMIT_700mA = 0,
	CURRENT_LIMIT_800mA,
	CURRENT_LIMIT_1000mA,
	CURRENT_LIMIT_1200mA
};

/**
 * enum ldo_num_enum -	Enum to represent the LDO Numbeer
 *
 * @LDO_1: 	LDO Number = 1
 * @LDO_2: 	LDO Number = 2
 * ...
 * @LDO_10: 	LDO Number = 10
 */
enum 	ldo_num_enum {
	LDO_1 = 1, LDO_2, LDO_3, LDO_4, LDO_5,
	LDO_6,	LDO_7,	LDO_8, LDO_9, LDO_10 
};

/**
 * enum buck_num_enum -	Enum to represent the Buck Number
 *
 * @BUCK_CORE: 	Buck Number = 1
 * @BUCK_PRO: 	Buck Number = 2
 * @BUCK_MEM	Buck Number = 3
 * @BUCK_PERI: 	Buck Number = 4
 */
enum buck_num_enum {
	BUCK_CORE = 1,
	BUCK_PRO,
	BUCK_MEM,
	BUCK_PERI 
};

/**
 * enum da9052_pm_events_enum - Enum to represent events handled by PM driver
 *
 * @DA9052_PM_SEQ_RDY_EVENT: 	seq_rdy event = 1
 * @DA9052_PM_ONKEY_EVENT: 	nONKEY event = 2
 * @DA9052_PM_ID_FLOAT_EVENT	ID_FLOAT event = 4
 * @DA9052_PM_ID_GND_EVENT: 	ID_GND event = 8
 * @DA9052_PM_GPI8_EVENT: 	GPI8 event = 16
 * @DA9052_PM_GPI9_EVENT: 	GPI8 event = 32
 * @DA9052_PM_GPI10_EVENT: 	GPI8 event = 64
 */
enum da9052_pm_events_enum {
	DA9052_PM_SEQ_RDY_EVENT = 1 << 0,
	DA9052_PM_ONKEY_EVENT = 1 << 1,
	DA9052_PM_ID_FLOAT_EVENT = 1 << 2,
	DA9052_PM_ID_GND_EVENT = 1 << 3,
	DA9052_PM_GPI8_EVENT = 1 << 4,
	DA9052_PM_GPI9_EVENT = 1 << 5,
	DA9052_PM_GPI10_EVENT = 1 << 6,
};

/**
 * enum da9052_powerdown_function_enum -
 *		Enum to represent functions that can be enabled/disabled in
 *		power down mode
 *
 * @GPIO_PD:		GPIO extender functionality
 * @ADC_TSI_PD:		ADC/TSI measurement
 * @PM_IF_PD:		Power manager interface
 * @HS_2_WIRE_PD:	HS-2-WIRE interface
 * @BAT_CHG_PD:		Battery charging
 * @BKUP_BAT_CHG_PD:	Backup-battery charging
 * @OUT_32K_PD:		OUT_32K
 * @PM_CONT_PD:		SYS_EN, PWR_EN, PWR1_EN
 */
enum da9052_powerdown_function_enum {
	DA9052_GPIO_PD = 1,
	DA9052_ADC_TSI_PD,
	DA9052_PM_IF_PD,
	DA9052_HS_2_WIRE_PD,
	DA9052_BAT_CHG_PD,
	DA9052_BKUP_BAT_CHG_PD,
	DA9052_OUT_32K_PD,
	DA9052_PM_CONT_PD,
};

/*--------------------------------------------------------------------------*/
/* Structure Definitions                                                    */
/*--------------------------------------------------------------------------*/

/**
 * struct pm_event_registration_status - 
 *		Struct to store registration status of PM events with EH
 *
 * @da9030_event_vddlow: 		VDDLOW event
 * @da9030_event_tbat: 			TBAT Event
 * @da9030_event_chgend: 		CHGEND Event
 * @da9030_event_chdet: 		CHGDT Event
 * @da9030_event_monitoring_fail: 	Monitoring fail event
 */
typedef struct {
	u8	seq_rdy:1;
	u8	nonkey:1;
	u8	id_float:1;
	u8	id_gnd:1;
	u8	gpi8:1;
	u8	gpi9:1;
	u8	gpi10:1;
}pm_event_registration_status;

/**
 * struct da9052_buck_config - 
 *				Struct to represent configuration of a Buck
 *
 * @buck_volt: 		Buck voltage in mV
 * @buck_num: 		Buck Number. Must be one of the buck_num_enum.
 * @buck_mode: 		Buck Mode, Must be one of the buck_mode_enum.
 * @current_level: 	Buck Current Level . Must be one of buck_cur_lim_enum
 * @buck_conf: 		Buck Config mode. Must be 1/0
 * @buck_pd: 		Buck powerdown mode. Must be 1/0
 * @bperi_hs: 		Buck Peri HS Enable. Must be 1/0
 */
typedef struct {
	u16 		buck_volt;
	u8		buck_num;
	u8 		buck_mode:2;
	u8 		current_level:2;
	u8 		buck_conf:1;
	u8 		buck_pd:1;
	u8		bperi_hs:1;
}da9052_buck_config;

/**
 * struct da9052_ldo_config - 
 *				Struct to represent configuration of a LDO
 *
 * @ldo_volt: 		LDO voltage in mV
 * @ldo_num: 		LDO Number. Must be one of the buck_num_enum.
 * @ldo_conf: 		LDO Config mode. Must be 1/0
 * @ldo_pd: 		LDO powerdown mode. Must be 1/0
 */
typedef struct {
	u16 		ldo_volt;
	u8	 	ldo_num;
	u8 		ldo_conf:1;
	u8 		ldo_pd:1;
}da9052_ldo_config;

/**
 * struct da9052_ldo_buck_control - 
 *				Struct to represent BUCK/LDO Enable
 *
 * @ldo_buck_num: 	LDO/BUCK number
 * @flag: 		Enable/Disable 
 */
typedef struct {
	u8 		ldo_buck_num:4;
	u8 		flag:1;
}da9052_ldo_buck_control;

/**
 * struct da9052_ldo_buck_go - 
 *				Struct to represent BUCK/LDO GO
 *
 * @ldo_buck_num: 	LDO/BUCK number
 * @flag: 		Ramp/Hold
 */
typedef struct {
	u8 		ldo_buck_num:4;
	u8 		flag:1;
}da9052_ldo_buck_go;

/**
 * struct da9052_func_ctrl_in_powerdown - 
 *				Struct to represent function control in power down mode
 *
 * @pd_func:	Function to be enabled/disabled during power down 
 * @is_disable:	Enable/Disable control, 1:Disable, 0:Enable
 */
typedef struct {
	enum da9052_powerdown_function_enum pd_func;
	u8 is_disable;
}da9052_func_ctrl_in_powerdown;

/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/

/*
 * IOCTL calls that are permitted to the /dev/da9052_pm interface
 */

/* Set BUCK Configuration */
/* Input Parameter - da9052_buck_config */
#define DA9052_PM_IOCTL_CONFIGURE_BUCK		3

/* Set LDO Configuration */
/* Input Parameter - da9052_ldo_config */
#define DA9052_PM_IOCTL_CONFIGURE_LDO		4

/* LDo GO Configuration */
/* Input Parameter - da9052_ldo_buck_go */
#define DA9052_PM_IOCTL_GO_LDO			5

/* BUCK GO Configuration */
/* Input Parameter - da9052_ldo_buck_go */
#define DA9052_PM_IOCTL_GO_BUCK			6

/* LDO ON/OFF Configuration */
/* Input Parameter - da9052_ldo_buck_control */
#define DA9052_PM_IOCTL_SET_LDO			7

/* BUCK ON/OFF Configuration */
/* Input Parameter - da9052_ldo_buck_control */
#define DA9052_PM_IOCTL_SET_BUCK		8

/* GPI 14 and 15 Reset Enable/Disable */
/* Input Parameter - u8 flag */
#define DA9052_PM_IOCTL_SET_GPIO1415_RESET	9

/* nONKY Reset Enable/Disable */
/* Input Parameter - u8 flag */
#define DA9052_PM_IOCTL_SET_ONKEY_RESET		10

/* ACCDET Enable/Disable */
/* Input Parameter - u8 flag */
#define DA9052_PM_IOCTL_SET_ACCDET		11

/* Get and clear Fault Log */
/* Input Parameter - Buffer Pointer */
#define DA9052_PM_IOCTL_GET_FAULTLOG		12

/* ShutDown DA9052 */
#define DA9052_PM_IOCTL_ASSERT_SHUTDOWN		13

/* DeepSleep DA9052 */
#define DA9052_PM_IOCTL_ASSERT_DEEPSLEEP	14

/* VMEM_SW Connect/Disconnect */
/* Input Parameter - u8 flag */
#define DA9052_PM_IOCTL_VMEM_SW			15

/* VPERI_SW Connect/Disconnect */
/* Input Parameter - u8 flag */
#define DA9052_PM_IOCTL_VPERI_SW		16

/* LDO/BUCK LOCK/UNLOCK */
/* Input Parameter - u8 flag */
#define DA9052_PM_IOCTL_SET_LOCK		17

/* Get EH Event from Driver */
/* Input Parameter - Buffer Pointer */
#define DA9052_PM_IOCTL_GET_EVENT		18

/* Register Event to PM */
/* Input Parameter - u8 event_type */
#define DA9052_PM_IOCTL_REGISTER_EVENT		19

/* Un-Register Event to PM */
/* Input Parameter - u8 event_type */
#define DA9052_PM_IOCTL_UNREGISTER_EVENT	20

/* Merge/UnMerge BuckCore and BuckPro */
/* Input Parameter - u8 flag */
#define DA9052_PM_IOCTL_BUCKMERGE		21

/* Enable/disable different functions in POWER-DOWN mode */
/* Input Parameter - da9052_func_ctrl_in_powerdown *  */
/* Set 
	0: To Enable
	1: To Disable
   a particular function in the input parameter */
#define DA9052_PM_IOCTL_FUNC_CTRL_IN_POWERDOWN	22

/*--------------------------------------------------------------------------*/
/* Inline Functions                                                         */
/*--------------------------------------------------------------------------*/

/**
 * buckpro_mV_to_reg:	Convert BUCKRO voltage to reg
 * 
 * @param u16 value:	buckpro voltage value.
 * @return u8:		Buckpro volt bit field value
 */
static inline  u8 buckpro_mV_to_reg(u16 value)
{
	return((value - 500)/25);
}

/**
 * validate_buckpro_mV:	Validate BUCKPRO voltage value
 * 
 * @param u16 value:	BUCKPRO voltage value.
 * @return u8:		Error Status SUCCESS and FAILURE
 */
static inline  u8 validate_buckpro_mV(u16 value)
{
	/* Check the voltage range */
	if((value >= BUCK_CORE_PRO_VOLT_LOWER) &&
		(value <= BUCK_CORE_PRO_VOLT_UPPER))
		return (((value - 500)%25 > 0) ? FAILURE : SUCCESS);
	return (FAILURE);
}

/**
 * buckmem_mV_to_reg:	Convert BUCKMEM pro voltage to reg
 * 
 * @param u16 value:	buckmem voltage value.
 * @return u8:		Buckmem volt bit field value
 */
static inline  u8 buckmem_mV_to_reg(u16 value)
{
	return ((value - 925)/25);
}

/**
 * validate_buckmem_mV:	Validate BUCKMEM voltage value
 * 
 * @param u16 value:	BUCKMEM voltage value.
 * @return u8:		Error Status SUCCESS and FAILURE
 */
static inline  u8 validate_buckmem_mV(u16 value)
{
	/* Check the voltage range */
	if ((value >= BUCK_MEM_VOLT_LOWER) && (value <= BUCK_MEM_VOLT_UPPER)) 
		return (((value - 925)%25 > 0) ? FAILURE : SUCCESS);
	return (FAILURE);
}

/**
 * ldo9_mV_to_reg:	Convert LDO9 voltage to reg
 * 
 * @param u16 value:	LDO9 voltage value.
 * @return u8:		LDO9 volt bit field value
 */
static inline  u8 ldo9_mV_to_reg(u16 value)
{
	return ((value - 1250)/50);
}

/**
 * validate_ldo9_mV:	Validate LDO9 voltage value
 * 
 * @param u16 value:	LDO9 voltage value.
 * @return u8:		Error Status SUCCESS and FAILURE
 */
static inline  u8 validate_ldo9_mV(u16 value)
{
	/* Check the voltage range */
	if ((value >= LDO9_VOLT_LOWER) && (value <= LDO9_VOLT_UPPER))
		return (((value - 1250)%50 > 0) ? FAILURE : SUCCESS);
	return (FAILURE);
}

/**
 * ldo567810_mV_to_reg:	Convert LDO5/LDO6/LDO7/LDO8/LDO10 voltage to reg
 * 
 * @param u16 value:	LDO5/LDO6/LDO7/LDO8/LDO10 voltage value.
 * @return u8:		LDO5/LDO6/LDO7/LDO8/LDO10 volt bit field value
 */
static inline  u8 ldo567810_mV_to_reg(u16 value)
{
	return ((value - 1200)/50);
}

/**
 * validate_ldo567810_mV:	Validate LDO5/LDO6/LDO7/LDO8/LDO10 voltage value
 * 
 * @param u16 value:	LDO5/LDO6/LDO7/LDO8/LDO10 voltage value.
 * @return u8:		Error Status SUCCESS and FAILURE
 */
static inline  u8 validate_ldo567810_mV(u16 value)
{
	/* Check the voltage range */
	if ((value >= LDO567810_VOLT_LOWER) && (value <= LDO567810_VOLT_UPPER)) 
		return (((value - 1200)%50 > 0) ? FAILURE : SUCCESS);
	return (FAILURE);
}

/**
 * ldo34_mV_to_reg:	Convert LDO3/LDO4 voltage to reg
 * 
 * @param u16 value:	LDO3/LDO4 voltage value.
 * @return u8:		LDO3/LDO4 volt bit field value
 */
static inline  u8 ldo34_mV_to_reg(u16 value)
{
	return ((value - 1725)/25);
}

/**
 * validate_ldo34_mV:	Validate LDO3/LDO4 voltage value
 * 
 * @param u16 value:	LDO3/LDO4 voltage value.
 * @return u8:		Error Status SUCCESS and FAILURE
 */
static inline  u8 validate_ldo34_mV(u16 value)
{
	/* Check the voltage range */
	if ((value >= LDO34_VOLT_LOWER) && (value <= LDO34_VOLT_UPPER))
		return (((value - 1725)%25 > 0) ? FAILURE : SUCCESS);
	return (FAILURE);
}

/**
 * ldo1_mV_to_reg:	Convert LDO1 voltage to reg
 * 
 * @param u16 value:	LDO1 voltage value.
 * @return u8:		LDO1 volt bit field value
 */
static inline  u8 ldo1_mV_to_reg(u16 value)
{
	return ((value - 600)/50);
}

/**
 * validate_ldo1_mV:	Validate LDO1 voltage value
 * 
 * @param u16 value:	LDO1 voltage value.
 * @return u8:		Error Status SUCCESS and FAILURE
 */
static inline  u8 validate_ldo1_mV(u16 value)
{
	/* Check the voltage range */
	if ((value >= LDO1_VOLT_LOWER) && (value <= LDO1_VOLT_UPPER))
		return (((value - 600)%50 > 0) ? FAILURE : SUCCESS);
	return (FAILURE);
}

/**
 * ldo2_mV_to_reg:	Convert LDO2 voltage to reg
 * 
 * @param u16 value:	LDO2 voltage value.
 * @return u8:		LDO2 volt bit field value
 */
static inline  u8 ldo2_mV_to_reg(u16 value)
{
	return ((value - 600)/25);
}

/**
 * validate_ldo2_mV:	Validate LDO2 voltage value
 * 
 * @param u16 value:	LDO2 voltage value.
 * @return u8:		Error Status SUCCESS and FAILURE
 */
static inline  u8 validate_ldo2_mV(u16 value)
{
	/* Check the voltage range */
	if ((value >= LDO2_VOLT_LOWER) && (value <= LDO2_VOLT_UPPER))
		return (((value - 600)%25 > 0) ? FAILURE : SUCCESS);
	return (FAILURE);
}

/**
 * buckperi_mV_to_reg:	Convert BUCKPERI voltage to reg
 * 
 * @param u16 value:	BUCKPERI voltage value.
 * @return u8:		BUCKPERI volt bit field value
 */
static inline  u8 buckperi_mV_to_reg(u16 value)
{
	if (value <= 3000 ) 
		return ((value - 1800)/50);
	return ((value - 3000)/100 + 24);
}

/**
 * validate_buckperi_mV:	Validate BUCKPERI voltage value
 * 
 * @param u16 value:	BUCKPERI voltage value.
 * @return u8:		Error Status SUCCESS and FAILURE
 */
static inline  u8 validate_buckperi_mV(u16 value)
{
	/* Check the voltage range */
	if (!( (value >= BUCK_PERI_VOLT_LOWER) &&
		(value <= BUCK_PERI_VOLT_UPPER) ))
		return (FAILURE);

	if (value <= 3000)
		return (((value - 1800)%50 > 0) ? FAILURE : SUCCESS );
	return (((value - 3000)%100 > 0) ? FAILURE : SUCCESS );
}

/*--------------------------------------------------------------------------*/
/* External Functions                                                       */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Other Functions                                                          */
/*--------------------------------------------------------------------------*/

s32 da9052_pm_configure_buck(da9052_buck_config);
s32 da9052_pm_configure_ldo(da9052_ldo_config);
s32 da9052_pm_set_buckmerge(u8 );
s32 da9052_pm_go_buck(u8 , u8 );
s32 da9052_pm_set_ldo(u8 , u8 );
s32 da9052_pm_set_buck(u8 , u8 );
s32 da9052_pm_go_ldo(u8 , u8 );
s32 da9052_pm_set_gpio1415_reset_function(u8 );
s32 da9052_pm_set_onkey_reset_function(u8 );
s32 da9052_pm_set_accdet(u8 );
s32 da9052_pm_get_faultlog(u8* );
s32 da9052_pm_assert_shutdown(void);
s32 da9052_pm_assert_deepsleep(void);
s32 da9052_pm_vmem_sw(u8);
s32 da9052_pm_vperi_sw(u8 );
s32 da9052_pm_set_lock(u8 );
s32 da9052_pm_function_ctrl_in_powerdown(u8 , u8 );

#endif /* _DA9052_PM_H */
