/*
 * da9052_ssc.h: SSC (Synchronous Serial Communication) driver for DA9052
 *
 * This driver provides DA9052 specific wrappers around SPI and I2C host -
 * drivers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * 05/05/2009:	Initial version.
 *		Curent version only supports SPI and I2C communication. 
 *		Hooks to underlying host SPI driver are very specific to -
 *		SMDK-6410 host driver and some what non standard.
 *
 * 27/06/2009:	Added caching feature for DA9052 Non-volatile registers
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef _DA9052_SSC_H
#define _DA9052_SSC_H

/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Type Definitions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Constant Definitions                                                     */
/*--------------------------------------------------------------------------*/

/* SSC device node name in /dev directory */
#define DA9052_SSC_DEVICE_NAME		"da9052_ssc"
#define DA9052_SSC_DEVICE_MAJOR_NO	243

/* Set one of the following according to your hardware interface with DA9052 */
#define	DA9052_SSC_CONFIG_SPI		1	
#define	DA9052_SSC_CONFIG_I2C		0

/* Supported IOCTLs */
#define DA9052_IOCTL_SSC_READ		0
#define DA9052_IOCTL_SSC_WRITE		1
#define DA9052_IOCTL_SSC_TEST		2
#define DA9052_IOCTL_DUMP_SPI_REG	3

/* Module specific error codes */
#define INVALID_REGISTER		2
#define INVALID_READ			3
#define INVALID_PAGE			4

/* Defines for Volatile and Non Volatile register types */
#define VOLATILE			0
#define NON_VOLATILE			1

/* Defines for cache state */
#define VALID				0
#define INVALID				1

/* Total number of registers in DA9057 */
#define DA9052_REG_CNT			DA9052_PAGE1_REG_END

/* Maximum number of registers that can be read/written by a singe request */
#define	MAX_READ_WRITE_CNT		16

/* To enable debug output for your module, set this to 1 */
#define		DA9052_SSC_DEBUG	0

#undef DA9052_DEBUG
#if DA9052_SSC_DEBUG
#define DA9052_DEBUG( fmt, args... ) printk( KERN_CRIT "" fmt, ##args )
#else
#define DA9052_DEBUG( fmt, args... )
#endif

/*--------------------------------------------------------------------------*/
/* SPI Specific defines */
/*--------------------------------------------------------------------------*/

/* SPI device name as defined in board info */
#define DA9052_SSC_SPI_DEVICE_NAME		"da9052_ssc_spi"

/* Address of first and last registers on PAGE-0 */
#define PAGE_0_START		1
#define PAGE_0_END		127

/* Address of first and last registers on PAGE-1 */
#define PAGE_1_START		128
#define PAGE_1_END		255

#define ACTIVE_PAGE_0		0
#define ACTIVE_PAGE_1		1

#define PAGECON_0		0
#define PAGECON_128		128

#define RW_POL			1

/*--------------------------------------------------------------------------*/
/* I2C Specific defines */
/*--------------------------------------------------------------------------*/

/* I2C device name as defined in board info */
#define DA9052_SSC_I2C_DEVICE_NAME		"da9052_ssc_i2c"

/* Device slave address */
#define	DA9052_I2C_ADDR				0x92

/* I2C write modes */
#define	DA9052_SSC_I2C_PAGE_WRITE_MODE		0
#define DA9052_SSC_I2C_REPEAT_WRITE_MODE 	1

/* Default write mode */
#define DA9052_SSC_I2C_WRITE_MODE  		DA9052_SSC_I2C_REPEAT_WRITE_MODE




/*--------------------------------------------------------------------------*/
/* Structure Definitions                                                    */
/*--------------------------------------------------------------------------*/
 /**
 * struct da9052_ssc_cache_entry - Structure to hold parameters for a single - 
 * entry of ssc register cache
 * @type: Type of register i.e. Volatile or Non-volatile
 * @status: Status of current cached value - Valid/Invalid
 * @val: Value of register
 */
 typedef struct {
	
	 u8	type:4;
	 u8	status:4;
	 u8	val;
 }da9052_ssc_cache_entry;

 /**
  * struct da9052_ssc_i2c_data - Structure to hold I2C specific data
  * @ adapter: I2C Adapter for this device
  * @ da9052_slave_addr: DA9052 I2C device address
  */
 typedef struct {
	 
	 struct	i2c_adapter 	*adapter;
	 u8			da9052_slave_addr;
 }da9052_ssc_i2c_data;
 
/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Inline Functions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* External Functions                                                       */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Other Functions                                                          */
/*--------------------------------------------------------------------------*/
 /* Generic Functions */
s32 da9052_ssc_write(da9052_ssc_msg *sscmsg);
s32 da9052_ssc_read(da9052_ssc_msg *sscmsg);

s32 da9052_ssc_write_many(da9052_ssc_msg *sscmsg, s32 cnt);
s32 da9052_ssc_read_many(da9052_ssc_msg *sscmsg, s32 cnt);

#if DA9052_SSC_CONFIG_SPI

/* SPI specific Functions */
static s32 da9052_ssc_spi_read(da9052_ssc_msg *msg);
static s32 da9052_ssc_spi_write(da9052_ssc_msg *msg);

#elif DA9052_SSC_CONFIG_I2C

/* I2C specific Functions */
static s32 da9052_ssc_i2c_read(da9052_ssc_msg *msg);
static s32 da9052_ssc_i2c_write(da9052_ssc_msg *msg);

static s32 da9052_ssc_i2c_write_many(da9052_ssc_msg *sscmsg, s32 msg_no);
static s32 da9052_ssc_i2c_read_many(da9052_ssc_msg *sscmsg, s32 msg_no);

#endif	/* DA9052_SSC_CONFIG_SPI */

#endif /* _DA9052_SSC_H */
