/*
 * da9052_tsi_calibrate.h: tsi driver file for DA9052
 *
 * This file declares global functions and defines macros 
 * related to calibrating DA9052 TSI driver.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (11/06/2009):	This file declares global functions and defines macros 
 *				related to calibrating DA9052 TSI driver.
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef DA9052_TSI_CALIBRATE_H
#define DA9052_TSI_CALIBRATE_H

/**
 * Calibration -
 *
 *  The following set of matrix equations represent a valid display 
 *  point given a corresponding set of touch screen points:
 *
 *
 *              |      |     |              |   |   Xs  |
 *              |  Xd  |     | A    B    C  |   |       |
 *              |      |  =  |              | * |   Ys  |
 *              |  Yd  |     | D    E    F  |   |       |
 *              |      |     |              |   |   1   |
 *
 * 
 *    where:
 *
 *           (Xd,Yd) represents the desired display point coordinates
 *                   on LCD /TFT screen,
 *
 *           (Xs,Ys) represents the available touch screen coordinates,
 *                   and the matrix
 *
 *           |A,B,C|
 *           |D,E,F| represents the transformation factors used to translate
 *                   available touch screen point values into the 
 *                   corresponding display coordinates.
 *
 *
 *    Note that for practical considerations, the utilitities 
 *     within this file do not use the matrix coefficients as
 *     defined above, but instead use the following 
 *     equivalents, since floating point math is not used:
 *
 *            A = An/Divider 
 *            B = Bn/Divider 
 *            C = Cn/Divider 
 *            D = Dn/Divider 
 *            E = En/Divider 
 *            F = Fn/Divider 
 *
 *
 *  Note: 
 *  digitizer resolution should not exceed 10 bits (1024 values).  
 *  Higher resolutions may cause the integer operations to overflow
 *  and return incorrect values. 
 *
 */

/*--------------------------------------------------------------------------*/
/* Global Typedefs                                                          */
/*--------------------------------------------------------------------------*/
/**
 * \struct Matrix - Structure description
 * \var An:		A = An/Divider 
 * \var Bn:		B = Bn/Divider
 * \var Cn:		C = Cn/Divider
 * \var Dn:		D = Dn/Divider
 * \var En:		E = En/Divider
 * \var Fn:		F = Fn/Divider
 * \var Divider:	common divider
 */
typedef struct Matrix {
	s32    An;
 	s32    Bn;
 	s32    Cn;
 	s32    Dn;
 	s32    En;
 	s32    Fn;
 	s32    Divider;
} Calib_xform_matrix_t ;

 /**
 * \struct calib_cfg_t - 
 * \var u8 calibrate_flag:	flag to indicate if calibration is to be used.
 */
 typedef struct {
	u8 calibrate_flag;
}calib_cfg_t;

/*--------------------------------------------------------------------------*/
/* Global Functions                                                         */
/*--------------------------------------------------------------------------*/
ssize_t da9052_tsi_get_calib_display_point(da9052_tsi_data* displayPtr);
				

ssize_t da9052_tsi_set_calib_matrix(da9052_tsi_data *displayPtr,
					da9052_tsi_data *screenPtr);


u8 configure_tsi_calib(calib_cfg_t *tsi_calib);

calib_cfg_t* get_calib_config(void);
#endif	//#ifndef DA9052_TSI_CALIBRATE_H
