/*
 * da9052_adc.c: ADC driver for DA9052
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (05/08/2009):	Initial version
 * (05/22/2009):	Changes as per code review done
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/platform_device.h>

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_reg.h"
#include "../include/da9052_lib.h"
#include "../include/da9052_adc.h"
#include "../include/da9052_eh.h"

/*--------------------------------------------------------------------------*/
/* Local Type Definitions                                                   */
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*/
/* Local Constant Definitions                                               */
/*--------------------------------------------------------------------------*/
static u8 banner[] __initdata = "DA9052 ADC Driver, v1.0\n";


/*--------------------------------------------------------------------------*/
/* Local Macro Definitions                                                  */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Local Structure Definitions                                              */
/*--------------------------------------------------------------------------*/
 
/**
 * struct adc_mode_status_t - Stores the status of mode selected for the ADC 
 *			    channel.
 * @adc_vddout	: Mode status for channel 0 VDDOUT.
 * @adc_ich	: Mode status for channel 1 ICH.
 * @adc_tbat	: Mode status for channel 2 TBAT.
 * @adc_vbat	: Mode status for channel 3 VBAT.
 * @adc_in4	: Mode status for channel 4 ADC_IN4.
 * @adc_in5	: Mode status for channel 5 ADC_IN5.
 * @adc_in6	: Mode status for channel 6 ADC_IN6.
 * @adc_tsi	: Mode status for channel 7 XY TSI
 * @adc_tsense	: Mode status for channel 8 Internal Tjunc.
 * @adc_vbbat	: Mode status for channel 9 VBBAT.
 */
 
typedef struct {
	u32 adc_vddout	:1;
	u32 adc_ich	:1;
	u32 adc_tbat	:1;
	u32 adc_vbat	:1;
	u32 adc_in4	:1;
	u32 adc_in5	:1;
	u32 adc_in6	:1;
	u32 adc_tsi	:1;
	u32 adc_tsense	:1;
	u32 adc_vbbat	:1;
	u32 dummy	:22;
} adc_mode_status_t;

/**
 * struct eh_registration_status_t - Interrupt registration status with EH
 * @adc4_event		: States whether ADC4 interrupt is registered with EH
 * @adc5_event		: States whether ADC5 interrupt is registered with EH
 * @adc6_event		: States whether ADC6 interrupt is registered with EH
 * @comp_1v2_event	: States whether COMP1V2 interrupt is registered with EH
 *
 */
typedef struct {
	u8 adc4_event		:1;
	u8 adc5_event		:1;
	u8 adc6_event		:1;
	u8 comp_1v2_event	:1;
	u8 dummy		:4; 
} eh_registration_status_t;

/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/
/* Stores the state of ADC driver */
static u8 adc_device_open = 0;
/* Major number for ADC driver */
static s32 adc_major_number = DA9052_ADC_DEVICE_MAJOR_NO;
/* Mutex for ADC manual conversion */
struct mutex da9052_adc_manconv_lock;
/* Structure to store the mode status */
static adc_mode_status_t adc_mode_status;
static struct platform_device *da9052_adc_platform_device;
static struct fasync_struct *adc_fasync_queue;

/* States whether a particular event is registered with EH */
static eh_registration_status_t is_eh_registered;

/* EH structures for all possible events */
da9052_eh_nb adc4_eh_data;
da9052_eh_nb adc5_eh_data;
da9052_eh_nb adc6_eh_data;
da9052_eh_nb comp_1v2_eh_data;

/* Storing the event status for multiple event handling */
static adc_event_occ_status_t adc_event_occ_status;

/* Mutex erquired for protecting the access to the event occurance status */
struct mutex adc_event_occ_lock;

/*--------------------------------------------------------------------------*/
/* Local Functions                                                          */
/*--------------------------------------------------------------------------*/

/**
 * da9052_manual_read	:
 *			This function is local to ADC module and it reads the
 * 			data from ADC manual conversion registers. Manual 
 *			conversion register is also accessed by TSI module, so
 *			mutex is used to make sure that no unathorised access 
 *			to this register is made.
 * 
 * @param channel 	- This parameter gives the channel number to be read
 * @param *data		- This is pointer to location where data is to written
 *
 * @return status:  
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */  
static s32 da9052_manual_read(u8 channel, u16 *data)
{
	u8 man_timeout_cnt = MAX_MANCONV_RETRY_COUNT;
	da9052_ssc_msg msg;
	u16 calc_data;
	/* lock mutex */
	mutex_lock(&da9052_adc_manconv_lock);
	
	msg.addr = DA9052_ADCMAN_REG;
	msg.data = channel;
	msg.data = set_bits(msg.data, DA9052_ADCMAN_MANCONV);
	/* Write data to start manual conversion */
	if (SUCCESS != da9052_ssc_write(&msg)){
		mutex_unlock(&da9052_adc_manconv_lock);
		return (ADC_SSC_BUS_ERROR);
	}

	/* Wait for the event */
	do {
		/* Read Sampling interval timer */
		msg.addr = DA9052_ADCCONT_REG;
		if (SUCCESS != da9052_ssc_read(&msg)){
			mutex_unlock(&da9052_adc_manconv_lock);
			return (ADC_SSC_BUS_ERROR);
		}
		
		if (DA9052_ADCCONT_ADCMODE & msg.data) {
			/* ADC sampling interval time is 1msec */
			msleep (1);
		} else {
			/* ADC sampling interval time is 10msec */
			msleep (10);
		}

		msg.addr = DA9052_ADCMAN_REG;
		/* read data to start manual conversion */
		if (SUCCESS != da9052_ssc_read(&msg)){
			mutex_unlock(&da9052_adc_manconv_lock);
			return (ADC_SSC_BUS_ERROR);
		}
		
		/* Counter to avoid endless while loop */
		man_timeout_cnt --;
		if(man_timeout_cnt == 1){
			if (!(msg.data & DA9052_ADCMAN_MANCONV)) {
				break;
			} else {
				mutex_unlock(&da9052_adc_manconv_lock);
				return (ADC_MANCONV_TIMEOUT_ERROR);
			}
		}
	/* Wait until the MAN_CONV bit is cleared to zero */
	} while(msg.data & DA9052_ADCMAN_MANCONV);
	
	msg.addr = DA9052_ADCRESH_REG; /* Read MSB data */
	if (SUCCESS != da9052_ssc_read(&msg)){
		mutex_unlock(&da9052_adc_manconv_lock);
		return (ADC_SSC_BUS_ERROR);
	}
	calc_data = (u16)msg.data;
	*data = (calc_data << 2);

	DA9052_DEBUG("ADCRESH_REG data = %x\n", *data);
	DA9052_DEBUG("ADCRESH_REG msg.data = %x\n", msg.data);
	
	msg.addr = DA9052_ADCRESL_REG; /* Read LSB data */
	if (SUCCESS != da9052_ssc_read(&msg)){
		mutex_unlock(&da9052_adc_manconv_lock);
		return (ADC_SSC_BUS_ERROR);
	}
	/* Clear first 14 bits before ORing */
	calc_data = (u16)msg.data & 0x0003;
	*data |= calc_data;

	DA9052_DEBUG("ADCRESL_REG data = %x\n", *data);
	DA9052_DEBUG("ADCRESL_REG msg.data = %x\n", msg.data);
	/* unlock mutex */
	mutex_unlock(&da9052_adc_manconv_lock);
	return (SUCCESS);
}

/**
 * da9052_adc_hw_init: This function initializes the hardware for ADC module/
 * 
 * @param void
 * @return 
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */
static s32 da9052_adc_hw_init(void)
{
	da9052_ssc_msg msg;

	/* Initialize the ADC pins */
	/* ADC channel 4 and 5 are by default enabled */
#if (ADC_CONF_ADC4)
	/* For channel 4 */
	msg.addr = DA9052_GPIO0001_REG;
	/* Read previous contents of the GPIO_0-1 register */
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Set the Pin functionality */
	msg.data = clear_bits(msg.data, DA9052_GPIO0001_GPIO0PIN);
	/* Write back the GPIO_0-1 register data */
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);
#endif /* ADC_CONF_ADC4 */
#if (ADC_CONF_ADC5)
	/* For channel 5 */
	msg.addr = DA9052_GPIO0001_REG;
	/* Read previous contents of the GPIO_0-1 register */
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Set the Pin functionality */
	msg.data = clear_bits(msg.data, DA9052_GPIO0001_GPIO1PIN);
	/* Write back the GPIO_0-1 register data */
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);
#endif /* ADC_CONF_ADC5 */
#if (ADC_CONF_ADC6)
	/* For channel 6 */
	msg.addr = DA9052_GPIO0203_REG;
	/* Read previous contents of the GPIO_0-1 register */
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Set the Pin functionality */
	msg.data = clear_bits(msg.data, DA9052_GPIO0203_GPIO2PIN);
	/* Write back the GPIO_0-1 register data */
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);
#endif /* ADC_CONF_ADC6 */

	/* By default configure the Measurement sequence interval to 1ms */
	msg.addr = DA9052_ADCCONT_REG; 
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Set the ADC MODE bit for 1msec sampling timer */
	msg.data = set_bits(msg.data, DA9052_ADCCONT_ADCMODE);
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);

	/* By default configure ADC channel to auto mode */
	adc_mode_status.adc_vddout 	= AUTO_MODE;
	adc_mode_status.adc_ich 	= AUTO_MODE;
	adc_mode_status.adc_tbat 	= AUTO_MODE; 
	adc_mode_status.adc_vbat 	= MANUAL_MODE;
	adc_mode_status.adc_in4 	= AUTO_MODE;
	adc_mode_status.adc_in5 	= AUTO_MODE;
	adc_mode_status.adc_in6 	= AUTO_MODE;
	adc_mode_status.adc_tsense 	= AUTO_MODE;
	adc_mode_status.adc_vbbat 	= MANUAL_MODE;

	/* Initialize mutex required for ADC Manual read */
	mutex_init(&da9052_adc_manconv_lock);

	/* Initialize mutex for ADC event occurance status */
	mutex_init(&adc_event_occ_lock);
	
	/* Init the status variable for event occurance */
	adc_event_occ_status.adc_event_status = FALSE;

	/* Set EH registration flag to 0 */
	is_eh_registered.adc4_event 	= FALSE;
	is_eh_registered.adc5_event 	= FALSE;
	is_eh_registered.adc6_event 	= FALSE;
	is_eh_registered.comp_1v2_event = FALSE;

	DA9052_DEBUG("Finished hardware initialization\n");
	return (SUCCESS);
}

/**
 * da9052_adc_hw_deinit: This function deinitializes the hardware.
 * 
 * @param void
 * @return s32
 *	SUCCESS	- On successful completion of the call.
 */
static s32 da9052_adc_hw_deinit(void)
{
	/* Before exiting - unregister events with EH */
	if (is_eh_registered.adc4_event)
		da9052_eh_unregister_nb(&adc4_eh_data);
	if (is_eh_registered.adc5_event)
		da9052_eh_unregister_nb(&adc5_eh_data);
	if (is_eh_registered.adc6_event)
		da9052_eh_unregister_nb(&adc6_eh_data);
	if (is_eh_registered.comp_1v2_event)
		da9052_eh_unregister_nb(&comp_1v2_eh_data);

	/* Delete Mutex */
	mutex_destroy(&da9052_adc_manconv_lock);

	/* Delete mutex for ADC event occurance status */
	mutex_destroy(&adc_event_occ_lock);
	
	DA9052_DEBUG("Finished hardware de-initialization\n");
	return (SUCCESS);
}

/**
 * da9052_adc_signal_to_user; 
 * This function signal to the library to call the call back function.
 * 
 * @param event
 * @return s32
 */
static s32 da9052_adc_signal_to_user(u8 event)
{
	DA9052_DEBUG("%s: event status: %d \n", __FUNCTION__, event);
	kill_fasync(&adc_fasync_queue, SIGIO, POLL_IN);

	return(SUCCESS);
}

/*--------------------------------------------------------------------------*/
/* Global Functions                                                         */
/*--------------------------------------------------------------------------*/
/**
 * da9052_set_adc_mode:	This function sets the mode for which the ADC channel
 *			is to be read. all channels can be configured in both 
 *			modes. Except for the following conditions
 *			1. TSI channel cannot be configured in any mode.
 *			2. VBAT and VBBAT cannot be configured in AUTO mode.
 * 
 * @param channel 	- This parameter selects the channel to be read.
 * @param mode 		- This parameter selects the mode for the ADC channel
 *			  selected.
 * @return status:
 *	ADC_INVALID_CHANNEL_ERROR - If the selected channel is out of range 
 *				    A0-A10
 *	MODE 	- If the selected mode is other then AUTO or MANUAL
 *	SUCCESS	- On successful completion of the call.
 */
s32 da9052_set_adc_mode(u8 channel, u8 mode)
{
	/* TSI channel cannot be read by ADC */
	if (channel == ADC_TSI)
		return (ADC_INVALID_CHANNEL_ERROR);

	/* Check for Illegal mode set */
	if ((mode != AUTO_MODE) && (mode != MANUAL_MODE))
		return (ADC_INVALID_MODE_ERROR);
	/* Check for Illegal channel set */
	if((mode == AUTO_MODE) && ((channel == ADC_VBAT) || 
		(channel == ADC_VBBAT)))
		return (ADC_INVALID_CHANNEL_ERROR);

	switch (channel) {
	case ADC_VDDOUT:
		adc_mode_status.adc_vddout = mode;
		break;
	case ADC_ICH:
		adc_mode_status.adc_ich = mode;
		break;
	case ADC_TBAT:
		adc_mode_status.adc_tbat = mode;
		break;
	case ADC_VBAT:
		adc_mode_status.adc_vbat = mode;
		break;
	case ADC_ADCIN4:
		adc_mode_status.adc_in4 = mode;
		break;
	case ADC_ADCIN5:
		adc_mode_status.adc_in5 = mode;
		break;
	case ADC_ADCIN6:
		adc_mode_status.adc_in6 = mode;
		break;
	case ADC_TSI:
		adc_mode_status.adc_tsi = mode;
		break;
	case ADC_TJUNC:
		adc_mode_status.adc_tsense = mode;
		break;
	case ADC_VBBAT:
		adc_mode_status.adc_vbbat = mode;
		break;
	default :
		/* Illegal Channel set */
		return (ADC_INVALID_CHANNEL_ERROR);
	}

	DA9052_DEBUG("vddout 	= %d\n",adc_mode_status.adc_vddout);
	DA9052_DEBUG("adc_ich 	= %d\n",adc_mode_status.adc_ich);
	DA9052_DEBUG("adc_tbat 	= %d\n",adc_mode_status.adc_tbat);
	DA9052_DEBUG("adc_vbat 	= %d\n",adc_mode_status.adc_vbat);
	DA9052_DEBUG("adc_in4 	= %d\n",adc_mode_status.adc_in4);
	DA9052_DEBUG("adc_in5 	= %d\n",adc_mode_status.adc_in5);
	DA9052_DEBUG("adc_in6 	= %d\n",adc_mode_status.adc_in6);
	DA9052_DEBUG("adc_tsi 	= %d\n",adc_mode_status.adc_tsi);
	DA9052_DEBUG("adc_tsense= %d\n",adc_mode_status.adc_tsense);
	DA9052_DEBUG("adc_vbbat = %d\n",adc_mode_status.adc_vbbat);
	
	return (SUCCESS);
}

/**
 * da9052_adc_set_sampling_interval	:
 *			This function sets the measurement sequence interval. 
 *			Only two measurement sequence interval setting is 
 *			possible 10 ms and 1 ms.
 * 
 * @param interval 	- This parameter selects the sampling speed.
 *
 * @return status:  
 *	ADC_INVALID_TIME_ERROR - If time is set to some other value other than 
 * 				1ms or 10ms.
 *	ADC_SSC_BUS_ERROR	- If there is an error on the SSC bus.
 *	SUCCESS			- On successful completion of the call.
 */ 
s32 da9052_adc_set_sampling_interval(u8 interval)
{
	da9052_ssc_msg msg;
	
	/* Read existing data in the register */
	msg.addr = DA9052_ADCCONT_REG;
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Modify the data to be written */
	if (interval == ADC_INTERVAL_10MS) {
		msg.data = clear_bits(msg.data, DA9052_ADCCONT_ADCMODE);
	} else if (interval == ADC_INTERVAL_1MS) {
		msg.data = set_bits(msg.data, DA9052_ADCCONT_ADCMODE);
	} else {
		return (ADC_INVALID_TIME_ERROR);
	}
	/* Write the data */
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);
	return (SUCCESS);
}

/**
 * da9052_adc_read_vddout	:
 *			This function reads the value from ADC
 *			channel 0. If the vddout channel is disabled, this
 *			function enables it and read the value and again
 * 			disables the channel. If vddout channel is already
 *			enabled this function directly reads the value from
 *			the ADC register.
 * 
 * @param *data 	- This parameter is pointer to u16 where data can be
 *			  written.
 *
 * @return status:
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */
s32 da9052_adc_read_vddout(u16 *data)
{
	da9052_ssc_msg msg;

	/* Check ADC (VDDOUT) status enabled/disabled */
	msg.addr = DA9052_ADCCONT_REG;
	if(SUCCESS != da9052_ssc_read(&msg))
		return(ADC_SSC_BUS_ERROR);
	
	if (!(msg.data & DA9052_ADCCONT_AUTOVDDEN)) {
		/* If disabled, enable it and then read the value */
		msg.data = set_bits(msg.data, DA9052_ADCCONT_AUTOVDDEN);
		if (SUCCESS != da9052_ssc_write(&msg))
			return (ADC_SSC_BUS_ERROR);

		/* Wait for the conversion time period */
		/* Read sampling interval set in the ADC_CONT register */
		if (SUCCESS != da9052_ssc_read(&msg))
			return (ADC_SSC_BUS_ERROR);

		if (msg.data & DA9052_ADCCONT_ADCMODE) {
			/* If sampling interval is 1ms */
			msleep (1);
		} else {
			/* If sampling interval is 10ms */
			msleep (10);
		}
		
		/* Read the ADC converted value */
		msg.addr = DA9052_VDDRES_REG;
		if (SUCCESS != da9052_ssc_read(&msg))
			return (ADC_SSC_BUS_ERROR);

		*data = (u16)msg.data;
		DA9052_DEBUG(
	       "In function: %s, VDDCONT_RES value read (1)= 0x%X \n", 
							__FUNCTION__, msg.data);
		/* Disable the VDDOUT ADC again */
		msg.addr = DA9052_ADCCONT_REG;
		if (SUCCESS != da9052_ssc_read(&msg))
			return (ADC_SSC_BUS_ERROR);

		msg.data = clear_bits(msg.data, DA9052_ADCCONT_AUTOVDDEN);
		if (SUCCESS != da9052_ssc_write(&msg))
			return (ADC_SSC_BUS_ERROR);
	} else {
		/* Read the ADC converted value */
		msg.addr = DA9052_VDDRES_REG;
		if (SUCCESS != da9052_ssc_read(&msg))
			return (ADC_SSC_BUS_ERROR);

		*data = (u16)msg.data;
		DA9052_DEBUG(
	       "In function: %s, VDDCONT_RES value read (1)= 0x%X \n", 
							__FUNCTION__, msg.data);
	}
	return (SUCCESS);
}

/**
 * da9052_adc_read_ich	:
 *			This function reads the value from ADC 
 *			channel 1. There is no need to call the channel mode,
 *			start and read functions. 
 * 
 * @param data 	- This parameter is pointer to u16 where data can be
 *			  written.
 *
 * @return status:  
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */ 
s32 da9052_adc_read_ich(u16 *data)
{
	da9052_ssc_msg msg;
	/* Read charging conversion register */
	msg.addr = DA9052_ICHGAV_REG;
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);

	*data = (u16)msg.data;
	DA9052_DEBUG(
       "In function: %s, ICHGAV_REG value read (1)= 0x%X \n", 
							__FUNCTION__, msg.data);
	return (SUCCESS);
}

/**
 * da9052_adc_read_tbat	:
 *			This function reads the value from ADC 
 *			channel 2. There is no need to call the channel mode,
 *			start and read functions. 
 * 
 * @param data 	- This parameter is pointer to u9 where data can be 
 *			  written.
 *
 * @return status:  
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */ 
s32 da9052_adc_read_tbat(u16 *data)
{
	da9052_ssc_msg msg;
	/* Read TBAT conversion result */
	msg.addr = DA9052_TBATRES_REG;
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);

	*data = (u16)msg.data;
	DA9052_DEBUG(
       "In function: %s, TBATRES_REG value read (1)= 0x%X \n", 
							__FUNCTION__, msg.data);
	return (SUCCESS);
}

/**
 * da9052_adc_read_vbat	:
 *			This function reads the value from ADC 
 *			channel 3. There is no need to call the channel mode,
 *			start and read functions. This function reads the 
 *			value by manual conversion (hardware limitation). 
 * 
 * @param *data		- This is pointer to location where data is to written
 *
 * @return status:  
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */ 
s32 da9052_adc_read_vbat(u16 *data)
{
	s32 error;
	u16 temp;

	error = da9052_manual_read(ADC_VBAT, &temp);
	DA9052_DEBUG(
       "In function: %s, VBAT value read (1)= 0x%X \n", 
							__FUNCTION__, temp);
	if(SUCCESS == error){
		*data = temp;
		return (SUCCESS);
	} else {
		return(error);
	}
}

/**
 * da9052_adc_read_tjunc	:
 *			This function reads the value from ADC 
 *			channel 8. There is no need to call the channel mode,
 *			start and read functions.
 *
 * @param *data		- This is pointer to location where data is to written
 *
 * @return status:  
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */ 
s32 da9052_adc_read_tjunc(u16 *data)
{
	da9052_ssc_msg msg;
	u16 temp;
	/* Read TJunction conversion result */
	msg.addr = DA9052_TJUNCRES_REG;
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	temp = msg.data;
	/* Read calibration for junction temperature */
	msg.addr = DA9052_TOFFSET_REG;
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Calculate Junction temperature */
	*data = temp - (u16)msg.data;
	DA9052_DEBUG(
       "In function: %s, Calc JUNC TEMP value (1)= 0x%X \n", 
							__FUNCTION__, *data);
	return(SUCCESS);
}


/**
 * da9052_adc_read_vbbat	:
 *			This function reads the value from ADC 
 *			channel 9. There is no need to call the channel mode,
 *			start and read functions. This function reads the 
 *			value by manual conversion (hardware limitation). 
 * 
 * @param *data		- This is pointer to location where data is to written
 *
 * @return status:
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */
s32 da9052_adc_read_vbbat(u16 *data)
{
	s32 error;
	u16 temp;

	error = da9052_manual_read(ADC_VBBAT, &temp);
	DA9052_DEBUG(
       "In function: %s, VBBAT value read (1)= 0x%X \n", 
							__FUNCTION__, temp);
	if(SUCCESS == error){
		*data = temp;
		return (SUCCESS);
	} else {
		return(error);
	}
}

/**
 * da9052_configure_thresholds	:
 *			This function sets the threshold value of the ADCIN4, 
 *			ADCIN5 and ADCIN6 channels. If value for any of the 
 *			channel is wrong this function will return error 
 *			without writing any value in the registers.
 * 
 * @param threshold_res thresholds (structure containing threshold settings) 
 *			  
 *
 * @return status:  
 *	ADC_INVALID_VALUE - Error in the high and low value.
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */  
s32 da9052_configure_thresholds(threshold_res thresholds)
{
	da9052_ssc_msg msg;

	DA9052_DEBUG("In function: %s \n", __FUNCTION__);
	DA9052_DEBUG("auto4_high	: %d \n", thresholds.auto4_high);
	DA9052_DEBUG("auto4_low		: %d \n", thresholds.auto4_low);
	DA9052_DEBUG("auto5_high	: %d \n", thresholds.auto5_high);
	DA9052_DEBUG("auto5_low		: %d \n", thresholds.auto5_low);
	DA9052_DEBUG("auto6_high	: %d \n", thresholds.auto6_high);
	DA9052_DEBUG("auto6_low		: %d \n", thresholds.auto6_low);

	/* Check if the High value is higher then Low value */
	if (thresholds.auto4_high <= thresholds.auto4_low)
		return (ADC_INVALID_VALUE);
	if (thresholds.auto5_high <= thresholds.auto5_low)
		return (ADC_INVALID_VALUE);
	if (thresholds.auto6_high <= thresholds.auto6_low)
		return (ADC_INVALID_VALUE);

	/* Write threshold value to the register */
	/* ADCIN4 */
	msg.addr = DA9052_AUTO4HIGH_REG;
	msg.data = thresholds.auto4_high;
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);

	msg.addr = DA9052_AUTO4LOW_REG;
	msg.data = thresholds.auto4_low;
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);
	
	/* ADCIN5 */
	msg.addr = DA9052_AUTO5HIGH_REG;
	msg.data = thresholds.auto5_high;
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);

	msg.addr = DA9052_AUTO5LOW_REG;
	msg.data = thresholds.auto5_low;
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);
		
	/* ADCIN6 */
	msg.addr = DA9052_AUTO6HIGH_REG;
	msg.data = thresholds.auto6_high;
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);

	msg.addr = DA9052_AUTO6LOW_REG;
	msg.data = thresholds.auto6_low;
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);

	return (SUCCESS);
}

/**
 * da9052_set_adcin5_comparator	:
 *			This function enables/disables the comparator for
 *			ADCIN5.
 * 
 * @param action 	-Enable or Disable action to be performed.
 *
 * @return status:  
 *	ADC_INVALID_VALUE - If the parameter passed is wrong.
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */  
s32 da9052_set_adcin5_comparator(u8 action)
{
	da9052_ssc_msg msg;
	
	msg.addr = DA9052_ADCCONT_REG;
	msg.data = 0;
	/* Read previous contents of the register */
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Modify the bit */
	if (action == ENABLE) {
		msg.data = set_bits(msg.data,DA9052_ADCCONT_COMP1V2EN);
	} else if (action == DISABLE) {
		msg.data = clear_bits(msg.data,DA9052_ADCCONT_COMP1V2EN);
	} else {
		return (ADC_INVALID_VALUE);
	}
	/* Write back the value to the register */
	if (SUCCESS != da9052_ssc_write(&msg))
		return(ADC_SSC_BUS_ERROR);

	return (SUCCESS);
}

/**
 * da9052_set_tbat_source_current	:
 *			This function enables the 50uA current source for 
 *			TBAT permanently or only for two slots of auto mode 
 *			ADC.
 * 
 * @param action 	- Two slots OR premanently.
 *
 * @return status:  
 *	ADC_INVALID_VALUE - If the parameter passed is wrong.
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */
s32 da9052_set_tbat_source_current(u8 action)
{
	da9052_ssc_msg msg;
	
	msg.addr = DA9052_ADCCONT_REG;
	msg.data = 0;
	/* Read previous contents of the register */
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Modify the bit value */
	if (action == TBAT_50UA_PREMANENTLY) {
		msg.data = set_bits(msg.data, DA9052_ADCCONT_TBATISRCEN);
	} else if (action == TBAT_50UA_ONE_SLOT) {
		msg.data = clear_bits(msg.data, DA9052_ADCCONT_TBATISRCEN);
	} else {
		return (ADC_INVALID_VALUE);
	}
	/* Write back to the register */
	if (SUCCESS != da9052_ssc_write(&msg))
		return(ADC_SSC_BUS_ERROR);
	return (SUCCESS);
}

/**
 * da9052_set_ad4_isrc_source_current	:
 *			This function enables the 15uA current source for
 *			ADCIN4.
 * 
 * @param action 	- Enable or Disable.
 *
 * @return status:  
 *	ADC_INVALID_VALUE - If the parameter passed is wrong.
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */
s32 da9052_set_ad4_isrc_source_current(u8 action)
{
	da9052_ssc_msg msg;
	
	msg.addr = DA9052_ADCCONT_REG;
	msg.data = 0;
	/* Read previous contents of the register */
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);
	/* Modify the bit value */
	if (action == ENABLE) {
		msg.data = set_bits(msg.data, DA9052_ADCCONT_AD4ISRCEN);
	} else if (action == DISABLE) {
		msg.data = clear_bits(msg.data, DA9052_ADCCONT_AD4ISRCEN);
	} else {
		return (ADC_INVALID_VALUE);
	}
	/* Write back to the register */
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);
	return (SUCCESS);
}
 /**
 * da9052_start_adc	:
 *			This function starts the ADC channel in AUTO mode.
 * 
 * @param channel 	- Channel No. to be started.
 *
 * @return status:  
 *	CHANNEL	- If the channel value passed to this functions is 
 *			  wrong.
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */  
s32 da9052_start_adc (u8 channel)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_ADCCONT_REG;
	msg.data = 0;
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);

	/* Set the bit for corresponding channel in ADC_CONT register */
	if (channel == ADC_VDDOUT) { 
		msg.data = set_bits(msg.data, DA9052_ADCCONT_AUTOVDDEN);
	} else if (channel == ADC_ADCIN4) {
		msg.data = set_bits(msg.data, DA9052_ADCCONT_AUTOAD4EN);
	} else if (channel == ADC_ADCIN5) {
		msg.data = set_bits(msg.data, DA9052_ADCCONT_AUTOAD5EN);
	} else if (channel == ADC_ADCIN6) {
		msg.data = set_bits(msg.data, DA9052_ADCCONT_AUTOAD6EN);
	} else {
		return (ADC_INVALID_CHANNEL_ERROR);
	}
	/* Write the value in the register */
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);
	return (SUCCESS);
}

 /**
 * da9052_stop_adc	:
 *			This function stops the ADC channel in AUTO mode.
 * 
 * @param channel 	- Channel No. to be stopped.
 *
 * @return status:  
 *	CHANNEL	- If the channel value passed to this functions is 
 *			  wrong.
 *	ADC_SSC_BUS_ERROR - If there is an error on the SSC bus.
 *	SUCCESS		  - On successful completion of the call.
 */  
s32 da9052_stop_adc (u8 channel)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_ADCCONT_REG;
	msg.data = 0;
	if (SUCCESS != da9052_ssc_read(&msg))
		return (ADC_SSC_BUS_ERROR);

	/* Clear the data in the ADC_CONT register */
	if (channel == ADC_VDDOUT) {
		msg.data = clear_bits(msg.data, DA9052_ADCCONT_AUTOVDDEN);
	} else if (channel == ADC_ADCIN4) {
		msg.data = clear_bits(msg.data, DA9052_ADCCONT_AUTOAD4EN);
	} else if (channel == ADC_ADCIN5) {
		msg.data = clear_bits(msg.data, DA9052_ADCCONT_AUTOAD5EN);
	} else if (channel == ADC_ADCIN6) {
		msg.data = clear_bits(msg.data, DA9052_ADCCONT_AUTOAD6EN);
	} else {
		return (ADC_INVALID_CHANNEL_ERROR);
	}
	/* Write the data */
	if (SUCCESS != da9052_ssc_write(&msg))
		return (ADC_SSC_BUS_ERROR);

	return (SUCCESS);
}
 
 /**
 * da9052_read_adc	:
 *			This function reads the ADC value for the channel 
 *			specified in the parameter.
 * 
 * @param channel 	- Channel No. to be read.
 * @param data		- Pointer to the location where data is to be copied.
 *
 * @return status:
 *	CHANNEL	- If the channel value passed to this functions is 
 *			  wrong.
 *	ADC_CH4_NOT_CONF	- If ADCIN4 pins not configured for ADC.
 *	ADC_CH5_NOT_CONF	- If ADCIN5 pins not configured for ADC.
 *	ADC_CH6_NOT_CONF	- If ADCIN6 pins not configured for ADC.
 *	ADC_CHANNEL_NOT_STARTED_ERROR - If the channel is not started in Auto 
 *					mode.
 *	ADC_SSC_BUS_ERROR	- If there is an error on the SSC bus.
 *	SUCCESS			- On successful completion of the call.
 */  
s32 da9052_read_adc (u8 channel, u16 *data)
{
	da9052_ssc_msg msg;
	s32 error;
	u16 calc_data;
	
	switch(channel){
	/* VDD_OUT channel */
	case ADC_VDDOUT:{
		/* If manual mode do manual read */
		if(adc_mode_status.adc_vddout == MANUAL_MODE){
			error = da9052_manual_read(channel, &calc_data);
			if (error != SUCCESS) {
				return (error);
			} else {
				/* 8 bit data in manual mode */
				*data = calc_data >> 2;
			}
		/* Check Auto mode */
		} else {
			/* Check if channel is enabled */
			msg.addr = DA9052_ADCCONT_REG;
			if (SUCCESS != da9052_ssc_read(&msg))
				return (ADC_SSC_BUS_ERROR);
			if(msg.data & DA9052_ADCCONT_AUTOVDDEN) {
				msg.addr = DA9052_VDDRES_REG;
				if (SUCCESS != da9052_ssc_read(&msg))
					return (ADC_SSC_BUS_ERROR);
				*data = (u16)msg.data;
			} else {
				return (ADC_CHANNEL_NOT_STARTED_ERROR);
			}
		}
		return (SUCCESS);
	}
	case ADC_ICH:{ 
		/* If manual mode do manual read */
		if(adc_mode_status.adc_ich == MANUAL_MODE) {
			error = da9052_manual_read(channel, &calc_data);
			if (error != SUCCESS) {
				return (error);
			} else {
				/* 8 bit data in manual mode */
				*data = calc_data >> 2;
			}
		} else {
			/* Read data in AUTO mode */
			msg.addr = DA9052_ICHGAV_REG;
			if (SUCCESS != da9052_ssc_read(&msg))
				return (ADC_SSC_BUS_ERROR);
			*data = (u16)msg.data;
		}
		return (SUCCESS);
	}
	case ADC_TBAT:{
		/* If manual mode do manual read */
		if(adc_mode_status.adc_tbat == MANUAL_MODE){
			error = da9052_manual_read(channel, &calc_data);
			if (error != SUCCESS) {
				return (error);
			} else {
				/* 8 bit data in manual mode */
				*data = calc_data >> 2;
			}
		} else {
			/* Read data in AUTO mode */
			msg.addr = DA9052_TBATRES_REG;
			if (SUCCESS != da9052_ssc_read(&msg))
				return (ADC_SSC_BUS_ERROR);
			*data = (u16)msg.data;
		}
		return (SUCCESS);
	}
	case ADC_VBAT:{
		if(adc_mode_status.adc_vbat == MANUAL_MODE) {
			return (da9052_manual_read(channel, data));
		} else {
			return (ADC_INVALID_CHANNEL_ERROR);
		}
		return (SUCCESS);
	}
	case ADC_ADCIN4:{
#if (ADC_CONF_ADC4)
		/* If manual mode do manual read */
		if(adc_mode_status.adc_in4 == MANUAL_MODE){
			error = da9052_manual_read(channel, &calc_data);
			if (error != SUCCESS) {
				return (error);
			} else {
				/* 8 bit data in manual mode */
				*data = calc_data >> 2;
			}
		/* Check Auto mode */
		} else {
			/* Check if channel is enabled */
			msg.addr = DA9052_ADCCONT_REG;
			if (SUCCESS != da9052_ssc_read(&msg))
				return (ADC_SSC_BUS_ERROR);
			if(msg.data & DA9052_ADCCONT_AUTOAD4EN) {
				msg.addr = DA9052_ADCIN4RES_REG;
				if (SUCCESS != da9052_ssc_read(&msg))
					return (ADC_SSC_BUS_ERROR);
				*data = (u16)msg.data;
			} else {
				return (ADC_CHANNEL_NOT_STARTED_ERROR);
			}
		}
		return (SUCCESS);
#else
		/* If the ADCIN4 pin not configured for ADC */
		return (ADC_CH4_NOT_CONF);
#endif
	}
	case ADC_ADCIN5:{/* ADCIN5 */
#if (ADC_CONF_ADC5)
		/* If manual mode do manual read */
		if(adc_mode_status.adc_in5 == MANUAL_MODE){
			error = da9052_manual_read(channel, &calc_data);
			if (error != SUCCESS) {
				return (error);
			} else {
				/* 8 bit data in manual mode */
				*data = calc_data >> 2;
			}
		/* Check Auto mode */
		} else {
			/* Check if channel is enabled */
			msg.addr = DA9052_ADCCONT_REG;
			if (SUCCESS != da9052_ssc_read(&msg))
				return (ADC_SSC_BUS_ERROR);
			if(msg.data & DA9052_ADCCONT_AUTOAD5EN) {
				msg.addr = DA9052_ADCIN5RES_REG;
				if (SUCCESS != da9052_ssc_read(&msg))
					return (ADC_SSC_BUS_ERROR);
				*data = (u16)msg.data;
			} else {
				return (ADC_CHANNEL_NOT_STARTED_ERROR);
			}
		}
		return (SUCCESS);
#else
		/* If the ADCIN5 pin not configured for ADC */
		return (ADC_CH5_NOT_CONF);
#endif
	}
	case ADC_ADCIN6:{/* ADCIN6 */
#if (ADC_CONF_ADC6)
		/* If manual mode do manual read */
		if(adc_mode_status.adc_in6 == MANUAL_MODE){
			error = da9052_manual_read(channel, &calc_data);
			if (error != SUCCESS) {
				return (error);
			} else {
				/* 8 bit data in manual mode */
				*data = calc_data >> 2;
			}
		/* Check Auto mode */
		} else {
			/* Check if channel is enabled */
			msg.addr = DA9052_ADCCONT_REG;
			if (SUCCESS != da9052_ssc_read(&msg))
				return (ADC_SSC_BUS_ERROR);
			if(msg.data & DA9052_ADCCONT_AUTOAD6EN) {
				msg.addr = DA9052_ADCIN6RES_REG;
				if (SUCCESS != da9052_ssc_read(&msg))
					return (ADC_SSC_BUS_ERROR);
				*data = (u16)msg.data;
			} else {
				return (ADC_CHANNEL_NOT_STARTED_ERROR);
			}
		}
		return (SUCCESS);
#else
		/* If the ADCIN6 pin not configured for ADC */
		return (ADC_CH6_NOT_CONF);
#endif
	}
	case ADC_TSI:{
		return (ADC_INVALID_CHANNEL_ERROR);
	}
	case ADC_TJUNC:{
		if (adc_mode_status.adc_tsense == MANUAL_MODE) {
			error = da9052_manual_read(channel, &calc_data);
			if (error != SUCCESS) {
				return (error);
			} else {
				/* 8 bit data in manual mode */
				*data = calc_data >> 2;
			}
		} else {
			/* Read data in AUTO mode */
			msg.addr = DA9052_TJUNCRES_REG;
			if (SUCCESS != da9052_ssc_read(&msg))
				return (ADC_SSC_BUS_ERROR);
			*data = (u16)msg.data;
		}
		return (SUCCESS);
	}
	case ADC_VBBAT:{
		if (adc_mode_status.adc_vbbat == MANUAL_MODE) {
			return (da9052_manual_read(channel, data));
		} else {
			return (ADC_INVALID_CHANNEL_ERROR);
		}
		return (SUCCESS);
	}
	/* Check Illegal Channel */
	default :
		return (ADC_INVALID_CHANNEL_ERROR);
	}

	return (SUCCESS);
}

/**
 * da9052_adc_adcin4_handler : register the event
 * 
 * @param void
 * 
 * @return s32   Error Code, zero: no error
 */
void da9052_adc_adcin4_handler(u32 data)
{
	/* Acquire lock before accessing the status variable */
	mutex_lock(&adc_event_occ_lock);
	/* Store the status of the event in the variable */
	adc_event_occ_status.event_bits_t.adc_adcin4_e_occ = TRUE;
	/* Release the lock */
	mutex_unlock(&adc_event_occ_lock);
	/* Signal to the user space */
	da9052_adc_signal_to_user(GPI0_EVE);
}

/**
 * da9052_adc_adcin5_handler : register the event
 * 
 * @param void
 * 
 * @return s32   Error Code, zero: no error
 */
void da9052_adc_adcin5_handler(u32 data)
{
	/* Acquire lock before accessing the status variable */
	mutex_lock(&adc_event_occ_lock);
	/* Store the status of the event in the variable */
	adc_event_occ_status.event_bits_t.adc_adcin5_e_occ = TRUE;
	/* Release the lock */
	mutex_unlock(&adc_event_occ_lock);
	/* Signal to the user space */
	da9052_adc_signal_to_user(GPI1_EVE);
}

/**
 * da9052_adc_adcin6_handler : register the event
 * 
 * @param void
 * 
 * @return s32   Error Code, zero: no error
 */
void da9052_adc_adcin6_handler(u32 data)
{
	/* Acquire lock before accessing the status variable */
	mutex_lock(&adc_event_occ_lock);
	/* Store the status of the event in the variable */
	adc_event_occ_status.event_bits_t.adc_adcin6_e_occ = TRUE;
	/* Release the lock */
	mutex_unlock(&adc_event_occ_lock);
	/* Signal to the user space */
	da9052_adc_signal_to_user(GPI2_EVE);
}


/**
 * da9052_1v2_comp_handler : register the event
 * 
 * @param void
 * 
 * @return s32   Error Code, zero: no error
 */
void da9052_1v2_comp_handler(u32 data)
{
	/* Acquire lock before accessing the status variable */
	mutex_lock(&adc_event_occ_lock);
	/* Store the status of the event in the variable */
	adc_event_occ_status.event_bits_t.adc_comp1v2_e_occ = TRUE;
	/* Release the lock */
	mutex_unlock(&adc_event_occ_lock);
	/* Signal to the user space */
	da9052_adc_signal_to_user(COMP_1V2);
}

/**
 * da9052_adc_register_event : register the event
 * 
 * @param u8 event_type	event type
 * 
 * @return s32   Error Code, zero: no error
 */
s32 da9052_adc_register_event(u8 event_type)
{
	/* Register the callback function with event handler */
	switch(event_type) {
	case GPI0_EVE:
#if ADC_CONF_ADC4
		/* Check if event is already registered */
		if (!is_eh_registered.adc4_event) {
			/* Set the EH structure */
			adc4_eh_data.call_back = da9052_adc_adcin4_handler;
			adc4_eh_data.eve_type = event_type;
			/* Register the event with EH */
			if (da9052_eh_register_nb(&adc4_eh_data))
				return (ADC_EVENT_REGISTRATION_FAILED);
			/* Set the registration flag to 1 */
			is_eh_registered.adc4_event = TRUE;
		}
#else
		return (ADC_CH4_NOT_CONF);
#endif
		break;
	case GPI1_EVE:
#if ADC_CONF_ADC5
		/* Check if event is already registered */
		if (!is_eh_registered.adc5_event) {
			/* Set the EH structure */
			adc5_eh_data.call_back = da9052_adc_adcin5_handler;
			adc5_eh_data.eve_type = event_type;
			/* Register the event with EH */
			if (da9052_eh_register_nb(&adc5_eh_data))
				return (ADC_EVENT_REGISTRATION_FAILED);
			/* Set the registration flag to 1 */
			is_eh_registered.adc5_event = TRUE;
		}
#else
		return (ADC_CH5_NOT_CONF);
#endif
		break;
	case GPI2_EVE:
#if ADC_CONF_ADC6
		/* Check if event is already registered */
		if (!is_eh_registered.adc6_event) {
			/* Set the EH structure */
			adc6_eh_data.call_back = da9052_adc_adcin6_handler;
			adc6_eh_data.eve_type = event_type;
			/* Register the event with EH */
			if (da9052_eh_register_nb(&adc6_eh_data))
				return (ADC_EVENT_REGISTRATION_FAILED);
			/* Set the registration flag to 1 */
			is_eh_registered.adc6_event = TRUE;
		}
#else
		return (ADC_CH6_NOT_CONF);
#endif
		break;
	case COMP_1V2:
		/* Check if event is already registered */
		if (!is_eh_registered.comp_1v2_event) {
			/* Set the EH structure */
			comp_1v2_eh_data.call_back = da9052_1v2_comp_handler;
			comp_1v2_eh_data.eve_type = event_type;
			/* Register the event with EH */
			if (da9052_eh_register_nb(&comp_1v2_eh_data))
				return (ADC_EVENT_REGISTRATION_FAILED);
			/* Set the registration flag to 1 */
			is_eh_registered.comp_1v2_event = TRUE;
		}		
		break;
	default:
		return (ADC_INVALID_EVENT);
	}

	return (SUCCESS);
}

/**
 * da9052_adc_get_event : Get the status of the events occured. Required by the 
 *							user space application
 * 
 * @param void
 * 
 * @return status
 */
s32 da9052_adc_get_event(unsigned long *argument)
{
	u8 events;
	/* Acquire lock */
	mutex_lock(&adc_event_occ_lock);
	/* Read the value to the local variable */
	events = adc_event_occ_status.adc_event_status;
	/* Clear the Global variable */
	adc_event_occ_status.adc_event_status = FALSE;
	/* Release the lock */
	mutex_unlock(&adc_event_occ_lock);
	/* Copy the event to the user space */
	return(copy_to_user((u8 *)(*argument), (void *)&events, sizeof(u8)));
}
 
/**
 * da9052_adc_unregister_event : register the event
 * 
 * @param u8 event_type	event type
 * 
 * @return s32   Error Code, zero: no error
 */
s32 da9052_adc_unregister_event(u8 event_type)
{
	/* Check event type */
	switch (event_type) {
	case GPI0_EVE:
#if ADC_CONF_ADC4
		/* Unregister event */
		if (is_eh_registered.adc4_event) {
			if (da9052_eh_unregister_nb(&adc4_eh_data)) {
				return (ADC_EVENT_UNREGISTRATION_FAILED);
			}
			/* Set flag to 0 */
			is_eh_registered.adc4_event = FALSE;
		}
#else
		return (ADC_CH4_NOT_CONF);
#endif
		break;
	case GPI1_EVE:
#if ADC_CONF_ADC5
		/* Unregister event */
		if (is_eh_registered.adc5_event) {
			if (da9052_eh_unregister_nb(&adc5_eh_data)) {
				return (ADC_EVENT_UNREGISTRATION_FAILED);
			}
			/* Set flag to 0 */
			is_eh_registered.adc5_event = FALSE;
		}
#else
		return (ADC_CH5_NOT_CONF);
#endif
		break;
	case GPI2_EVE:
#if ADC_CONF_ADC6
		/* Unregister event */
		if (is_eh_registered.adc6_event) {
			if (da9052_eh_unregister_nb(&adc6_eh_data)) {
				return (ADC_EVENT_UNREGISTRATION_FAILED);
			}
			/* Set flag to 0 */
			is_eh_registered.adc6_event = FALSE;
		}
#else
		return (ADC_CH6_NOT_CONF);
#endif
		break;
	case COMP_1V2:
		/* Unregister event */
		if (is_eh_registered.comp_1v2_event) {
			if (da9052_eh_unregister_nb(&comp_1v2_eh_data)) {
				return (ADC_EVENT_UNREGISTRATION_FAILED);
			}
			/* Set flag to 0 */
			is_eh_registered.comp_1v2_event = FALSE;
		}
		break;
	default:
		return (ADC_INVALID_EVENT);
		break;
	}
	return (SUCCESS);
}
/*--------------------------------------------------------------------------*/
/* Infrastructure Functions                                                 */
/*--------------------------------------------------------------------------*/
/**
 * da9052_adc_open: 
 * 
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return s32   Error Code, zero: no error
 */
s32 da9052_adc_open(struct inode *inode, struct file *file)
{
	/* Check if device is already open */
	if(adc_device_open) {
		printk("DA9052: ADC device already open.\n");
		return (-EBUSY);
	} else {
		adc_device_open++;
		return (SUCCESS);
	}
}

/**
 * da9052_adc_release:
 * 
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return s32   Error Code, zero: no error
 */
s32 da9052_adc_release(struct inode *inode, struct file *file)
{
	adc_device_open--;
	printk("DA9052: ADC device closed.\n");
	return (SUCCESS);
}

/**
 * da9052_adc_suspend: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @param state  pm state
 * @return s32 status of suspend operation
 */
static s32 da9052_adc_suspend(struct platform_device *dev, pm_message_t state)
{
     /* Put your suspend related operations here */
     printk(KERN_INFO "%s: called\n", __FUNCTION__);
     return (0);
}

/**
 * da9052_adc_resume: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @return s32 status of resume operation
 */
static s32 da9052_adc_resume(struct platform_device *dev)
{
     /* Put your resume related operations here */
     printk(KERN_INFO "%s: called\n", __FUNCTION__);
     return (0);
}

/**
 * da9052_adc_ioctl: 
 * 
 * @param *inode   pointer to device inode
 * @param *file   file pointer
 * @param cmd   command to be executed
 * @param arg   argument to command
 * @return s32   Error Code, zero: no error
 */
s32 da9052_adc_ioctl(struct inode *inode, struct file *file, 
					u32 cmd, unsigned long arg)
{
	s32 error = SUCCESS;
	u8 event_data, *event_ptr;
	da9052_adc_data_t usr_param, *ptr;
#if DA9052_ADC_DEBUG
	u16 data;
#endif /* DA9052_UNIT_TESTING_ADC */

	
	ptr = (da9052_adc_data_t *) arg;

	/* Switch as per the command requested */
	switch(cmd) {
	case DA9052_ADC_IOCTL_SET_ADC_MODE:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_set_adc_mode(usr_param.channel, usr_param.mode);
		break;
	case DA9052_ADC_IOCTL_SET_SAMPLING_INTERVAL:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_adc_set_sampling_interval(usr_param.interval);
		break;
	case DA9052_ADC_IOCTL_CONFIGURE_THRESHOLDS:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_configure_thresholds (usr_param.threshold);
		break;
	case DA9052_ADC_IOCTL_SET_ADCIN5_COMPARATOR:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_set_adcin5_comparator(usr_param.action);
		break;
	case DA9052_ADC_IOCTL_SET_TBAT_SOURCE_CURRENT:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_set_tbat_source_current(usr_param.action);
		break;
	case DA9052_ADC_IOCTL_SET_ADCIN4_SOURCE_CURRENT:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_set_ad4_isrc_source_current(usr_param.action);
		break;
	case DA9052_ADC_IOCTL_START_ADC:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_start_adc(usr_param.channel);
		break;
	case DA9052_ADC_IOCTL_STOP_ADC:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_stop_adc(usr_param.channel);
		break;
	case DA9052_ADC_IOCTL_READ_ADC:
		if (copy_from_user(&usr_param, ptr, sizeof(da9052_adc_data_t)))
			return (FAILURE);
		error = da9052_read_adc(usr_param.channel, usr_param.data);
		if (copy_to_user((u16 *)arg, (void *)&usr_param, 
						sizeof(da9052_adc_data_t)))
			return (FAILURE);
		break;
	case DA9052_ADC_IOCTL_REGISTER_EVENT:
		event_ptr = (u8 *) arg;
		if (copy_from_user(&event_data, event_ptr, sizeof(u8)))
			return (FAILURE);
		error = da9052_adc_register_event(event_data);
		break;
	case DA9052_ADC_IOCTL_UNREGISTER_EVENT:
		event_ptr = (u8 *) arg;
		if (copy_from_user(&event_data, event_ptr, sizeof(u8)))
			return (FAILURE);
		error = da9052_adc_unregister_event(event_data);			
		break;
	case DA9052_ADC_IOCTL_GET_EVENT:
		if (da9052_adc_get_event(&arg))
			return (FAILURE);
		break;
#if DA9052_ADC_DEBUG
	case DA9052_ADC_IOCTL_UT_VDDOUT:
		error = da9052_adc_read_vddout(&data);
		if (copy_to_user((u16 *)arg, (void *)&data, sizeof(u16)))
			return (FAILURE);
		break;
	case DA9052_ADC_IOCTL_UT_ICH:
		error = da9052_adc_read_ich(&data);
		if (copy_to_user((u16 *)arg, (void *)&data, sizeof(u16)))
			return (FAILURE);
		break;
	case DA9052_ADC_IOCTL_UT_TBAT:
		error = da9052_adc_read_tbat(&data);
		if (copy_to_user((u16 *)arg, (void *)&data, sizeof(u16)))
			return (FAILURE);
		break;
	case DA9052_ADC_IOCTL_UT_VBAT:
		error = da9052_adc_read_vbat(&data);
		if (copy_to_user((u16 *)arg, (void *)&data, sizeof(u16)))
			return (FAILURE);
		break;
	case DA9050_ADC_IOCTL_UT_TJUNC:
		error = da9052_adc_read_tjunc(&data);
		if (copy_to_user((u16 *)arg, (void *)&data, sizeof(u16)))
			return (FAILURE);
		break;
	case DA9052_ADC_IOCTL_UT_VBBAT:
		error = da9052_adc_read_vbbat(&data);
		if (copy_to_user((u16 *)arg, (void *)&data, sizeof(u16)))
			return (FAILURE);
		break;
#endif /* DA9052_ADC_DEBUG */
	default:
		DA9052_DEBUG("Invalid ioctl command\n");
		return (ADC_NO_IOCTL_CMD);	
	}
	return (error);
}

/**
 * da9052_bat_fasync: 
 * 
 * @param void 	 <Direction> <Description>
 * @return s32   <Description>
 */
s32 da9052_adc_fasync (s32 fd, struct file *filp, s32 on)
{
  DA9052_DEBUG ("In %s: %s\n",__FILE__, __FUNCTION__);
  return(fasync_helper(fd, filp, on, &adc_fasync_queue));
}

/**
 * static struct file_operations da9052_adc_fops -  
 * This structure definition has to be defined here as an exception. 
 * @owner: member description
 * @open : member description
 * @release: member description
 * @read: member description
 * @write: member description
 * @ioctl: member description
 *
 */
static const struct file_operations da9052_adc_fops = {
        .owner          = THIS_MODULE,
        .open           = da9052_adc_open,
        .release        = da9052_adc_release,
        .ioctl          = da9052_adc_ioctl,
	.fasync		= da9052_adc_fasync
};

/**
 * da9052_adc_probe: Called when a device gets attached to driver 
 * @param *dev   pointer to platform device
 * @return s32   Error Code, zero: no error
 */
static s32 __devinit da9052_adc_probe(struct platform_device *dev)
{
	s32 ret;
	
	/* Register the device */
	ret = register_chrdev(adc_major_number,
				DA9052_ADC_DEVICE_NAME, &da9052_adc_fops);
	if (ret != 0) {
		printk(KERN_ERR "Unable to register %s\n",
						DA9052_ADC_DEVICE_NAME);
		return (-EFAULT);	
	} else {
		printk(KERN_INFO "%s: Major number is: %d.\n", 
				DA9052_ADC_DEVICE_NAME, adc_major_number);
		
		if (da9052_adc_hw_init()) {
			/* Error in hw initialization */
			unregister_chrdev(adc_major_number, 		\
						DA9052_ADC_DEVICE_NAME);
			return (-EFAULT);
		} else {
			return (SUCCESS);
		}
	}
	return(SUCCESS);
}

/**
 * da9052_adc_remove: Called when ditaching device from driver
 * @param void
 * @return void
 */
static s32 __devexit da9052_adc_remove(struct platform_device *dev)
{
	DA9052_DEBUG(KERN_DEBUG "Removing %s \n", DA9052_ADC_DEVICE_NAME);
	return (SUCCESS);
}

/**
 * static struct platform_driver da9052_adc_driver -
 * This structure definition has to be defined here as an exception.
 * @probe: Probe function for this device. 
 * @remove: Function to be called when removing this device from platform
 * @driver: Contains glue logic to bind platform device and plarform driver
 */
static struct platform_driver da9052_adc_driver = {
	.probe		= da9052_adc_probe,
	.remove		= __devexit_p(da9052_adc_remove),
	.suspend	= da9052_adc_suspend,
	.resume		= da9052_adc_resume,
	.driver		= {
		.name	= DA9052_ADC_DEVICE_NAME,
		.owner	= THIS_MODULE,
	},
};

/**
 * da9052_adc_init: Initiales the driver
 * 
 * @param void
 * @return s32   Error Code, zero: no error
 */
static s32 __init da9052_adc_init(void)
{
	s32 retval;
	printk(banner);

	da9052_adc_platform_device = platform_device_alloc("da9052_adc", 0);
	if (!da9052_adc_platform_device)
		return (-ENOMEM);

	retval = platform_device_add(da9052_adc_platform_device);
	if (retval < 0) {
		platform_device_put(da9052_adc_platform_device);
		return (retval);
	}

	retval = platform_driver_register(&da9052_adc_driver);
	if (retval < 0)
		platform_device_unregister(da9052_adc_platform_device);

	return (retval);
}
/**
 * da9052_adc_exit: 
 * 
 * @param void
 * @return void
 */
static void __exit da9052_adc_exit(void)
{
	da9052_adc_hw_deinit();
	printk("DA9052: Unregistering ADC device.\n");
	unregister_chrdev(adc_major_number, DA9052_ADC_DEVICE_NAME);

	/* Platform_Device */
	platform_driver_unregister(&da9052_adc_driver);
	platform_device_unregister(da9052_adc_platform_device);
}


module_init(da9052_adc_init);
module_exit(da9052_adc_exit);

MODULE_AUTHOR("Dialog Semiconductor Ltd");
MODULE_DESCRIPTION("DA9052 ADC Device Driver");
MODULE_LICENSE("GPL");

/*--------------------------------------------------------------------------*/
/* Exports                                                                  */
/*--------------------------------------------------------------------------*/

EXPORT_SYMBOL(da9052_set_adc_mode);
EXPORT_SYMBOL(da9052_configure_thresholds);
EXPORT_SYMBOL(da9052_set_adcin5_comparator);
EXPORT_SYMBOL(da9052_set_tbat_source_current);
EXPORT_SYMBOL(da9052_set_ad4_isrc_source_current);
EXPORT_SYMBOL(da9052_start_adc);
EXPORT_SYMBOL(da9052_stop_adc);
EXPORT_SYMBOL(da9052_read_adc);
EXPORT_SYMBOL(da9052_adc_set_sampling_interval);
EXPORT_SYMBOL(da9052_adc_read_vddout);
EXPORT_SYMBOL(da9052_adc_read_ich);
EXPORT_SYMBOL(da9052_adc_read_tbat);
EXPORT_SYMBOL(da9052_adc_read_vbat);
EXPORT_SYMBOL(da9052_adc_read_tjunc);
EXPORT_SYMBOL(da9052_adc_read_vbbat);
/* Mutex structure required for TSI */
EXPORT_SYMBOL(da9052_adc_manconv_lock);
