/*
 * da9052_eh.c: Event Handler module for DA9052 PM IC
 *
 * DA9052 has a single IRQ line which can be used to indicate 32 various events.
 * Event Handler (EH) module has following primary functions.
 *
 * (1) To receive and acknowledge interrupts from DA9052.
 * (2) Extract all events form DA9052 event registers after receiving interrupt.
 * (3) To provide a mechanism to register call back for specific events of
 *     their interest, and call these call backs when corresponding event occurs.
 * (4) To retrive, store and provide TSI data whenever requested by TSI module.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * 12/05/2009:	Initial version
 * 19/05/2009:	First unit tested version

 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/workqueue.h>
#include <linux/platform_device.h>
#include <linux/interrupt.h>

#include <asm/mach/irq.h>
#include <asm/gpio.h>
#include <asm/delay.h>
#include <plat/gpio-cfg.h>

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_reg.h"
#include "../include/da9052_lib.h"
#include "../include/da9052_eh.h"

/*--------------------------------------------------------------------------*/
/* Local Type Definitions                                                   */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Local Constant Definitions                                               */
/*--------------------------------------------------------------------------*/
static u8 banner[] __initdata = "DA9052 Event Handler (EH) Driver, v1.0\n";


/*--------------------------------------------------------------------------*/
/* Local Macro Definitions                                                  */
/*--------------------------------------------------------------------------*/

/* Increment with wrap operation for TSI FIFO */
#define incr_with_wrap(x)	\
		if(++x >= TSI_FIFO_SIZE)	\
			x = 0
/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/

/* Variable to store "open" status of EH device node */
static u8 eh_device_open = 0;

/* Variable to store major number of EH device node */
static s32 eh_major_number = DA9052_EH_DEVICE_MAJOR_NO;

/* Event notifier block array */
da9052_eh_nb eve_nb_array[EVE_CNT];

/* Event priority mappings */
u8 eve_prio_map[PRIO_CNT];

/* Lock for event notifier block array */
struct semaphore eve_nb_array_lock;

/* Lock for event notifier block array */
struct semaphore tsi_fifo_lock;

/* TSI Sampling status */
u8 tsi_sampling = INACTIVE; 

/* State of TSI state machine */
u8 tsi_state = WAIT_FOR_PEN_DOWN;

/* Fifo to store TSI samples */
da9052_tsi_reg tsi_fifo[TSI_FIFO_SIZE];
u32 tsi_fifo_start = 0;
u32 tsi_fifo_end = 0;

/* Master function to process events */
static s32 process_events(s32 events_sts);

/* Function to process TSI related events */
static s32 process_tsi_event(u8 event);

/* Platform device structure for EH */
static struct platform_device *da9052_eh_platform_device;

/* Work queue structure */
struct work_struct eh_isr_work;

/* Work queur function, heart of EH! */
void eh_workqueue_isr(struct work_struct* work);
/*--------------------------------------------------------------------------*/
/* Local Functions                                                          */
/*--------------------------------------------------------------------------*/

/**
 * da9052_eh_setup_irq:	Function to setup IRQ line which is connected to DA9052 
 * @param void	
 * @return void
 */
static void da9052_eh_setup_irq(void)
{
	/* Put your platform and board specific code here */
	
	/* Set up GPI which is connected to nIRQ of DA9052 */
	s3c_gpio_setpull(DA9052_IRQ_PIN, S3C_GPIO_PULL_UP);
	
	/* Set GPI as LOW LEVEL interrupt source */ 
	set_irq_type(DA9052_IRQ, IRQ_TYPE_LEVEL_LOW);
	
}

/**
 * da9052_eh_restore_irq:Function to free IRQ line which is connected to DA9052 
 * @param void	
 * @return void
 */
static void da9052_eh_restore_irq(void)
{
	/* Put your platform and board specific code here */
	free_irq(DA9052_IRQ, NULL);
}

/**
 * da9052_eh_isr: ISR to handle interrupts from DA9052
 * 
 * @param irq	IRQ number
 * @param *dev_id Accociated device id
 * @return irqreturn_t
 */
static irqreturn_t da9052_eh_isr(int irq, void *dev_id)
{	
	/* Schedule work to be done */
	schedule_work(&eh_isr_work);
	
	/* Disable IRQ */
	disable_irq_nosync(DA9052_IRQ);
	
	return (IRQ_HANDLED);
}

/**
 * da9052_eh_init_eve_prio_map:	Initialize mapping of EVENTS with their PRIORITIES
 * @param void	
 * @return void
 */
static void da9052_eh_init_eve_prio_map(void)
{
	/*
	 * Initialize priority array by assigning corresponding event for each
	 * priotiry
	*/
	eve_prio_map[PRIO_0] = PRIO_0_EVE;
	eve_prio_map[PRIO_1] = PRIO_1_EVE;
	eve_prio_map[PRIO_2] = PRIO_2_EVE;
	eve_prio_map[PRIO_3] = PRIO_3_EVE;
	eve_prio_map[PRIO_4] = PRIO_4_EVE;
	eve_prio_map[PRIO_5] = PRIO_5_EVE;
	eve_prio_map[PRIO_6] = PRIO_6_EVE;
	eve_prio_map[PRIO_7] = PRIO_7_EVE;
	eve_prio_map[PRIO_8] = PRIO_8_EVE;
	eve_prio_map[PRIO_9] = PRIO_9_EVE;
	eve_prio_map[PRIO_10] = PRIO_10_EVE;
	eve_prio_map[PRIO_11] = PRIO_11_EVE;
	eve_prio_map[PRIO_12] = PRIO_12_EVE;
	eve_prio_map[PRIO_13] = PRIO_13_EVE;
	eve_prio_map[PRIO_14] = PRIO_14_EVE;
	eve_prio_map[PRIO_15] = PRIO_15_EVE;
	eve_prio_map[PRIO_16] = PRIO_16_EVE;
	eve_prio_map[PRIO_17] = PRIO_17_EVE;
	eve_prio_map[PRIO_18] = PRIO_18_EVE;
	eve_prio_map[PRIO_19] = PRIO_19_EVE;
	eve_prio_map[PRIO_20] = PRIO_20_EVE;
	eve_prio_map[PRIO_21] = PRIO_21_EVE;
	eve_prio_map[PRIO_22] = PRIO_22_EVE;
	eve_prio_map[PRIO_23] = PRIO_23_EVE;
	eve_prio_map[PRIO_24] = PRIO_24_EVE;
	eve_prio_map[PRIO_25] = PRIO_25_EVE;
	eve_prio_map[PRIO_26] = PRIO_26_EVE;
	eve_prio_map[PRIO_27] = PRIO_27_EVE;
	eve_prio_map[PRIO_28] = PRIO_28_EVE;
	eve_prio_map[PRIO_29] = PRIO_29_EVE;
	eve_prio_map[PRIO_30] = PRIO_30_EVE;
	eve_prio_map[PRIO_31] = PRIO_31_EVE;
}

/*--------------------------------------------------------------------------*/
/* Global Functions                                                         */
/*--------------------------------------------------------------------------*/
/**
 * da9052_eh_register_nb:	Function to register a notifier block
 * @param nb	: Notifier block to register
 * @return s32	: Status of registration
 */
s32 da9052_eh_register_nb(da9052_eh_nb* nb){
	
	if(nb == NULL){
		/* Invalid notifier block */
		return (INVALID_NB);
	}
	
	if(nb->eve_type >= EVE_CNT){
		/* Invalid event type */
		return (INVALID_EVE);
	}
		
	DA9052_DEBUG("%s() called for event %d \n", __FUNCTION__, nb->eve_type);

	/* Initialize list head inside notifier block */
	INIT_LIST_HEAD(&nb->nb_list);
	
	/* Acqurie NB array lock */
	if(down_interruptible(&eve_nb_array_lock))
		return (FAILURE);
	
	/* Add passed NB to corresponsing EVENT list */
	list_add_tail(&nb->nb_list, &(eve_nb_array[nb->eve_type].nb_list));
	
	/* Release NB array lock */
	up(&eve_nb_array_lock);
	
	return (SUCCESS);
}

/**
 * da9052_eh_unregister_nb:	Function to de-register a notifier block
 *
 * Caller must not pass any non-existing notifier block or notifiler block
 * that has already been de-registered. Its caller's responsibility to 
 * allocate/deallocate memory associated with notifier block. EH will only 
 * Link/De-Link passed notifier blocks.
 *
 * @param nb	: Notifier block to unregister
 * @return s32	: Status of unregistration
 */
s32 da9052_eh_unregister_nb(da9052_eh_nb* nb){
	
	if(nb == NULL)
		return (INVALID_NB);
	
	DA9052_DEBUG("%s() called for event %d \n", __FUNCTION__, nb->eve_type);
	
	/* Acqurie nb array lock */
	if(down_interruptible(&eve_nb_array_lock))
		return (FAILURE);
	
	/* Remove passed NB from list */
	list_del_init(&(nb->nb_list));
	
	/* Release NB array lock */
	up(&eve_nb_array_lock);
	
	return (SUCCESS);
}

/**
 * da9052_tsi_start_sampling:	Called by TSI driver to indicate start of data
 * sampling
 * @param void
 * @return int status
 */
s32 da9052_tsi_start_sampling(void){
	
	/* This request is only valid when we are waiting for one */
	if(tsi_state == WAIT_FOR_SAMPLING){
		DA9052_DEBUG("__%s__ \n", __FUNCTION__);
		tsi_state =  SAMPLING_ACTIVE;
		return (SUCCESS);
	}
	else{
		DA9052_DEBUG("Invalid request for da9052_tsi_start_sampling() \n");
		return (FAILURE);
	}	
}

/**
 * da9052_tsi_stop_sampling:	Called by TSI driver to indicate end of data
 * sampling
 * @param void
 * @return int status
 */
s32 da9052_tsi_stop_sampling(void){
	
	/* Irrespective of current state, move to initial state */
	DA9052_DEBUG("__%s__ \n", __FUNCTION__);
	tsi_state =  WAIT_FOR_PEN_DOWN;

	return (SUCCESS);
}

/**
 * da9052_tsi_get_data:	Called by TSI driver to fetch sampled data
 *
 * This function tries to copy cnt number of samples into tsi fifo.
 * If available samples are less than cnt, then it copies all available
 * samples and returns number of samples copied.
 *
 * @param buf pointer tsi fifo
 * @param cnt number of samples to be copied
 * @return int number of samples copied in to tsi fifo
 */
s32 da9052_tsi_get_data(da9052_tsi_reg *buf, u8 cnt){
	
	u32 data_cnt = 0;
	u32 rem_data_cnt = 0;
	
	/* Acquire TSI FIFO lock */
	if(down_interruptible(&tsi_fifo_lock))
		return (0);

	if(tsi_fifo_start < tsi_fifo_end){
		/* FIFO partly or completely full but NOT overflown */
		
		/* Get available data count */
		data_cnt = (tsi_fifo_end - tsi_fifo_start);
		
		/* Now see how much data is requested */
		if(cnt < data_cnt)
			data_cnt = cnt;
		
		/* Copy requested (or available in case if not enough data) data */
		memcpy(buf, &tsi_fifo[tsi_fifo_start], sizeof(da9052_tsi_reg)*data_cnt);
		
		/* update start pointer */
		tsi_fifo_start += data_cnt;
		
		/* Reset both pointers if fifo in empty */
		if(tsi_fifo_start == tsi_fifo_end){
			//DA9052_DEBUG("1:FIFO Empty after copying \n");
			tsi_fifo_start = 0;
			tsi_fifo_end = 0;
		}
	}
	else if(tsi_fifo_start > tsi_fifo_end){
		/* 
		 * Overflow occured, FIFO may be full 
		 * Data is devided in to two parts in fifo
		 */
		
		/* Get available data count */
		data_cnt = ((TSI_FIFO_SIZE - tsi_fifo_start) + tsi_fifo_end);
		
		/* Now see how much data is requested */
		if(cnt < data_cnt)
			data_cnt = cnt;
	
		/* Check if we need to copy data from both parts or only one part */
		if(data_cnt <= (TSI_FIFO_SIZE - tsi_fifo_start)){
			/* We only need to copy data from first part */
			/* Copy from start ptr till required lenght */
			memcpy(buf, &tsi_fifo[tsi_fifo_start], sizeof(da9052_tsi_reg)*data_cnt);
			
			/* update start pointer */
			tsi_fifo_start += data_cnt;
			if(tsi_fifo_start >= TSI_FIFO_SIZE)
				tsi_fifo_start = 0;
		}else{
			/* We need to copy data from both parts */
			
			/* First copy from start ptr till end of fifo */
			memcpy(buf, &tsi_fifo[tsi_fifo_start], sizeof(da9052_tsi_reg)*(TSI_FIFO_SIZE - tsi_fifo_start));
			
			rem_data_cnt = (data_cnt - (TSI_FIFO_SIZE - tsi_fifo_start));
			 
			/* Then copy from start of fifo till required length */
			memcpy(buf, &tsi_fifo[0], sizeof(da9052_tsi_reg) * rem_data_cnt);
			
			/* update start pointer */
			tsi_fifo_start = rem_data_cnt;
		}
		
		/* Reset both pointers if fifo in empty */
		if(tsi_fifo_start == tsi_fifo_end){
			//DA9052_DEBUG("2:FIFO Empty after copying \n");
			tsi_fifo_start = 0;
			tsi_fifo_end = 0;
		}
	}
	else{
		/* FIFO empty */
		data_cnt = 0;
	}
	
	/* Release TSI FIFO lock */
	up(&tsi_fifo_lock);
	
	//DA9052_DEBUG("Copy Cnt - %d \n", data_cnt);
	return (data_cnt);
}

/**
 * eh_workqueue_isr:	Function to perform actual work of EH 
 * @param
 * @return int
 */
void eh_workqueue_isr(struct work_struct* work){
	
	da9052_ssc_msg eve_data[4];
	s32 events_sts, ret;
	u8 cnt = 0;
	
	//unsigned long start = 0;
	
	/* nIRQ is asserted, read event registeres to know what happered */
	events_sts = 0;
	
	/* Prepare ssc message to read all four event registers */
	for(cnt=0; cnt < 4; cnt++){
		eve_data[cnt].addr = (DA9052_EVENTA_REG + cnt);
		eve_data[cnt].data = 0;
	}
	
	/* Now read all event registers */
	ret = da9052_ssc_read_many(eve_data, 4);
	if(ret != SUCCESS) {
		DA9052_DEBUG("da9052_ssc_read_many failed in %s \n", __FUNCTION__);
		enable_irq(DA9052_IRQ);
		return;
	}
	
	/* Collect all events */
	for(cnt=0; cnt < 4; cnt++){
		events_sts |= (eve_data[cnt].data << (8 * cnt));
	}
	
	/*
	 * Check if we really got any event. If we haven't got any event then this 
	 * may be a fake interrupt. Just ignore it and come out of this function
	 * without wasting any time in processing
	 */
	if(events_sts == 0){
        //printk(KERN_CRIT "Fake   Interrupt!!! \n");
		enable_irq(DA9052_IRQ);
		return;
	}
	
	//DA9052_DEBUG("Events = 0x%8.8x \n", events_sts);
	
	/* Process all events occured */
	process_events(events_sts);

	/* Now clear EVENT registers */
    for(cnt=0; cnt < 4; cnt++){
		/* Here we clear only those registers which have some event(s) set */
		if(eve_data[cnt].data){
            ret = da9052_ssc_write(&eve_data[cnt]);
			if(ret != SUCCESS) {
				DA9052_DEBUG("da9052_ssc_write_many failed in %s \n", __FUNCTION__);
				enable_irq(DA9052_IRQ);
				return;
			}
		}
    }
	/*
	 * This delay is necessary to avoid hardware fake interrupts from DA9052.
	 * Because of the 33 us internal delay in sync operation of DA9052, nIRQ line
	 * changes state from LOW to HIGH after max. 33 us of clearing event register.
	 * If we don't wait for this time (by adding udelay()) before re-enabling the
	 * IRQ, we will immediately get another interrupt, as it is configured as 
	 * LOW LEVEL triggered. Experiments showed that delay of 50 us seems to be
	 * safe enough to get rid of fake interrupts.
	 */
	udelay(50);
	
	//printk(KERN_CRIT "T=%d ", (jiffies - start));
	
	/* Enable HOST interrupt */
	enable_irq(DA9052_IRQ);
}

/**
 * process_events: Function to process all events occured
 * @param
 * @return int
 */
static s32 process_events(s32 events_sts){
	
	s32 cnt = 0;
	s32 tmp_events_sts = 0;
	u8 event = 0;
		
	struct list_head *ptr;
	da9052_eh_nb *nb_ptr;
	
	//DA9052_DEBUG("__%s__\n", __FUNCTION__);

	/* Now we have retrieved all events, process them one by one */
	for(cnt = 0; cnt < PRIO_CNT; cnt++){
		
		/* Starting with highest priority event,traverse through all event */
		
		tmp_events_sts = events_sts;
		
		/* Find the event associted with higher priority */
		event = eve_prio_map[cnt];
		
		/* Check if interrupt is received for this event */
		if(!((tmp_events_sts >> event) & 0x1)) {
			
			/* Event bit is not set for this event */
			/* Move to next priority event */
			continue;
		}
		
		//DA9052_DEBUG("Event - %d occured \n", event);
		
		/*
		 * PEN DOWN event has to be handled in a special way. It should be
		 * ignored in following cases.
		 * (1)	If we are in a state other than WAIT_FOR_PEN_DOWN, i.e.
		 *	we have already received PEN DOWN
		 * (2)	If TSI hasn't registered (or deregistered) call back for
		 *	PEN DOWN, i.e. One possible case when TSI does this is
		 *	while shifting from AUTO to MANUAL mode 
		 */
		if(event == PEN_DOWN_EVE){
			if((tsi_state != WAIT_FOR_PEN_DOWN) || 
				list_empty(&(eve_nb_array[event].nb_list)))
				continue;
		}
		
		/* Valid TSI event, process it */
		if((event == TSI_READY_EVE) || (event == PEN_DOWN_EVE))
			process_tsi_event(event);

		/* Event bit is set, execute all registered call backs */
		if(down_interruptible(&eve_nb_array_lock)){
			printk(KERN_CRIT "Can't acquire eve_nb_array_lock \n");
			return (FAILURE);
		}
		
		list_for_each(ptr, &(eve_nb_array[event].nb_list)){
			
			/*
			 * nb_ptr will point to the structure in which nb_list
			 * is embedded
			 */
			nb_ptr = list_entry(ptr, da9052_eh_nb, nb_list);
			
			//DA9052_DEBUG("Executing call back @ 0x%8.8x \n", nb_ptr->call_back);
			nb_ptr->call_back(events_sts);
		}		
		up(&eve_nb_array_lock);
	}
	return (SUCCESS);
}

/**
 * process_tsi_event: Function to process all TSI related events
 * @param 
 * @return int
 */
static s32 process_tsi_event(u8 event){
	
	s32 ret = 0;
	da9052_ssc_msg tsi_data[4];
	
	//DA9052_DEBUG("__%s__\n", __FUNCTION__);
	
	switch(tsi_state){
		case WAIT_FOR_PEN_DOWN:
			if(event == PEN_DOWN_EVE){
				DA9052_DEBUG("PEN DOWN Detected \n");
				/* Move to WAIT_FOR_SAMPLING state */
				tsi_state = WAIT_FOR_SAMPLING;
			}
			break;
				
		case WAIT_FOR_SAMPLING:
			/* Do nothing */
			break;
				
		case SAMPLING_ACTIVE:
			if(event == TSI_READY_EVE){
				/* Read TSI registers */
				/*Read 107, 108, 109 & 110 */
				tsi_data[0].addr  = DA9052_TSIXMSB_REG;
				tsi_data[1].addr  = DA9052_TSIYMSB_REG;
				tsi_data[2].addr  = DA9052_TSILSB_REG;
				tsi_data[3].addr  = DA9052_TSIZMSB_REG;
					
				tsi_data[0].data  = 0;
				tsi_data[1].data  = 0;
				tsi_data[2].data  = 0;
				tsi_data[3].data  = 0;
				
				//ret = da9052_ssc_read_many(tsi_data, 4);
				ret = da9052_ssc_read_many(tsi_data, 4);
				if(ret != SUCCESS) {
					DA9052_DEBUG("Error in reading TSI data \n" );
					return (FAILURE);
				}
				/* Put this sample in TSI FIFO */
				/* Acqurie TSI FIFO lock */
				if(down_interruptible(&tsi_fifo_lock))
					break;
				tsi_fifo[tsi_fifo_end].x_msb = tsi_data[0].data;
				tsi_fifo[tsi_fifo_end].y_msb = tsi_data[1].data;
				tsi_fifo[tsi_fifo_end].z_msb = tsi_data[2].data;
				tsi_fifo[tsi_fifo_end].lsb   = tsi_data[3].data;
				incr_with_wrap(tsi_fifo_end);
				
				//DA9052_DEBUG("X_MSB = 0x%8.8x \n", tsi_data[0].data);
				
				/* Check if we have reached at the end of fifo */
				if(tsi_fifo_end == tsi_fifo_start)
					tsi_fifo_start++;
				/* Release TSI FIFO lock */
				up(&tsi_fifo_lock);
			}
			break;
		default:
			break;
	}
	return (SUCCESS);
}


/**
 * dump_nb_tree: Prints NB tree for all events on console.
 * This function is purely for DEBUG purpose
 * @param
 * @return void
 */
void dump_nb_tree(void){
	
	da9052_eh_nb *nb_ptr;
	struct list_head *ptr;
	u8 cnt = 0;
	
	/* Now we have retrieved all events, process them one by one */
	for(cnt = 0; cnt < EVE_CNT; cnt++){

		printk(KERN_CRIT "NB_LIST[%2.2d]--> ", cnt);
		/* Check if it's nb list contains atleast one nb for this event */

		/* Event bit is set and there are registered nbs for this event */
		/* Execute all registered call backs */
		
		list_for_each(ptr, &(eve_nb_array[cnt].nb_list)){			
			/*
			 * nb_ptr will point to the structure in which nb_list
			 * is embedded
			 */
			nb_ptr = list_entry(ptr, da9052_eh_nb, nb_list);
			printk(" [%d] ", nb_ptr->eve_type);
		}
		printk(KERN_CRIT " \n");
	}
	if(cnt == EVE_CNT)
		printk(KERN_CRIT " \n");
}

/*--------------------------------------------------------------------------*/
/* Infrastructure Functions                                                 */
/*--------------------------------------------------------------------------*/
/**
 * da9052_eh_open: Function that is called when some one opens EH device node
 * 
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return int   Error Code, zero: no error
 */
s32 da9052_eh_open(struct inode *inode, struct file *file)
{
	/* Check if device is already open */
	if(eh_device_open) {
		printk( KERN_INFO "DA9052: EH device already open.\n");
		return (-EBUSY);
	} else {
		eh_device_open++;
		return (SUCCESS);
	}
}

/**
 * da9052_eh_release:	Function that is called when some one closes EH device node
 * 
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return s32   Error Code, zero: no error
 */
s32 da9052_eh_release(struct inode *inode, struct file *file)
{
	eh_device_open--;
	printk( KERN_INFO "DA9052: EH device closed.\n");
	return (SUCCESS);
}

/**
 * da9052_eh_suspend: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @param state  pm state
 * @return s32 status of suspend operation
 */
static s32 da9052_eh_suspend(struct platform_device *dev, pm_message_t state)
{
	/* Put your suspend related operations here */
	printk(KERN_INFO "%s: called\n", __FUNCTION__);
	return (SUCCESS);
}

/**
 * da9052_eh_resume: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @return s32 status of resume operation
 */
static s32 da9052_eh_resume(struct platform_device *dev)
{
	/* Put your resume related operations here */
	printk(KERN_INFO "%s: called\n", __FUNCTION__);
	return (SUCCESS);
}

/**
 * da9052_eh_ioctl: Function that is called when some one calls ioctl() on SSC device node
 * 
 * @param *inode Inode number of device node
 * @param *file  File pointer
 * @param cmd	ioctl command to be executed
 * @param arg	arguments for cmd
 * @return s32	status of executing cmd
 */
s32 da9052_eh_ioctl(struct inode *inode, struct file *file, 
					unsigned int cmd, unsigned long arg)
{	
	DA9052_DEBUG("IOCTL - %d \n", cmd);
	
	switch(cmd) {
		
		case DA9052_IOCTL_EH_DUMP_TREE:
			DA9052_DEBUG("IOCTL - %d \n", cmd);
			dump_nb_tree();
			break;
		default:
			DA9052_DEBUG("INVALID IOCTL - %d \n", cmd);
			break;
	}
	return (SUCCESS);
}

/**
 * static struct file_operations da9052_eh_fops  
 * This structure definition has to be defined here as an exception. 
 * @owner: Owner of this module
 * @open : open() function for this module
 * @release: release() function for this module
 * @ioctl: ioctl() function for this module
 */
static const struct file_operations da9052_eh_fops = {
        .owner          = THIS_MODULE,
        .open           = da9052_eh_open,
        .release        = da9052_eh_release,
        .ioctl          = da9052_eh_ioctl
};

/**
 * da9052_eh_probe: Called when a device gets attached to driver 
 * 
 * @param *dev   pointer to platform device
 * @return int   Error Code, zero: no error
 */
static s32 __devinit da9052_eh_probe(struct platform_device *dev)
{
	s32 result, cnt;
	
	/* Register the device */
	result = register_chrdev(eh_major_number, 
				 DA9052_EH_DEVICE_NAME, &da9052_eh_fops);
	if(result < 0) {
		printk(KERN_ERR "Unable to register %s\n", 
		       DA9052_EH_DEVICE_NAME);
		return (-EFAULT);	
	} else {
		printk(KERN_INFO "@ %s: Major number is: %d.\n",
		       DA9052_EH_DEVICE_NAME, eh_major_number);
	}
	
	/* Initialize priority map with statically defined priorities */
	da9052_eh_init_eve_prio_map();
	
	/* Initialize eve_nb_array */
	for(cnt = 0; cnt < EVE_CNT; cnt++)
		INIT_LIST_HEAD(&(eve_nb_array[cnt].nb_list));
	
	/* Initialize NB array lock */
	init_MUTEX(&eve_nb_array_lock);

	/* Initialize TSI FIFO lock */
	init_MUTEX(&tsi_fifo_lock);

	/* Setup IRQ line */
	da9052_eh_setup_irq();
	
	/* Set state of TSI state machine */
	tsi_state = WAIT_FOR_PEN_DOWN;
		
	INIT_WORK(&eh_isr_work, eh_workqueue_isr);
	
	if (request_irq(DA9052_IRQ, da9052_eh_isr, IRQ_TYPE_LEVEL_LOW, DA9052_EH_DEVICE_NAME, NULL)) {
		unregister_chrdev(eh_major_number, DA9052_EH_DEVICE_NAME);
		printk(KERN_ERR "Can't register IRQ - %d for da9052_eh_isr!\n", DA9052_IRQ);
		return (-EIO);
	}else{
		printk(KERN_INFO "DA9052 IRQ - %d registered successfully \n", DA9052_IRQ);
	}
	
	return (SUCCESS);
}

/**
 * da9052_eh_remove: Called when detaching device from driver
 * @param *dev   pointer to platform device
 * @return int   Error Code, zero: no error
 */
static int __devexit da9052_eh_remove(struct platform_device *dev)
{
	DA9052_DEBUG(KERN_DEBUG "Removing %s \n", DA9052_EH_DEVICE_NAME);
	return (SUCCESS);
}

/**
 * static struct platform_driver da9052_eh_driver -
 * @probe: Probe function for this device. 
 * @remove: Function to be called when removing this device from platform
 * @suspend: Function to be called in suspend mode
 * @resume: Function to be called in resume mode
 * @driver: Contains glue logic to bind platform device and plarform driver
 */
static struct platform_driver da9052_eh_driver = {
	.probe		= da9052_eh_probe,
	.remove		= __devexit_p(da9052_eh_remove),
	.suspend	= da9052_eh_suspend,
	.resume		= da9052_eh_resume,
	.driver		= {
		.name	= DA9052_EH_DEVICE_NAME,
		.owner	= THIS_MODULE,
	},
};

/**
 * da9052_eh_init:	Function that gets called at the time of module loading
 * @param void
 * @return int status of module initialization
 */
static s32 __init da9052_eh_init(void)
{
	s32 ret;
	
	printk(banner);
	
	da9052_eh_platform_device = platform_device_alloc("da9052_eh", 0);
	if (!da9052_eh_platform_device)
		return (-ENOMEM);

	/* Add platform device to device hierarchy */
	ret = platform_device_add(da9052_eh_platform_device);
	if (ret < 0) {
		/* Free all memory associated with a platform device */
		platform_device_put(da9052_eh_platform_device);
		return (ret);
	}

	ret = platform_driver_register(&da9052_eh_driver);
	if (ret < 0)
		/* Unregister the platform level device */
		platform_device_unregister(da9052_eh_platform_device);

	return (ret);
}

/**
 * da9052_eh_exit:	Function that gets called at the time of module unloading
 * @param void
 * @return void
 */
static void __exit da9052_eh_exit(void)
{
	printk(KERN_INFO "DA9052: Unregistering EH device.\n");
	unregister_chrdev(eh_major_number, DA9052_EH_DEVICE_NAME);
		
	/* Restore IRQ line */
	da9052_eh_restore_irq();
	
	platform_driver_unregister(&da9052_eh_driver);
	
	/* Unregister the platform level device */
	platform_device_unregister(da9052_eh_platform_device);
}

module_init(da9052_eh_init);
module_exit(da9052_eh_exit);

MODULE_AUTHOR("Dialog Semiconductor Ltd");
MODULE_DESCRIPTION("DA9052 EH Device Driver");
MODULE_LICENSE("GPL");

/*--------------------------------------------------------------------------*/
/* Exports                                                                  */
/*--------------------------------------------------------------------------*/
EXPORT_SYMBOL(da9052_eh_register_nb);
EXPORT_SYMBOL(da9052_eh_unregister_nb);
EXPORT_SYMBOL(da9052_tsi_start_sampling);
EXPORT_SYMBOL(da9052_tsi_stop_sampling);
EXPORT_SYMBOL(da9052_tsi_get_data);
