/*
 * da9052_gpio.c : Driver for GPIO peripheral on DA9052 chip.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (05/08/2009):	Initial version
 *
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/platform_device.h>

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_reg.h"
#include "../include/da9052_lib.h"
#include "../include/da9052_eh.h"
#include "../include/da9052_gpio.h"

/*--------------------------------------------------------------------------*/
/* Local Type Definitions                                                   */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Local Constant Definitions                                               */
/*--------------------------------------------------------------------------*/
/* Display message during Initialization */
static u8 banner[] __initdata = "DA9052 GPIO Driver, v1.0\n";
/* Array for registration of the events */
static u8 event_registration[16];
/* typedef of the call back functions */
typedef void ( *gpio_callbacks_t)(u32);
/* Array of the Callback functions */
gpio_callbacks_t gpio_callback[16];


/*--------------------------------------------------------------------------*/
/* Local Macro Definitions                                                  */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/
/* unsigned char to check the device already open */
static u8 gpio_device_open = 0;		
/* GPIO device number */
static s32 major_number = DA9052_GPIO_DEVICE_MAJOR_NO;
/* Structure of async queue */
static struct fasync_struct *gpio_fasync_queue;
/* structure for the device details */
static struct platform_device *da9052_gpio_platform_device;
/* mutex for events */
struct mutex da9052_gpio_event_lock;
/* Stores last occured event */
static u16 gpio_event;

/* array of EH structures */
da9052_eh_nb eh_nb[16];


/*--------------------------------------------------------------------------*/
/* Local Functions                                                          */
/*--------------------------------------------------------------------------*/
/**
 * da9052_gpio_signal_to_user:	This function signal to the library to call 
 *				the callback function for the event.
 * 
 * @param event			event for which library need to be signalled
 * @return int			Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_gpio_signal_to_user(u16 event)
{
	/* Changed the following statement to implement multiple Events */
	/* gpio_event = event; */
	mutex_lock(&da9052_gpio_event_lock);
	gpio_event |= event;
	mutex_unlock(&da9052_gpio_event_lock);
	DA9052_DEBUG("%s: event status: %d \n", __FUNCTION__, event);
	kill_fasync(&gpio_fasync_queue, SIGIO, POLL_IN);

	return(SUCCESS);
}


/**
 * da9052_gpio0_handler : EH callback function for GPI 0 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio0_handler(u32 event)
{
	DA9052_DEBUG("In gpio 0 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_0_EVENT);
}

/**
 * da9052_gpio1_handler : EH callback function for GPI 1 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio1_handler(u32 event)
{
	DA9052_DEBUG("In gpio 1 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_1_EVENT);
}

/**
 * da9052_gpio2_handler : EH callback function for GPI 2 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio2_handler(u32 event)
{
	DA9052_DEBUG("In gpio 2 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_2_EVENT);
}

/**
 * da9052_gpio3_handler : EH callback function for GPI 3 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio3_handler(u32 event)
{
	DA9052_DEBUG("In gpio 3 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_3_EVENT);
}

/**
 * da9052_gpio4_handler : EH callback function for GPI 4 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio4_handler(u32 event)
{
	DA9052_DEBUG("In gpio 4 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_4_EVENT);
}

/**
 * da9052_gpio5_handler : EH callback function for GPI 5 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio5_handler(u32 event)
{
	DA9052_DEBUG("In gpio 5 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_5_EVENT);
}

/**
 * da9052_gpio6_handler : EH callback function for GPI 6 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio6_handler(u32 event)
{
	DA9052_DEBUG("In gpio 6 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_6_EVENT);
}

/**
 * da9052_gpio7_handler : EH callback function for GPI 7 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio7_handler(u32 event)
{
	DA9052_DEBUG("In gpio 7 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_7_EVENT);
}

/**
 * da9052_gpio8_handler : EH callback function for GPI 8 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio8_handler(u32 event)
{
	DA9052_DEBUG("In gpio 8 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_8_EVENT);
}

/**
 * da9052_gpio9_handler : EH callback function for GPI 9 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio9_handler(u32 event)
{
	DA9052_DEBUG("In gpio 9 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_9_EVENT);
}

/**
 * da9052_gpio10_handler : EH callback function for GPI 10 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio10_handler(u32 event)
{
	DA9052_DEBUG("In gpio 10 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_10_EVENT);
}

/**
 * da9052_gpio11_handler : EH callback function for GPI 11 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio11_handler(u32 event)
{
	DA9052_DEBUG("In gpio 11 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_11_EVENT);
}

/**
 * da9052_gpio12_handler : EH callback function for GPI 12 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio12_handler(u32 event)
{
	DA9052_DEBUG("In gpio 12 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_12_EVENT);
}

/**
 * da9052_gpio13_handler : EH callback function for GPI 13 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio13_handler(u32 event)
{
	DA9052_DEBUG("In gpio 13 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_13_EVENT);
}

/**
 * da9052_gpio14_handler : EH callback function for GPI 14 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio14_handler(u32 event)
{
	DA9052_DEBUG("In gpio 14 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_14_EVENT);
}

/**
 * da9052_gpio15_handler : EH callback function for GPI 15 event
 * 
 * @param u32 event	complete event status
 * @return void
 */
void da9052_gpio15_handler(u32 event)
{
	DA9052_DEBUG("In gpio 15 Event: %d\n",event);
	da9052_gpio_signal_to_user(GPI_15_EVENT);
}

/**
 * register_with_eh : Registers with EH for given interrupt
 * 
 * @param is_registered_flag	Tells whether already registered
 * @param eh_data		EH data to be filled up
 * @param event			Event to be registered
 * @param call_back_fn		Callback function for the event
 * @return void
 */
s32 register_with_eh(u8 is_registered_flag, da9052_eh_nb *eh_data, u8 event, 
						void (*call_back_fn)(u32))
{
	/* Set the EH data structure to register GPI event */
	eh_data->eve_type = event;
	eh_data->call_back = call_back_fn;

	/* Register to the EH for the requested event */
	if (SUCCESS == da9052_eh_register_nb(eh_data)) {
		DA9052_DEBUG("\tEvent is registered\n");
	} else {
		return (EVENT_REGISTRATION_FAILED);
	}
	return (SUCCESS);
}

/**
 * create_gpio_config_value: Forms the GPIO value from given configuration
 * 
 * @param gpio_function	GPIO PIN FUNCTION
 * @param gpio_type	GPIO PIN TYPE
 * @param gpio_mode	GPIO PIN MODE
 * @return int   Error Code, zero: no error
 */
u8 create_gpio_config_value(u8 gpio_function, u8 gpio_type, u8 gpio_mode)
{
	/* The format is -
		function - 2 bits 
		type - 1 bit 
		mode - 1 bit */
	return (gpio_function | (gpio_type << 2) | (gpio_mode << 3));
}

/**
 * write_default_gpio_values: Initialize the GPIO registers to default
 * 
 * @param void
 * @return int   Error Code, zero: no error
 */
s32 write_default_gpio_values(void)
{
	/* SSC read/write structure */
	da9052_ssc_msg msg;
	/* receiving formed values */
	u8 created_val = 0;
	/* Default configuration of GPIO 0-15 */
	/* Even GPIOs (e.g. 0,2,4 ..) are placed in lower nibble */
	/* Odd GPIOs (e.g. 1,3,4 ..) are placed in upper nibble */
	
	/* GPIO 0-1*/
	#if (DA9052_GPIO_PIN_0 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0001_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 0 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO0_FUNCTION,
				DEFAULT_GPIO0_TYPE, DEFAULT_GPIO0_MODE);
		msg.data &= MASK_UPPER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 0 write default failed \n");
			return (FAILURE);
		}
	#endif
	#if (DA9052_GPIO_PIN_1 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0001_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 1 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO1_FUNCTION,
				DEFAULT_GPIO1_TYPE, DEFAULT_GPIO1_MODE);
		created_val = created_val << NIBBLE_SHIFT;
		msg.data &= MASK_LOWER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 1 write default failed \n");
			return (FAILURE);
		}
	#endif
	/* GPIO 2-3*/
	#if (DA9052_GPIO_PIN_2 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0203_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 2 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO2_FUNCTION,
				DEFAULT_GPIO2_TYPE, DEFAULT_GPIO2_MODE);
		msg.data &= MASK_UPPER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 2 write default failed \n");
			return (FAILURE);
		}
	#endif
	#if (DA9052_GPIO_PIN_3 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0203_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 3 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO3_FUNCTION,
				DEFAULT_GPIO3_TYPE, DEFAULT_GPIO3_MODE);
		created_val = created_val << NIBBLE_SHIFT;
		msg.data &= MASK_LOWER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 3 write default failed \n");
			return (FAILURE);
		}
	#endif
	/* GPIO 4-5*/
	#if (DA9052_GPIO_PIN_4 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0405_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 4 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO4_FUNCTION,
				DEFAULT_GPIO4_TYPE, DEFAULT_GPIO4_MODE);
		msg.data &= MASK_UPPER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 4 write default failed \n");
			return (FAILURE);
		}
	#endif
	#if (DA9052_GPIO_PIN_5 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0405_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 5 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO5_FUNCTION,
				DEFAULT_GPIO5_TYPE, DEFAULT_GPIO5_MODE);
		created_val = created_val << NIBBLE_SHIFT;
		msg.data &= MASK_LOWER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 5 write default failed \n");
			return (FAILURE);
		}
	#endif
	/* GPIO 6-7*/
	#if (DA9052_GPIO_PIN_6 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0607_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 6 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO6_FUNCTION,
				DEFAULT_GPIO6_TYPE, DEFAULT_GPIO6_MODE);
		msg.data &= MASK_UPPER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 6 write default failed \n");
			return (FAILURE);
		}
	#endif
	#if (DA9052_GPIO_PIN_7 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0607_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 7 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO7_FUNCTION,
				DEFAULT_GPIO7_TYPE, DEFAULT_GPIO7_MODE);
		created_val = created_val << NIBBLE_SHIFT;
		msg.data &= MASK_LOWER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 7 write default failed \n");
			return (FAILURE);
		}
	#endif
	/* GPIO 8-9*/
	#if (DA9052_GPIO_PIN_8 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0809_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 8 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO8_FUNCTION,
				DEFAULT_GPIO8_TYPE, DEFAULT_GPIO8_MODE);
		msg.data &= MASK_UPPER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 8 write default failed \n");
			return (FAILURE);
		}
	#endif
	#if (DA9052_GPIO_PIN_9 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO0809_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 9 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO9_FUNCTION,
				DEFAULT_GPIO9_TYPE, DEFAULT_GPIO9_MODE);
		created_val = created_val << NIBBLE_SHIFT;
		msg.data &= MASK_LOWER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 9 write default failed \n");
			return (FAILURE);
		}
	#endif
	/* GPIO 10-11*/
	#if (DA9052_GPIO_PIN_10 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO1011_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 10 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO10_FUNCTION,
				DEFAULT_GPIO10_TYPE, DEFAULT_GPIO10_MODE);
		msg.data &= MASK_UPPER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 10 write default failed \n");
			return (FAILURE);
		}
	#endif
	#if (DA9052_GPIO_PIN_11 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO1011_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 11 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO11_FUNCTION,
				DEFAULT_GPIO11_TYPE, DEFAULT_GPIO11_MODE);
		created_val = created_val << NIBBLE_SHIFT;
		msg.data &= MASK_LOWER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 11 write default failed \n");
			return (FAILURE);
		}
	#endif
	/* GPIO 12-13*/
	#if (DA9052_GPIO_PIN_12 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO1213_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 12 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO12_FUNCTION,
				DEFAULT_GPIO12_TYPE, DEFAULT_GPIO12_MODE);
		msg.data &= MASK_UPPER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 12 write default failed \n");
			return (FAILURE);
		}
	#endif
	#if (DA9052_GPIO_PIN_13 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO1213_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 13 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO13_FUNCTION,
				DEFAULT_GPIO13_TYPE, DEFAULT_GPIO13_MODE);
		created_val = created_val << NIBBLE_SHIFT;
		msg.data &= MASK_LOWER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 13 write default failed \n");
			return (FAILURE);
		}
	#endif
	/* GPIO 14-15*/
	#if (DA9052_GPIO_PIN_14 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO1415_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 14 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO14_FUNCTION,
				DEFAULT_GPIO14_TYPE, DEFAULT_GPIO14_MODE);
		msg.data &= MASK_UPPER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 14 write default failed \n");
			return (FAILURE);
		}
	#endif
	#if (DA9052_GPIO_PIN_15 == DA9052_GPIO_CONFIG)
		msg.addr = DA9052_GPIO1415_REG;
		msg.data = 0;	
		if (da9052_ssc_read(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 15 read default failed \n");
			return (FAILURE);
		}
		created_val = create_gpio_config_value(DEFAULT_GPIO15_FUNCTION,
				DEFAULT_GPIO15_TYPE, DEFAULT_GPIO15_MODE);
		created_val = created_val << NIBBLE_SHIFT;
		msg.data &= MASK_LOWER_NIBBLE;
		msg.data |= created_val;
		
		if (da9052_ssc_write(&msg) != SUCCESS)	{
			DA9052_DEBUG("GPIO 15 write default failed \n");
			return (FAILURE);
		}
	#endif
	return (SUCCESS);
}

/**
 * da9052_gpio_hw_init: Initializes hardware with default settings
 * 
 * @param void
 * @return s32   Error Code, zero: no error
 */
s32 da9052_gpio_hw_init(void)
{
	/* index of the for loop */
	u8 index = 0;
	/* Write default GPIO Values */
	if (SUCCESS != write_default_gpio_values()) {
		DA9052_DEBUG("GPIO Hardware initialization failed\n");
		return (FAILURE);
	}
	/* Initialize the Callback Function Array  and */
	/* Set EH registration flag to 0 */
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio0_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio1_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio2_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio3_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio4_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio5_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio6_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio7_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio8_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio9_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio10_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio11_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio12_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio13_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio14_handler;
	event_registration[index] = CLEAR;
	gpio_callback[index++] = da9052_gpio15_handler;
		
	/* Initialize mutex required for Gpio Events read */
	mutex_init(&da9052_gpio_event_lock);
	return (SUCCESS);
}

/**
 * da9052_gpio_hw_exit: Un-register events with EH
 * 
 * @param void
 * @return void
 */
static void da9052_gpio_hw_exit(void)
{
	/* unsigned char for int */
	u8 index =0;
	DA9052_DEBUG("I am in function: %s\n", __FUNCTION__);

	/* Before exiting - unregister events with EH */
	/* While exiting set EH registration flag to 0 */
	for (index = 0; index < sizeof(event_registration);index++)
	{
		/* check if the event is registered 
		If registered then unregister the corresponding notifier 
		block */
		if (event_registration[index])
			da9052_eh_unregister_nb(&eh_nb[index]);
		event_registration[index] = CLEAR;
	}
	/* Delete Mutex */
	mutex_destroy(&da9052_gpio_event_lock);
}

/*--------------------------------------------------------------------------*/
/* Global Functions                                                         */
/*--------------------------------------------------------------------------*/

/**
 * da9052_gpio_read_port: Reads the specified GPIO port
 * 
 * @param read_port   GPIO Port to be read
 * @return int   Error Code, zero: no error
 */
s32 da9052_gpio_read_port(da9052_gpio_read_write_t *read_port)
{
	/* SSC struture variable */
	da9052_ssc_msg msg;
	/* u8 for the Shifted 8 bit value */
	u8 shift_value = 0;
	/* variable for the current pin function */
	u8 port_functionality = 0;
	
	/* Find out the register to modify */
	msg.addr = (read_port->port_number / 2) + DA9052_GPIO0001_REG;
	msg.data = 0;
	if (da9052_ssc_read(&msg) != SUCCESS)
		return (FAILURE);
		
	/* Even GPIOs (e.g. 0,2,4 ..) are placed in lower nibble */
	/* Odd GPIOs (e.g. 1,3,4 ..) are placed in upper nibble */
	/* Check whether to read the upper nibble or lower */
	port_functionality = 
			(read_port->port_number % 2) ? 
			((msg.data & ODD_PORT_FUNCTIONALITY) >> NIBBLE_SHIFT):
			(msg.data & EVEN_PORT_FUNCTIONALITY);

	/* Check if the port is not configured for Input functionality */
	if (port_functionality != INPUT)
		return (INVALID_PORTNUMBER);
	
	
	/* Check if Port Number greater than MAX_PORTNUMBER */
	if (read_port->port_number >= (MAX_PORTNUMBER)) {
		return (INVALID_PORTNUMBER);
	}
	/* Check if Port Number greater than the no of ports in the register */
	if (read_port->port_number < MAX_PORTS_PER_REGISTER)
		msg.addr = DA9052_STATUSC_REG;
	else 
	msg.addr = DA9052_STATUSD_REG;
	msg.data = 0;
	
	/* Read the corresponding register through ssc */
	if (SUCCESS != da9052_ssc_read (&msg)) {
		DA9052_DEBUG(" SSC READ FAILED in %s()\n",__FUNCTION__);
		return (FAILURE);
	}
	
	/* Get the value of the required bit corresponding to the port number*/
	shift_value = msg.data & (1 << SHIFT_COUNT(read_port->port_number));
	read_port->read_write_value = (shift_value >> 
					SHIFT_COUNT(read_port->port_number));
	
	return (SUCCESS);
}

/**
 * da9052_gpio_multiple_read: Reads all the GPIO ports
 * 
 * @param multiple_port   Array of Signals to be read
 * @return int   Error Code, zero: no error
 */
s32 da9052_gpio_multiple_read(da9052_gpio_multiple_read_t *multiple_port)
{
	/* SSC data structure Vareiable */
	da9052_ssc_msg msg[2];
	/* Variable to hold the POrt Number */
	u8 port_number =0;
	/* Unsigned Char to check array indexing */
	u8 loop_index = 0;
	
	
	/* Read Status C register */
	msg[loop_index++].addr = DA9052_STATUSC_REG;
	/* Read Status D register */
	msg[loop_index++].addr = DA9052_STATUSD_REG;
	
	/* Use SSC read Many function */
	if (SUCCESS != da9052_ssc_read_many(msg,loop_index)) {
		DA9052_DEBUG(" SSC READ MANY FAILED in %s()\n",__FUNCTION__);
		return (FAILURE);
	}
	/* Return the Status Reg values */
	loop_index = 0;
	for (port_number =0; port_number < MAX_PORTS_PER_REGISTER;port_number++) {
		multiple_port->signal_value[port_number] =
			msg[loop_index].data & 1;
        msg[loop_index].data = msg[loop_index].data >> 1;
	}
	loop_index++;
	for (port_number =MAX_PORTS_PER_REGISTER; port_number < MAX_PORTNUMBER;
		port_number++) {
		multiple_port->signal_value[port_number] =
			msg[loop_index].data & 1;
		msg[loop_index].data = msg[loop_index].data >> 1;
	}
	return (SUCCESS);
}

/**
 * da9052_gpio_write_port: Writes to specified GPIO port
 * 
 * @param write_port   GPIO Port to be written
 * @return int   Error Code, zero: no error
 */
s32 da9052_gpio_write_port(da9052_gpio_read_write_t *write_port)
{
	/*SSC Communication Structure */	
	da9052_ssc_msg msg;
	/* variable to find the Currently used Functionality */
	u8 port_functionality  = 0;
	/*variable to get the Bit position for setting the mode */
	u8 bit_pos = 0;
	
	/* Check if the port is an output port */
	msg.addr = DA9052_GPIO0001_REG + (write_port->port_number / 2);
	msg.data = 0;
	if (da9052_ssc_read (&msg) != SUCCESS) {
		DA9052_DEBUG(" SSC READ FAILED in %s()\n",__FUNCTION__);
		return (FAILURE);
	}
	port_functionality = 
			(write_port->port_number % 2) ? 
			((msg.data & ODD_PORT_FUNCTIONALITY) >> NIBBLE_SHIFT): 
			(msg.data & EVEN_PORT_FUNCTIONALITY);

	if (port_functionality < 2)
		return (INVALID_PORTNUMBER);
	
	/* Bit position is the highest bit in a nibble */
	/* Even GPIOs (e.g. 0,2,4 ..) are placed in lower nibble */
	/* Odd GPIOs (e.g. 1,3,4 ..) are placed in upper nibble */
	bit_pos = (write_port->port_number % 2) ? ODD_PORT_WRITE_MODE : 
				EVEN_PORT_WRITE_MODE;
				
	/* Set the Corresponding Write port signal with the value
		from the user */
	msg.data = write_port->read_write_value ? set_bits(msg.data, bit_pos) :
						clear_bits(msg.data, bit_pos);

	/* Write register */
	if (da9052_ssc_write(&msg) != SUCCESS) {
		return (FAILURE);
	}
	return (SUCCESS);
}

/**
 * da9052_gpio_configure_port: Configures the GPIO port
 * 
 * @param gpio_data   GPIO Port to be configured
 * @return int   Error Code, zero: no error
 */
s32 da9052_gpio_configure_port(da9052_gpio_t *gpio_data)
{
	/* SSC data type variable*/	
	da9052_ssc_msg msg;
	/* variable for the configure value */
	u8 register_value = 0;
	/* variable for the input pin function */
	u8 function = 0; 
	/* variable for the current pin function */
	u8 port_functionality = 0;
	
	/* Find out the register to modify */
	msg.addr = (gpio_data->port_number / 2) + DA9052_GPIO0001_REG;
	msg.data = 0;
	if (da9052_ssc_read(&msg) != SUCCESS)
		return (FAILURE);
		
	/* Even GPIOs (e.g. 0,2,4 ..) are placed in lower nibble */
	/* Odd GPIOs (e.g. 1,3,4 ..) are placed in upper nibble */
	/* Check whether to read the upper nibble or lower */
	port_functionality = 
			(gpio_data->port_number % 2) ? 
			((msg.data & ODD_PORT_FUNCTIONALITY) >> NIBBLE_SHIFT):
			(msg.data & EVEN_PORT_FUNCTIONALITY);

	/* Check if the port is not configured for Altenate functionality */
	if (port_functionality < INPUT)
		return (INVALID_PORTNUMBER);
	/* Check  GPIO type */
	if (gpio_data->gpio_config.input.type > ACTIVE_HIGH)
		return (INVALID_TYPE);
	/* Check Input GPIO mode */
	if (gpio_data->gpio_config.input.mode > DEBOUNCING_ON)
		return (INVALID_MODE);
	/* Check Input GPIO function */
	function = gpio_data->gpio_function;
	switch (function) {
	case INPUT:
		register_value = create_gpio_config_value(function,
					gpio_data->gpio_config.input.type,
					gpio_data->gpio_config.input.mode);
		break;
	case OUTPUT_OPENDRAIN:
	case OUTPUT_PUSHPULL:
		register_value = create_gpio_config_value(function,
					gpio_data->gpio_config.input.type,
					gpio_data->gpio_config.input.mode);
		break;
	default:
		return (DA9052_GPIO_INVALID_FUNCTION);
		break;
	}

	/* Even GPIOs (e.g. 0,2,4 ..) are placed in lower nibble */
	/* Odd GPIOs (e.g. 1,3,4 ..) are placed in upper nibble */
	/* Check whether to modify the upper nibble or lower */
	if (gpio_data->port_number % 2) {
		/* It is upper nibble */
		/* Clear the Upper Nibble and Configure it with the new value */
		msg.data = (clear_bits(msg.data, MASK_UPPER_NIBBLE)) | 
				(register_value << NIBBLE_SHIFT);
	} else {
		/* It is lower nibble */
		/* Clear the lower Nibble and Configure it with the new value */
		msg.data = (clear_bits(msg.data, MASK_LOWER_NIBBLE)) |
				register_value;
	}

	/* Write the register */
	if (da9052_ssc_write(&msg) != SUCCESS) 
		printk ("Write failed in configure\n");
	return (SUCCESS);
}

/**
 * da9052_gpio_register_event : Register gpio event with EH
 * 
 * @param u8 event_type	GPI Event Type
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_gpio_register_event(u8 event_type)
{
	da9052_ssc_msg msg;
	u8 port_functionality = 0;
	s32 ret = 0;
	DA9052_DEBUG("I am in function: %s \n", __FUNCTION__);

	/* Check if the port is an output port by reading the register 
	Contenets*/
	msg.addr = DA9052_GPIO0001_REG + (event_type/2);
	msg.data = 0;
	if (SUCCESS != da9052_ssc_read (&msg)) {
		DA9052_DEBUG(" SSC READ FAILED in %s()\n",__FUNCTION__);
		return (FAILURE);
	}
	port_functionality = 
			(event_type % 2) ? 
			((msg.data & ODD_PORT_FUNCTIONALITY) >> NIBBLE_SHIFT): 
			(msg.data & EVEN_PORT_FUNCTIONALITY);
			
	if (INPUT != port_functionality)
		return (INVALID_PORTNUMBER);
	
	/* Check event type is already registered */
	if (event_registration[event_type])
		return (EVENT_ALREADY_REGISTERED);
	/* IF unregistered register it with EH */
	ret = register_with_eh(event_registration[event_type],
					&eh_nb[event_type],
					GPI0_EVE + event_type,
					gpio_callback[event_type]);
	
	if (SUCCESS == ret) {
		/* Set the event variable to 1 */
		event_registration[event_type] = SET;
	} else {
		/* Return Failure cause */
		return (ret);
	}
	return (SUCCESS);
}

/**
 * da9052_gpio_unregister_event : Un-register gpio event with EH
 * 
 * @param u8 event_type	Event Number
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_gpio_unregister_event(u8 event_type)
{
	DA9052_DEBUG("I am in function: %s \n", __FUNCTION__);
	/* Check if the Event is not Registerd for the Corresponding Pin*/
	if (!event_registration[event_type])
		return (EVENT_UNREGISTERED);
	/* Else Unregister the Evant from EH */
	if (SUCCESS != da9052_eh_unregister_nb(&eh_nb[event_type]))
		return (EVENT_UNREGISTRATION_FAILED);
	/* Set flag to 0 */
	event_registration[event_type] = CLEAR;
	return (SUCCESS);
}

/*--------------------------------------------------------------------------*/
/* Infrastructure Functions						    */
/*--------------------------------------------------------------------------*/

/**
 * da9052_gpio_open: 
 * 
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return int   Error Code, zero: no error
 */
s32 da9052_gpio_open(struct inode *inode, struct file *file)
{
	/* Check if device is already open */
	if(gpio_device_open == OPEN) {
		DA9052_DEBUG("DA9052: gpio device already open.\n");
		return (-EBUSY);
	} else {
		gpio_device_open++;
		return SUCCESS;
	}
}

/**
 * da9052_gpio_release: 
 * 
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return int   Error Code, zero: no error
 */
s32 da9052_gpio_release(struct inode *inode, struct file *file)
{
	gpio_device_open--;
	DA9052_DEBUG("DA9052: GPIO device closed.\n");
	return SUCCESS;
}

/**
 * da9052_gpio_suspend: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @param state  pm state
 * @return 0
 */
static s32 da9052_gpio_suspend(struct platform_device *dev, pm_message_t state)
{
	/* Put your suspend related operations here */
	printk(KERN_INFO "%s: called\n", __FUNCTION__);
    return (0);
}

/**
 * da9052_gpio_resume: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @return 0
 */
static s32 da9052_gpio_resume(struct platform_device *dev)
{
	/* Put your resume related operations here */
	printk(KERN_INFO "%s: called\n", __FUNCTION__);
    return (0);
}

/**
 * da9052_gpio_ioctl: Provides the IOCTL interface
 * 
 * @param *inode   pointer to device inode
 * @param *file   file pointer
 * @param cmd   command to be executed
 * @param arg   argument to command
 * @return int   Error Code, zero: no error
 */
s32 da9052_gpio_ioctl(struct inode *inode, struct file *file,
        unsigned int cmd, unsigned long arg)
{
	da9052_gpio_read_write_t read_write_params;
	da9052_gpio_multiple_read_t multiple_read;
	da9052_gpio_t config_port;
	u8 input;
	u16 copy_event = 0;
	s32 return_value;

	/* Check the IOCTL command */
	switch(cmd) {
	case DA9052_GPIO_IOCTL_READ_PORT:
		 DA9052_DEBUG("%s():DA9052_GPIO_IOCTL_READ_PORT \
		 called\n", __FUNCTION__);
		/* Copy parameters from user space */
		if (copy_from_user(&read_write_params,
					(da9052_gpio_read_write_t *) arg, 
					sizeof(da9052_gpio_read_write_t)))
			return (COPY_FROM_USER_FAILED);
		/* In/Out Parameter - da9052_gpio_read_write_t* */	
		return_value = da9052_gpio_read_port(&read_write_params);
		if (return_value != SUCCESS)
			return(return_value);

		/* Copy parameters to user space */
		if(copy_to_user((da9052_gpio_read_write_t *)arg, 
			&read_write_params, sizeof(da9052_gpio_read_write_t)))
			return (COPY_TO_USER_FAILED);
		break;
	case DA9052_GPIO_IOCTL_WRITE_PORT:
		DA9052_DEBUG("%s():DA9052_GPIO_IOCTL_WRITE_PORT \
		called\n", __FUNCTION__);
		/* Copy parameters from user space */
		if (copy_from_user(&read_write_params,
		(da9052_gpio_read_write_t *) arg,
		sizeof(da9052_gpio_read_write_t)))
			return (COPY_FROM_USER_FAILED);
		/* In Parameter - da9052_gpio_read_write_t* */
		return_value = da9052_gpio_write_port(&read_write_params);
		if (return_value != SUCCESS)
			return (return_value);
		break;
	case DA9052_GPIO_IOCTL_CONFIGURE_PORT:
		DA9052_DEBUG("%s():DA9052_GPIO_IOCTL_CONFIGURE_PORT \
		called\n", __FUNCTION__);
		/* Copy parameters from user space */
		if (copy_from_user( &config_port,
		(da9052_gpio_t *) arg, sizeof(da9052_gpio_t)))
			return(COPY_FROM_USER_FAILED);
		/* In Parameter - da9052_gpio_t* */	
		return_value = da9052_gpio_configure_port(&config_port);
		if (return_value != SUCCESS)
			return (return_value);
		 break;
	case DA9052_GPIO_IOCTL_READ_ALL_PORTS:
		DA9052_DEBUG("%s():DA9052_GPIO_IOCTL_READ_ALL_PORTS \
		called\n", __FUNCTION__);
		/* In/Out Parameter - da9052_gpio_multiple_read_t* */
		return_value = da9052_gpio_multiple_read (&multiple_read);
		if (return_value != SUCCESS)
			return (return_value);

		/* Copy parameters to user space */
		if(copy_to_user((da9052_gpio_multiple_read_t *) arg, 
		& multiple_read, 
		sizeof(da9052_gpio_multiple_read_t)))
			return (COPY_TO_USER_FAILED);
		break;
	case DA9052_GPIO_IOCTL_GET_EVENT:
		/* Out Parameter - unsigned char* */
		/* Acquire the the Mutex */
		mutex_lock(&da9052_gpio_event_lock);
		/* Copy event status into the variable */
		copy_event = gpio_event;
		/* Clear the global variable */
		gpio_event =0;
		/*Unlock the mutex */
		mutex_unlock(&da9052_gpio_event_lock);
		DA9052_DEBUG ("Event = %d\n",copy_event);
		if(copy_to_user((u16 *)arg, &copy_event, sizeof(u16)))
			return (FAILURE);
		break;
	case DA9052_GPIO_IOCTL_REGISTER_EVENT:
		if (copy_from_user(&input, (u8*) arg, sizeof(input)))
			return (FAILURE);
		/* Input Parameter - unsigned char* */
		return (da9052_gpio_register_event(input));
		break;
	case DA9052_GPIO_IOCTL_UNREGISTER_EVENT:
		if (copy_from_user(&input, (u8*) arg, sizeof(input)))
			return (FAILURE);
		/* Input Parameter - unsigned char* */
		return (da9052_gpio_unregister_event(input));
		break;
	default:
		DA9052_DEBUG ("Invalid ioctl command\n");
		return (DA9052_GPIO_INVALID_IOCTL);
		break;		
	}
	return (SUCCESS);
}

/**
 * da9052_gpio_fasync:	Register file descriptor asynchronous notification 
 *			for events
 *
 * @param s32 fd		File descriptor
 * @param strcut file *filp 	File structure
 * @param s32 on		fasync state
 * @return s32			Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_gpio_fasync(s32 fd, struct file *filp, s32 on)
{
	s32 ret;
	DA9052_DEBUG("__%s__\n", __FUNCTION__);
	DA9052_DEBUG ("In %s: %s\n",__FILE__, __FUNCTION__);
	ret = fasync_helper(fd, filp, on, &gpio_fasync_queue);
	DA9052_DEBUG("Ret from fasync_helper = %d \n", ret);
	return (ret);
}

/**
 * static struct file_operations da9052_wl_fops -  
 * This structure definition has to be defined here as an exception. 
 * @owner: member description
 * @open : member description
 * @release: member description
 * @ioctl: member description
 *
 */
static const struct file_operations da9052_gpio_fops = {
        .owner          = THIS_MODULE,
        .open           = da9052_gpio_open,
        .release        = da9052_gpio_release,
        .ioctl          = da9052_gpio_ioctl,
	.fasync		= da9052_gpio_fasync
};

/**
 * da9052_gpio_probe: Called when a device gets attached to driver
 * 
 * @param *dev   pointer to platform device
 * @return int   Error Code, zero: no error
 */
static int __devinit da9052_gpio_probe(struct platform_device *dev)
{
	s32 ret;

	/* Register the device */
	ret = register_chrdev(major_number, DA9052_GPIO_DEVICE_NAME,
							&da9052_gpio_fops);
	if (ret != 0) {
		printk("DA9052: Unable to get major number.\n");
		return (-EFAULT);	
	} else {
		printk(KERN_INFO "%s: Major number is: %d.\n",
				DA9052_GPIO_DEVICE_NAME, major_number);
		
		if (da9052_gpio_hw_init()) {
			printk(KERN_INFO"DA9052: Failed to initialize \
							DA9052 Device.\n");
			unregister_chrdev(major_number, DA9052_GPIO_DEVICE_NAME);
			return (-EFAULT);
		} else
			return (SUCCESS);
	}
}

/**
 * da9052_gpio_remove: Called when detaching device from driver
 * @param *dev   pointer to platform device
 * @return int   Error Code, zero: no error
 */
static int __devexit da9052_gpio_remove(struct platform_device *dev)
{
	DA9052_DEBUG(KERN_DEBUG "Removing %s \n", DA9052_GPIO_DEVICE_NAME);
	return (SUCCESS);
}

/**
 * static struct platform_driver da9052_gpio_driver -
 * This structure definition has to be defined here as an exception.
 * @probe: Probe function for this device.
 * @remove: Function to be called when removing this device from platform
 * @suspend: Function to be called in suspend mode
 * @resume: Function to be called in resume mode
 * @driver: Contains glue logic to bind platform device and plarform driver
 */
static struct platform_driver da9052_gpio_driver = {
	.probe		= da9052_gpio_probe,
	.remove		= __devexit_p(da9052_gpio_remove),
	.suspend	= da9052_gpio_suspend,
	.resume		= da9052_gpio_resume,
	.driver		= {
		.name	= DA9052_GPIO_DEVICE_NAME,
		.owner	= THIS_MODULE,
	},
};

/**
 * da9052_gpio_init: Initiates the driver
 * 
 * @param void
 * @return int   Error Code, zero: no error
 */
static int __init da9052_gpio_init(void)
{
	int retval;
	printk(banner);

	/* Create a platform device object */
	da9052_gpio_platform_device =
			platform_device_alloc(DA9052_GPIO_DEVICE_NAME, 0);
	if (!da9052_gpio_platform_device)
		return (-ENOMEM);

	/* Add platform device to device hierarchy */
	retval = platform_device_add(da9052_gpio_platform_device);
	if (retval < 0) {
		/* Free all memory associated with a platform device */
		platform_device_put(da9052_gpio_platform_device);
		return (retval);
	}

	retval = platform_driver_register(&da9052_gpio_driver);
	if (retval < 0)
		/* Unregister the platform level device */
		platform_device_unregister(da9052_gpio_platform_device);
	return (retval);
}

/**
 * da9052_gpio_exit: Closes the driver
 * 
 * @param void
 * @return void
 */
static void __exit da9052_gpio_exit(void)
{
	printk("DA9052: Unregistering gpio device.\n");

	/* De-initializes */
	da9052_gpio_hw_exit();

	/* Unregister the driver */
	unregister_chrdev(major_number, DA9052_GPIO_DEVICE_NAME);

	platform_driver_unregister(&da9052_gpio_driver);
	/* Unregister the platform level device */
	platform_device_unregister(da9052_gpio_platform_device);
}

module_init(da9052_gpio_init);
module_exit(da9052_gpio_exit);

MODULE_AUTHOR("Dialog Semiconductor Ltd");
MODULE_DESCRIPTION("DA9052 GPIO Device Driver");
MODULE_LICENSE("GPL");
/*--------------------------------------------------------------------------*/
/* Exports                                                                  */
/*--------------------------------------------------------------------------*/

/* All functions accessible from other drivers */
EXPORT_SYMBOL(da9052_gpio_read_port);
EXPORT_SYMBOL(da9052_gpio_write_port);
EXPORT_SYMBOL(da9052_gpio_configure_port);
EXPORT_SYMBOL(da9052_gpio_multiple_read);
/* END OF FILE */
