/*
 * da9052_pm.c: Power Management driver file for DA9052
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 *
 * History:
 *
 * (07/05/2009):	First release Version
 *
 * (23/06/2009):	Added support for multiple events occurring at the same
 *			time
 *
 * (25/06/2009):	Added support to control functions in powerdown mode
 *
 * (29/06/2009):	Implemented review comments:
 *			Used macros, modified function names etc.
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


/*--------------------------------------------------------------------------*/
/* System wide include files						    */
/*--------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/platform_device.h>
/*--------------------------------------------------------------------------*/
/* Module specific include files					    */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_reg.h"
#include "../include/da9052_pm.h"
#include "../include/da9052_eh.h"

/*--------------------------------------------------------------------------*/
/* Local Type Definitions						    */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Local Constant Definitions						    */
/*--------------------------------------------------------------------------*/
static u8 banner[] __initdata = "DA9052 Power Manager Driver, v1.0\n";

/*--------------------------------------------------------------------------*/
/* Local Macro Definitions						    */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Global Variables							    */
/*--------------------------------------------------------------------------*/
static u8 pm_device_open = 0;
static s32 pm_major_number = DA9052_PM_DEVICE_MAJOR_NO;
static struct fasync_struct *pm_fasync_queue;
static struct platform_device *da9052_pm_platform_device;

/* Variable to store the status of event registration with EH */
static pm_event_registration_status event_registered_status;

/* Notifier blocks for individual events that can occur */
static da9052_eh_nb seq_rdy_eh_data;
static da9052_eh_nb nonkey_eh_data;
static da9052_eh_nb id_float_eh_data;
static da9052_eh_nb id_gnd_eh_data;
static da9052_eh_nb gpi8_eh_data;
static da9052_eh_nb gpi9_eh_data;
static da9052_eh_nb gpi10_eh_data;

/* Variable to store status of fault log register */
static u8 fault_log;

/* Variable to store the status of events occured */
static u32 event_occurance_status;

/* Semaphore to access status of events occured
   This semaphore is used at 2 places
   1. in callback from EH when event occurs -
				event status is updated in the callback
   2. in IOCTL when user requestes for status of events occured -
				event status is returned
*/
static struct semaphore event_occurance_status_sem;

/*--------------------------------------------------------------------------*/
/* Local Functions							    */
/*--------------------------------------------------------------------------*/
/**
 * da9052_pm_data_init: Initializes the data with default settings
 *
 * @param void
 * @return int		Error status 0:SUCCESS, Non Zero: Error
 */
static s32 da9052_pm_data_init(void)
{
	/* Mark all events as not occured */
	event_occurance_status = 0;

	/* Initialize event occurance status lock */
	init_MUTEX(&event_occurance_status_sem);

	/* Initialize fault log value */
	fault_log = 0;

	DA9052_DEBUG("Finished initialization\n");
	return (SUCCESS);
}

/**
 * da9052_pm_data_deinit: Performed when PM Driver is unloaded. It unregisters 
 *			registered events from EH
 * 
 * @parasm void
 * @return void
 */
static void da9052_pm_data_deinit(void)
{
	/* Un-register to the EH for all registered event */
	if (event_registered_status.seq_rdy == TRUE) {
		da9052_eh_unregister_nb(&seq_rdy_eh_data);
		event_registered_status.seq_rdy = FALSE;
	}
	if (event_registered_status.nonkey == TRUE) {
		da9052_eh_unregister_nb(&nonkey_eh_data);
		event_registered_status.nonkey = FALSE;
	}
	if (event_registered_status.id_float == TRUE) {
		da9052_eh_unregister_nb(&id_float_eh_data);
		event_registered_status.id_float = FALSE;
	}
	if (event_registered_status.id_gnd == TRUE) {
		da9052_eh_unregister_nb(&id_gnd_eh_data);
		event_registered_status.id_gnd = FALSE;
	}
	if (event_registered_status.gpi8 == TRUE) {
		da9052_eh_unregister_nb(&gpi8_eh_data);
		event_registered_status.gpi8 = FALSE;
	}
	if (event_registered_status.gpi9 == TRUE) {
		da9052_eh_unregister_nb(&gpi9_eh_data);
		event_registered_status.gpi8 = FALSE;
	}
	if (event_registered_status.gpi10 == TRUE) {
		da9052_eh_unregister_nb(&gpi10_eh_data);
		event_registered_status.gpi10 = FALSE;
	}

	DA9052_DEBUG("Finished hardware exit\n");
}

/**
 * da9052_pm_signal_to_user:	This function signal to the library to call 
 *				the call back function for the event.
 * 
 * @param event			event for which library need to be signal.
 * @return int			Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_signal_to_user(u8 event)
{
	/* Acquire semaphore to update event status */
	if (down_interruptible(&event_occurance_status_sem))
		return (EINTR);

	/* Mark the particular event as set */
	event_occurance_status |= event;

	/* Release semaphore */
	up(&event_occurance_status_sem);

	DA9052_DEBUG("I am in function: %s, event :%d \n", __FUNCTION__,event);
	kill_fasync (&pm_fasync_queue, SIGIO, POLL_IN);

	return(SUCCESS);
}

/**
 * da9052_pm_nonkey_handler : EH callback function for nONKEY event
 * 
 * @param u32 event_type	complete event status.
 * @return void
 */
void da9052_pm_nonkey_handler(u32 event_type)
{
	da9052_pm_signal_to_user(DA9052_PM_ONKEY_EVENT);
}

/**
 * da9052_pm_seqrdy_handler : EH callback function for SEQ_RDY event
 * 
 * @param u32 event_type	complete event status.
 * @return void
 */
void da9052_pm_seqrdy_handler(u32 event_type)
{
	DA9052_DEBUG("I am in function: %s, event :%d \n", __FUNCTION__,
								event_type);
	da9052_pm_signal_to_user(DA9052_PM_SEQ_RDY_EVENT);
}

/**
 * da9052_pm_idfloat_handler : EH callback function for ID_FLOAT event
 * 
 * @param u32 event_type	complete event status.
 * @return void
 */
void da9052_pm_idfloat_handler(u32 event_type)
{
	da9052_pm_signal_to_user(DA9052_PM_ID_FLOAT_EVENT);
}

/**
 * da9052_pm_idgnd_handler : EH callback function for ID_GND event
 * 
 * @param u32 event_type	complete event status.
 * @return void
 */
void da9052_pm_idgnd_handler(u32 event_type)
{
	da9052_pm_signal_to_user(DA9052_PM_ID_GND_EVENT);
}

/**
 * da9052_pm_gpi8_handler : EH callback function for GPI_8 event
 * 
 * @param u32 event_type	complete event status.
 * @return void
 */
void da9052_pm_gpi8_handler(u32 event_type)
{
	da9052_pm_signal_to_user(DA9052_PM_GPI8_EVENT);
}

/**
 * da9052_pm_gpi9_handler : EH callback function for GPI_9 event
 * 
 * @param u32 event_type	complete event status.
 * @return void
 */
void da9052_pm_gpi9_handler(u32 event_type)
{
	da9052_pm_signal_to_user(DA9052_PM_GPI9_EVENT);
}

/**
 * da9052_pm_gpi10_handler : EH callback function for GPI_10 event
 * 
 * @param u32 event_type	complete event status.
 * @return void
 */
void da9052_pm_gpi10_handler(u32 event_type)
{
	da9052_pm_signal_to_user(DA9052_PM_GPI10_EVENT);
}
/*--------------------------------------------------------------------------*/
/* Global Functions							    */
/*--------------------------------------------------------------------------*/

/**
 * da9052_pm_configure_buck: Sets the Buck attributes as per input
 * 
 * @param da9052_buck_config buck_config   Buck configuration settings
 * @return int		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_configure_buck(da9052_buck_config buck_config)
{
	da9052_ssc_msg msg;
	u8 buck_volt_conf_reg;
	u8 buck_volt;
	u8 buck_volt_bit;
	u8 buck_conf_bit;
	u8 buck_en_bit;
	u8 buck_curr_conf_reg;
	u8 buck_mode_bit;
	u8 buck_current_bit;
	u8 bperi_hs_bit;
	u8 buck_pd_bit=0;
	u8 shift_by; /* To specify the shift for lower/upper nibble */

	DA9052_DEBUG("I am in function: %s \n", __FUNCTION__);

	msg.addr = DA9052_CONTROLB_REG;
	/* Read CONTROL_B register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Check the buck number */
	switch (buck_config.buck_num) {
	case BUCK_CORE:
		/* Validate the BUCK Voltage configuration */
		if (SUCCESS != (validate_buckpro_mV(buck_config.buck_volt)))
			return (INVALID_BUCKCORE_VOLT_VALUE);
		/* Convert the user configuration to bit value */
		buck_volt=buckpro_mV_to_reg(buck_config.buck_volt);
		/* Assign register and Mask for BUCK Configuration */
		buck_volt_conf_reg = DA9052_BUCKCORE_REG;
		buck_volt_bit = DA9052_BUCKCORE_VBCORE;
		buck_conf_bit = DA9052_BUCKCORE_BCORECONF;
		buck_en_bit = DA9052_BUCKCORE_BCOREEN;
		buck_curr_conf_reg = DA9052_BUCKA_REG;
		buck_mode_bit = DA9052_BUCKA_BCOREMODE;
		buck_current_bit = DA9052_BUCKA_BCOREILIM;
		buck_pd_bit = DA9052_PULLDOWN_COREPDDIS;
		/* Lower nibble of DA9052_BUCKA_REG */
		shift_by = 0;
		break;
	case BUCK_PRO:
		if(msg.data & DA9052_CONTROLB_BUCKMERGE)
			return(INVALID_OPERATION_ON_MERGED_BUCK);
		/* Validate the BUCK Voltage configuration */
		if (SUCCESS != (validate_buckpro_mV(buck_config.buck_volt)))
			return (INVALID_BUCKPRO_VOLT_VALUE);
		/* Convert the user configuration to bit value */
		buck_volt=buckpro_mV_to_reg(buck_config.buck_volt);
		/* Assign register and Mask for BUCK Configuration */
		buck_volt_conf_reg = DA9052_BUCKPRO_REG;
		buck_volt_bit = DA9052_BUCKPRO_VBPRO;
		buck_conf_bit = DA9052_BUCKPRO_BPROCONF;
		buck_en_bit = DA9052_BUCKPRO_BPROEN;
		buck_curr_conf_reg = DA9052_BUCKA_REG;
		buck_mode_bit = DA9052_BUCKA_BPROMODE;
		buck_current_bit = DA9052_BUCKA_BPROILIM;
		buck_pd_bit = DA9052_PULLDOWN_PROPDDIS;
		/* Upper nibble of DA9052_BUCKA_REG */
		shift_by = 4;
		break;
	case BUCK_MEM:
		/* Validate the BUCK Voltage configuration */
		if (SUCCESS != (validate_buckmem_mV(buck_config.buck_volt)))
			return (INVALID_BUCKMEM_VOLT_VALUE);
		/* Convert the user configuration to bit value */
		buck_volt=buckmem_mV_to_reg(buck_config.buck_volt);
		/* Assign register and Mask for BUCK Configuration */
		buck_volt_conf_reg = DA9052_BUCKMEM_REG;
		buck_volt_bit = DA9052_BUCKMEM_VBMEM;
		buck_conf_bit = DA9052_BUCKMEM_BMEMCONF;
		buck_en_bit = DA9052_BUCKMEM_BMEMEN;
		buck_curr_conf_reg = DA9052_BUCKB_REG;
		buck_mode_bit = DA9052_BUCKB_BMEMMODE;
		buck_current_bit = DA9052_BUCKB_BMEMILIM;
		buck_pd_bit = DA9052_PULLDOWN_MEMPDDIS;
		/* Lower nibble of DA9052_BUCKB_REG */
		shift_by = 0;
		break;
	case BUCK_PERI:
		/* Validate the BUCK Voltage configuration */
		if (SUCCESS != (validate_buckperi_mV(buck_config.buck_volt)))
			return (INVALID_BUCKPERI_VOLT_VALUE);
		/* Convert the user configuration to bit value */
		buck_volt=buckperi_mV_to_reg(buck_config.buck_volt);
		/* Assign register and Mask for BUCK Configuration */
		buck_volt_conf_reg = DA9052_BUCKPERI_REG;
		buck_volt_bit = DA9052_BUCKPERI_VBPERI;
		buck_conf_bit = DA9052_BUCKPERI_BPERICONF;
		buck_en_bit = DA9052_BUCKPERI_BPERIEN;
		bperi_hs_bit = DA9052_BUCKPERI_BPERIHS;
		buck_curr_conf_reg = DA9052_BUCKB_REG;
		buck_mode_bit = DA9052_BUCKB_BPERIMODE;
		buck_current_bit = DA9052_BUCKB_BERIILIM;
		/* Upper nibble of DA9052_BUCKB_REG */
		shift_by = 4;
		break;
	default:
		return (INVALID_BUCK_NUM);
		break;
	}

	/* Configure BUCK Voltage, CONF bits */
	msg.addr = buck_volt_conf_reg;
	/* Read register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	msg.data = buck_volt | (buck_config.buck_conf ? buck_conf_bit : 0) |
		(msg.data & buck_en_bit);

	/* For BUCK_PERI, HS bit should be configured */
	if (BUCK_PERI == buck_config.buck_num) {
		msg.data = msg.data | (buck_config.bperi_hs ? bperi_hs_bit : 0);
	}
	/* Write register */
	DA9052_DEBUG("Reg addr :%u    Reg value: %u\n",msg.addr,msg.data);
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	/* Configure BUCK PD bit */
	/* For BUCK_PERI, this is not required */
	if (BUCK_PERI != buck_config.buck_num) {
		msg.addr = DA9052_PULLDOWN_REG;
		/* Read register */
		if (da9052_ssc_read(&msg))
			return (FAILURE);

		msg.data = (buck_config.buck_pd ?
				set_bits(msg.data, buck_pd_bit) :
				clear_bits(msg.data, buck_pd_bit));

		/* Write register */
		if (da9052_ssc_write(&msg))
			return (FAILURE);
	}

	/* Configure BUCK Current, Mode bits */
	msg.addr = buck_curr_conf_reg;
	/* Read register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set CURRENT and MODE bit fields with the folowing steps */
	/* 1. Clear CURRENT and MODE bit fields */
	/* 2. Form the bit field value by shift operation for given value */
	/* Bit position - MODE bit (0, 1), CURRENT bit (2, 3)*/
	msg.data = (clear_bits(msg.data, (buck_current_bit | buck_mode_bit)) |
				(buck_config.current_level <<(shift_by+2)) | 
				(buck_config.buck_mode << shift_by));

	/* Write register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_configure_ldo: Configure LDO base on user inputs
 * 
 * @param da9052_ldo_config ldo_config   LDO configuration settings
 * @return int		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_configure_ldo(da9052_ldo_config ldo_config)
{
	da9052_ssc_msg msg;
	u8 reg_num;
	u8 ldo_volt;
	u8 ldo_volt_bit = 0;
	u8 ldo_conf_bit = 0;
	u8 ldo_en_bit = 0;
	s8 ldo_pd_bit = -1; /* Set to -1 except for LDO1, LDO2 and LDO5 */

	DA9052_DEBUG("I am in function: %s\n", __FUNCTION__);

	/* Check the LDO number */
	switch (ldo_config.ldo_num) {
	case LDO_1:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo1_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO1_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo1_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO1_REG;
		ldo_volt_bit = DA9052_LDO1_VLDO1;
		ldo_conf_bit = DA9052_LDO1_LDO1CONF;
		ldo_en_bit = DA9052_LDO1_LDO1EN;
		ldo_pd_bit = DA9052_PULLDOWN_LDO1PDDIS;
		break;
	case LDO_2:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo2_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO2_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo2_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO2_REG;
		ldo_volt_bit = DA9052_LDO2_VLDO2;
		ldo_conf_bit = DA9052_LDO2_LDO2CONF;
		ldo_en_bit = DA9052_LDO2_LDO2EN;
		ldo_pd_bit = DA9052_PULLDOWN_LDO2PDDIS;
		break;
	case LDO_3:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo34_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO3_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo34_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO3_REG;
		ldo_volt_bit = DA9052_LDO3_VLDO3;
		ldo_conf_bit = DA9052_LDO3_LDO3CONF;
		ldo_en_bit = DA9052_LDO3_LDO3EN;
		break;
	case LDO_4:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo34_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO4_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo34_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO4_REG;
		ldo_volt_bit = DA9052_LDO4_VLDO4;
		ldo_conf_bit = DA9052_LDO4_LDO4CONF;
		ldo_en_bit = DA9052_LDO4_LDO4EN;
		break;
	case LDO_5:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo567810_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO5_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo567810_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO5_REG;
		ldo_volt_bit = DA9052_LDO5_VLDO5;
		ldo_conf_bit = DA9052_LDO5_LDO5CONF;
		ldo_en_bit = DA9052_LDO5_LDO5EN;
		ldo_pd_bit = DA9052_PULLDOWN_LDO5PDDIS;
		break;
	case LDO_6:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo567810_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO6_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo567810_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO6_REG;
		ldo_volt_bit = DA9052_LDO6_VLDO6;
		ldo_conf_bit = DA9052_LDO6_LDO6CONF;
		ldo_en_bit = DA9052_LDO6_LDO6EN;
		break;
	case LDO_7:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo567810_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO7_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo567810_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO7_REG;
		ldo_volt_bit = DA9052_LDO7_VLDO7;
		ldo_conf_bit = DA9052_LDO7_LDO7CONF;
		ldo_en_bit = DA9052_LDO7_LDO7EN;
		break;
	case LDO_8:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo567810_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO8_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo567810_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO8_REG;
		ldo_volt_bit = DA9052_LDO8_VLDO8;
		ldo_conf_bit = DA9052_LDO8_LDO8CONF;
		ldo_en_bit = DA9052_LDO8_LDO8EN;
		break;
	case LDO_9:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo9_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO9_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo9_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO9_REG;
		ldo_volt_bit = DA9052_LDO9_VLDO9;
		ldo_conf_bit = DA9052_LDO9_LDO9CONF;
		ldo_en_bit = DA9052_LDO9_LDO9EN;
		break;
	case LDO_10:
		/* Validate the LDO Voltage configuration */
		if (SUCCESS != (validate_ldo567810_mV(ldo_config.ldo_volt)))
			return (INVALID_LDO10_VOLT_VALUE);
		/* Convert the LDO Voltage configuration to bits value */
		ldo_volt = ldo567810_mV_to_reg(ldo_config.ldo_volt);
		/* Assign register and Mask */
		reg_num = DA9052_LDO10_REG;
		ldo_volt_bit = DA9052_LDO10_VLDO10;
		ldo_conf_bit = DA9052_LDO10_LDO10CONF;
		ldo_en_bit = DA9052_LDO10_LDO10EN;
		break;
	default:
		return (INVALID_LDO_NUM);
		break;
	}

	/* Configure LDO Voltage, CONF bits */
	msg.addr = reg_num;
	/* Read register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set voltage and configuration bits */
	/* Keep the LDO_EN bit setting, as is */
	msg.data = ldo_volt |
		(ldo_config.ldo_conf ? ldo_conf_bit : 0) |
		(msg.data & ldo_en_bit);

	/* Write register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	/* Configure LDO PD bit */
	/* For LDO1, LDO2, LDO5 only */
	if (-1 != ldo_pd_bit) {
		msg.addr = DA9052_PULLDOWN_REG;
		/* Read register */
		if (da9052_ssc_read(&msg))
			return (FAILURE);

		msg.data = (ldo_config.ldo_pd ?
				set_bits(msg.data, ldo_pd_bit) :
				clear_bits(msg.data, ldo_pd_bit));

		/* Write register */
		if (da9052_ssc_write(&msg))
			return (FAILURE);
	}

	return (SUCCESS);
}
/**
 * da9052_pm_go_buck :  BUCK voltage Ramp/Hold at current setting
 * 
 * @param u8 buck_num	BUCK Number
 * @param u8 flag   	Hold/Ramp setting
 * @return int		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_go_buck(u8 buck_num, u8 flag)
{
	da9052_ssc_msg msg;

	DA9052_DEBUG("I am in function: %s\n", __FUNCTION__);

	msg.addr = DA9052_SUPPLY_REG;
	/* Read SUPPLY register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Check the buck number and respectively form the BUCK_GO value*/
	/* Funciton is not aviable for BUCK_PPERI */
	switch (buck_num) {
	case BUCK_CORE:
		msg.data = flag ?
			set_bits(msg.data, DA9052_SUPPLY_VBCOREGO) :
			clear_bits(msg.data, DA9052_SUPPLY_VBCOREGO);
		break;
	case BUCK_PRO:
		msg.data = flag ?
			set_bits(msg.data, DA9052_SUPPLY_VBPROGO) :
			clear_bits(msg.data, DA9052_SUPPLY_VBPROGO);
		break;
	case BUCK_MEM:
		msg.data = flag ?
			set_bits(msg.data, DA9052_SUPPLY_VBMEMGO) :
			clear_bits(msg.data, DA9052_SUPPLY_VBMEMGO);
		break;
	default:
		return (INVALID_BUCK_NUM);
		break;
	}

	/* Write SUPPLY register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_go_ldo : LDO voltage Ramp/Hold at current setting
 * 
 * @param u8 ldo_num	LDO Number
 * @param u8 flag   	Hold/Ramp setting
 * @return int		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_go_ldo(u8 ldo_num, u8 flag)
{
	da9052_ssc_msg msg;

	DA9052_DEBUG("I am in function: %s\n", __FUNCTION__);

	msg.addr = DA9052_SUPPLY_REG;
	/* Read SUPPLY register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Check the buck number and respectively form the BUCK_GO value*/
	/* Funciton is aviable for LDO2 and LDO3 only */
	switch (ldo_num) {
	case LDO_2:
		msg.data = flag ?
			set_bits(msg.data, DA9052_SUPPLY_VLDO2GO) :
			clear_bits(msg.data, DA9052_SUPPLY_VLDO2GO);
		break;
	case LDO_3:
		msg.data = flag ?
			set_bits(msg.data, DA9052_SUPPLY_VLDO3GO) :
			clear_bits(msg.data, DA9052_SUPPLY_VLDO3GO);
		break;
	default:
		return (INVALID_LDO_NUM);
		break;
	}

	/* Write SUPPLY register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_set_ldo : Turn ON/OFF LDO
 * 
 * @param u8 ldo_num	LDO Number
 * @param u8 flag   	turn ON/OFF setting
 * @return int		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_set_ldo(u8 ldo_num, u8 flag)
{
	da9052_ssc_msg msg;
	u8 reg_num = 0;
	u8 value = 0;

	DA9052_DEBUG("I am in function: %s\n", __FUNCTION__);

	/* Check the LDO number and assign register and MASK */
	switch (ldo_num) {
	case LDO_1:
		reg_num = DA9052_LDO1_REG;
		value =	DA9052_LDO1_LDO1EN;
		break;
	case LDO_2:
		reg_num = DA9052_LDO2_REG;
		value =	DA9052_LDO2_LDO2EN;
		break;
	case LDO_3:
		reg_num = DA9052_LDO3_REG;
		value =	DA9052_LDO3_LDO3EN;
		break;
	case LDO_4:
		reg_num = DA9052_LDO4_REG;
		value =	DA9052_LDO4_LDO4EN;
		break;
	case LDO_5:
		reg_num = DA9052_LDO5_REG;
		value =	DA9052_LDO5_LDO5EN;
		break;
	case LDO_6:
		reg_num = DA9052_LDO6_REG;
		value =	DA9052_LDO6_LDO6EN;
		break;
	case LDO_7:
		reg_num = DA9052_LDO7_REG;
		value =	DA9052_LDO7_LDO7EN;
		break;
	case LDO_8:
		reg_num = DA9052_LDO8_REG;
		value =	DA9052_LDO8_LDO8EN;
		break;
	case LDO_9:
		reg_num = DA9052_LDO9_REG;
		value =	DA9052_LDO9_LDO9EN;
		break;
	case LDO_10:
		reg_num = DA9052_LDO10_REG;
		value =	DA9052_LDO10_LDO10EN;
		break;
	default:
		return (INVALID_LDO_NUM);
		break;
	}

	msg.addr = reg_num;
	/* Read LDO register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	msg.data = flag ?
		set_bits(msg.data, value) :
		clear_bits(msg.data, value);
	/* Write LDO register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_set_buck : Turn ON/OFF BUCK
 * 
 * @param u8 buck_num	BUCK Number
 * @param u8 flag   	turn ON/OFF setting
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_set_buck(u8 buck_num, u8 flag)
{
	da9052_ssc_msg msg;
	u8 reg_num=0;
	u8 value=0;

	DA9052_DEBUG("I am in function: %s:\n", __FUNCTION__);

	/* Check the BUCK number and assign register and MASK */
	switch (buck_num) {
	case BUCK_CORE	:
		reg_num = DA9052_BUCKCORE_REG;
		value =	DA9052_BUCKCORE_BCOREEN;
		break;
	case BUCK_PRO:
		reg_num = DA9052_BUCKPRO_REG;
		value =	DA9052_BUCKPRO_BPROEN;
		break;
	case BUCK_MEM:
		reg_num = DA9052_BUCKMEM_REG;
		value =	DA9052_BUCKMEM_BMEMEN;
		break;
	case BUCK_PERI:
		reg_num = DA9052_BUCKPERI_REG;
		value =	DA9052_BUCKPERI_BPERIEN;
		break;
	default:
		return (INVALID_BUCK_NUM);
		break;
	}

	msg.addr = reg_num;
	/* Read BUCK register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);
	msg.data = flag ?
		set_bits(msg.data, value) :
		clear_bits(msg.data, value);

	/* Write BUCK register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_set_buckmerge: Enable/Disables the bcuk merge
 * 
 * @param flag		BUCKCORE and BUCKPRO merging enabled/Disabled
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_set_buckmerge(u8 flag)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_CONTROLB_REG;
	msg.data = 0;
	/* Read CONTROLD register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set/Clear the GPIO_14_14_SD bit */
	msg.data = flag ?
			set_bits(msg.data, DA9052_CONTROLB_BUCKMERGE) :
			clear_bits(msg.data, DA9052_CONTROLB_BUCKMERGE);
	/* Write CONTROLD register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}
/**
 * da9052_pm_set_gpio1415_reset_function: Enable/Disables the GPIO14_15 reset event.
 * 
 * @param flag		GPIO14_15 reset settings
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_set_gpio1415_reset_function(u8 flag)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_CONTROLD_REG;
	msg.data = 0;
	/* Read CONTROLD register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set/Clear the GPIO_14_14_SD bit */
	msg.data = flag ?
			set_bits(msg.data, DA9052_CONTROLD_GPI1415SD) :
			clear_bits(msg.data, DA9052_CONTROLD_GPI1415SD);
	/* Write CONTROLD register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_set_onkey_reset_function: Enable/Disables the nONKEY reset event.
 * 
 * @param flag		nONKEY reset settings
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_set_onkey_reset_function(u8 flag)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_CONTROLD_REG;
	msg.data = 0;
	/* Read CONTROLD register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set/Clear the nONKEY_SD bit */
	msg.data = flag ?
			set_bits(msg.data, DA9052_CONTROLD_NONKEYSD) :
			clear_bits(msg.data, DA9052_CONTROLD_NONKEYSD);
	/* Write CONTROLD register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_set_accdet: Enable/Disables the ACCDET.
 * 
 * @param flag		ACCDET Enable/Disable
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_set_accdet(u8 flag)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_CONTROLD_REG;
	msg.data = 0;
	/* Read CONTROLD register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set/Clear the ACCDETEN bit */
	msg.data = flag ?
			set_bits(msg.data, DA9052_CONTROLD_ACCDETEN) :
			clear_bits(msg.data, DA9052_CONTROLD_ACCDETEN);
	/* Write CONTROLD register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_get_faultlog: Read and clear the fault log register 
 * 
 * @param u8* buffer	Buffer pointer to return the read data.
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_get_faultlog(u8* buffer)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_FAULTLOG_REG;
	msg.data = 0;
	/* Read FAULTLOG register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Check if fault log is not 0 */
	if (msg.data) {
		/* Storing the fault log on first read */
		fault_log = msg.data;

		/* Clear FAULTLOG register */
		msg.data = 0xFF;
		/* Write FAULTLOG register */
		if (da9052_ssc_write(&msg))
			return (FAILURE);
	}

	*buffer = fault_log;
	DA9052_DEBUG("I am in function: %s  faultlog : %u\n", __FUNCTION__,
								*buffer);

	return (SUCCESS);
}

/**
 * da9052_pm_assert_shutdown: Shutdown DA9052 through s/w.
 * 
 * @param void
 * @return s32	Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_assert_shutdown(void)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_CONTROLB_REG;
	msg.data = 0;
	/* Read CONTROLD register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set the SHUTDOWN bit */
	msg.data = set_bits(msg.data, DA9052_CONTROLB_SHUTDOWN);

	/* Write CONTROLB register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_assert_deepsleep: Put DA9052 in deepsleep through s/w.
 * 
 * @param void   
 * @return s32	Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_assert_deepsleep(void)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_CONTROLB_REG;
	msg.data = 0;
	/* Read CONTROLD register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set the DEEPSLEEP bit */
	msg.data = set_bits(msg.data, DA9052_CONTROLB_DEEPSLEEP);

	/* Write CONTROLB register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_vmem_sw: Connect/Disconnect the VMEM_SW.
 * 
 * @param u8 flag	VMEM_SW configuration setting   
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_vmem_sw(u8 flag)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_SUPPLY_REG;
	msg.data = 0;
	/* Read SUPPLY register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set/clear the  VMEMSWEN bit */
	msg.data = flag ? 
		set_bits(msg.data, DA9052_SUPPLY_VMEMSWEN):
		clear_bits(msg.data, DA9052_SUPPLY_VMEMSWEN);

	/* Write SUPPLY register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_vperi_sw: Connect/Disconnect the VPERI_SW.
 * 
 * @param u8 flag	VPERI_SW configuration setting   
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_vperi_sw(u8 flag)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_SUPPLY_REG;
	msg.data = 0;
	/* Read SUPPLY register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set/clear the VPERISWEN bit */
	msg.data = flag ? 
		set_bits(msg.data, DA9052_SUPPLY_VPERISWEN):
		clear_bits(msg.data, DA9052_SUPPLY_VPERISWEN);

	/* Write SUPPLY register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/**
 * da9052_pm_set_lock: lock/Unlock Buck and LDO.
 * 
 * @param u8 flag	V_LOCK configuration setting   
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_set_lock(u8 flag)
{
	da9052_ssc_msg msg;

	msg.addr = DA9052_SUPPLY_REG;
	msg.data = 0;
	/* Read SUPPLY register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Set/clear the VLOCK bit */
	msg.data = flag ? 
		set_bits(msg.data, DA9052_SUPPLY_VLOCK):
		clear_bits(msg.data, DA9052_SUPPLY_VLOCK);

	/* Write SUPPLY register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}


/**
 * da9052_pm_register_event : register PM event with EH
 * 
 * @param u8 event_type	Event Number
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_register_event(u8 event_type)
{

	DA9052_DEBUG("I am in function: %s  event:%d \n", __FUNCTION__,
								event_type);
	/* check event_type and assign callback function for EH */
	switch(event_type) {
	case SEQ_RDY_EVE:
		if (TRUE != event_registered_status.seq_rdy) {
			seq_rdy_eh_data.call_back =da9052_pm_seqrdy_handler;
			seq_rdy_eh_data.eve_type = event_type;
			if(da9052_eh_register_nb(&seq_rdy_eh_data))
				return (FAILURE);
			event_registered_status.seq_rdy = TRUE;
		}
		break;
	case ONKEY_EVE:
		if (TRUE != event_registered_status.nonkey) {
			nonkey_eh_data.call_back =da9052_pm_nonkey_handler;
			nonkey_eh_data.eve_type = event_type;
			if(da9052_eh_register_nb(&nonkey_eh_data))
				return (FAILURE);
			event_registered_status.nonkey = TRUE;
		}
		break;
	case ID_FLOAT_EVE:
		if (TRUE != event_registered_status.id_float) {
			id_float_eh_data.call_back =da9052_pm_idfloat_handler;
			id_float_eh_data.eve_type = event_type;
			if(da9052_eh_register_nb(&id_float_eh_data))
				return (FAILURE);
			event_registered_status.id_float = TRUE;
		}
		break;
	case ID_GND_EVE:
		if (TRUE != event_registered_status.id_gnd) {
			id_gnd_eh_data.call_back =da9052_pm_idgnd_handler;
			id_gnd_eh_data.eve_type = event_type;
			if(da9052_eh_register_nb(&id_gnd_eh_data))
				return (FAILURE);
			event_registered_status.id_gnd = TRUE;
		}
		break;
	case GPI8_EVE:
		if (TRUE != event_registered_status.gpi8) {
			gpi8_eh_data.call_back =da9052_pm_gpi8_handler;
			gpi8_eh_data.eve_type = event_type;
			if(da9052_eh_register_nb(&gpi8_eh_data))
				return (FAILURE);
			event_registered_status.gpi8 = TRUE;
		}
		break;
	case GPI9_EVE:
		if (TRUE != event_registered_status.gpi9) {
			gpi9_eh_data.call_back =da9052_pm_gpi9_handler;
			gpi9_eh_data.eve_type = event_type;
			if(da9052_eh_register_nb(&gpi9_eh_data))
				return (FAILURE);
			event_registered_status.gpi8 = TRUE;
		}
		break;
	case GPI10_EVE:
		if (TRUE != event_registered_status.gpi10) {
			gpi10_eh_data.call_back =da9052_pm_gpi10_handler;
			gpi10_eh_data.eve_type = event_type;
			if(da9052_eh_register_nb(&gpi10_eh_data))
				return (FAILURE);
			event_registered_status.gpi10 = TRUE;
		}
		break;
	default:
		return(INVALID_PM_EVENT);
	}
	
	return (SUCCESS);
}

/**
 * da9052_pm_unregister_event : Un-register PM event with EH
 * 
 * @param u8 event_type	Event Number
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_unregister_event(u8 event_type)
{
	DA9052_DEBUG("I am in function: %s \n", __FUNCTION__);
	
	/* Un-register to the EH for the requested event */
	switch(event_type) {
	case SEQ_RDY_EVE:
		if (event_registered_status.seq_rdy == TRUE) {
			if(da9052_eh_unregister_nb(&seq_rdy_eh_data))
				return (FAILURE);
			event_registered_status.seq_rdy = FALSE;
		}
		break;
	case ONKEY_EVE:
		if (event_registered_status.nonkey == TRUE) {
			if(da9052_eh_unregister_nb(&nonkey_eh_data))
				return (FAILURE);
			event_registered_status.nonkey = FALSE;
		}
		break;
	case ID_FLOAT_EVE:
		if (event_registered_status.id_float == TRUE) {
			if(da9052_eh_unregister_nb(&id_float_eh_data))
				return (FAILURE);
			event_registered_status.id_float = FALSE;
		}
		break;
	case ID_GND_EVE:
		if (event_registered_status.id_gnd == TRUE) {
			if(da9052_eh_unregister_nb(&id_gnd_eh_data))
				return (FAILURE);
			event_registered_status.id_gnd = FALSE;
		}
		break;
	case GPI8_EVE:
		if (event_registered_status.gpi8 == TRUE) {
			if(da9052_eh_unregister_nb(&gpi8_eh_data))
				return (FAILURE);
			event_registered_status.gpi8 = FALSE;
		}
		break;
	case GPI9_EVE:
		if (event_registered_status.gpi9 == TRUE) {
			if(da9052_eh_unregister_nb(&gpi9_eh_data))
				return (FAILURE);
			event_registered_status.gpi8 = FALSE;
		}
		break;
	case GPI10_EVE:
		if (event_registered_status.gpi10 == TRUE) {
			if(da9052_eh_unregister_nb(&gpi10_eh_data))
				return (FAILURE);
			event_registered_status.gpi10 = FALSE;
		}
		break;
	default:
		return(INVALID_PM_EVENT);
	}

	return (SUCCESS);
}

/**
 * da9052_pm_function_ctrl_in_powerdown: 
 *		Enable/Disable given function in power down mode
 *			0 : Enable, 1 : Disable
 * 
 * @param pd_func	Function to be controlled
 * @param is_disable	Enable/disable settings for a function
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_function_ctrl_in_powerdown(u8 pd_func, u8 is_disable)
{
	da9052_ssc_msg msg;
	u8 bit_pos = 0;

	switch (pd_func) {
	case DA9052_GPIO_PD: /* Check GPIO extender function */
		bit_pos = DA9052_PDDIS_GPIOPD;
		break;
	case DA9052_ADC_TSI_PD: /* Check ADC/TSI measurement function */
		bit_pos = DA9052_PDDIS_GPADCPD;
		break;
	case DA9052_PM_IF_PD: /* Check power manager interface function */
		bit_pos = DA9052_PDDIS_PMIFPD;
		break;
	case DA9052_HS_2_WIRE_PD: /* Check HS-2-WIRE function */
		bit_pos = DA9052_PDDIS_HS2WIREPD;
		break;
	case DA9052_BAT_CHG_PD: /* Check battery charger function */
		bit_pos = DA9052_PDDIS_CHGPD;
		break;
	case DA9052_BKUP_BAT_CHG_PD: /* Check backup battery charger function */
		bit_pos = DA9052_PDDIS_CHGBBATPD;
		break;
	case DA9052_OUT_32K_PD: /* Check OUT_32K function */
		bit_pos = DA9052_PDDIS_OUT32KPD;
		break;
	case DA9052_PM_CONT_PD: /* Check SYS_EN, PWR_EN, PWR1_EN function */
		bit_pos = DA9052_PDDIS_PMCONTPD;
		break;
	default:
		return (INVALID_PD_FUNCTION);
		break;
	}

	/* Set SSC message */
	msg.addr = DA9052_PDDIS_REG;
	msg.data = 0;

	/* Read PD_DIS register */
	if (da9052_ssc_read(&msg))
		return (FAILURE);

	/* Check if function to be disabled */
	/* Set 1: To disable, 0: To enable */
	msg.data = is_disable ? set_bits(msg.data, bit_pos) :
				clear_bits(msg.data, bit_pos);

	/* Write PD_DIS register */
	if (da9052_ssc_write(&msg))
		return (FAILURE);

	return (SUCCESS);
}

/*--------------------------------------------------------------------------*/
/* Infrastructure Functions						    */
/*--------------------------------------------------------------------------*/
/**
 * da9052_pm_open: Opens the device
 * 
 * @param *inode   	pointer to device inode
 * @param *file   	file pointer
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_open(struct inode *inode, struct file *file)
{
	/* Check if device is already open */
	DA9052_DEBUG("__%s__\n", __FUNCTION__);
	if (pm_device_open) {
		DA9052_DEBUG("DA9052: Power Manager device already open.\n");
		return (-EBUSY);
	} else {
		pm_device_open++;
		return (SUCCESS);
	}
}

/**
 * da9052_pm_release: Releases the device
 * 
 * @param *inode   	pointer to device inode
 * @param *file   	file pointer
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_release(struct inode *inode, struct file *file)
{
	pm_device_open--;
	DA9052_DEBUG("DA9052: Power Manager device closed.\n");
	return (SUCCESS);
}

/**
 * da9052_pm_suspend: Power Management support function
 * 
 * @param *dev   	pointer to platform device
 * @param state  	pm state
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
static s32 da9052_pm_suspend(struct platform_device *dev, pm_message_t state)
{
	/* Put your suspend related operations here */
	printk(KERN_INFO "%s: called\n", __FUNCTION__);
	return (0);
}

/**
 * da9052_pm_resume: Power Management support function
 * 
 * @param *dev   	pointer to platform device
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
static s32 da9052_pm_resume(struct platform_device *dev)
{
	/* Put your resume related operations here */
	printk(KERN_INFO "%s: called\n", __FUNCTION__);
	return (0);
}

/**
 * da9052_pm_ioctl: Provides the IOCTL interface
 * 
 * @param *inode   	pointer to device inode
 * @param *file   	file pointer
 * @param cmd   	command to be executed
 * @param arg   	argument to command
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_ioctl(struct inode *inode, struct file *file, 
					unsigned int cmd, unsigned long arg)
{
	da9052_buck_config usr_param_buck_config;
	da9052_ldo_config usr_param_ldo_config;
	da9052_ldo_buck_go usr_param_go;
	da9052_ldo_buck_control usr_param_cntr;
	u8 buffer;
	u32 ret = 0;
	da9052_func_ctrl_in_powerdown pd_dis_ctrl;

	/* Temporary Variable to store the status of events occured */
	u32 temp_event_occurance_status;

	DA9052_DEBUG(KERN_INFO "DA9052: Power Manager device IOCTL.\n");

	/* Check command to be executed */
	switch (cmd) {
	case DA9052_PM_IOCTL_CONFIGURE_BUCK:
		if (copy_from_user(&usr_param_buck_config, 
					(da9052_buck_config *) arg, 
					sizeof(usr_param_buck_config)))
			return (FAILURE);
		ret = da9052_pm_configure_buck(usr_param_buck_config);
		return (ret);
		break;
	case DA9052_PM_IOCTL_CONFIGURE_LDO:
		if (copy_from_user(&usr_param_ldo_config, 
					(da9052_ldo_config *) arg, 
					sizeof(usr_param_ldo_config)))
			return (FAILURE);
		ret = da9052_pm_configure_ldo(usr_param_ldo_config);
			return (ret);
		break;
	case DA9052_PM_IOCTL_BUCKMERGE:
		if (copy_from_user(&buffer, (u8*) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_set_buckmerge(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_GO_LDO:
		if (copy_from_user(&usr_param_go, 
			(da9052_ldo_buck_go *) arg, sizeof(usr_param_go)))
			return (FAILURE);
		ret = da9052_pm_go_ldo(usr_param_go.ldo_buck_num,
							usr_param_go.flag);
			return (ret);
		break;
	case DA9052_PM_IOCTL_GO_BUCK:
		if (copy_from_user(&usr_param_go, 
		(da9052_ldo_buck_go *) arg, sizeof(usr_param_go)))
			return (FAILURE);
		ret = da9052_pm_go_buck(usr_param_go.ldo_buck_num,
							usr_param_go.flag);
			return (ret);
		break;
	case DA9052_PM_IOCTL_SET_LDO:
		if (copy_from_user(&usr_param_cntr, 
			(da9052_ldo_buck_control *) arg, sizeof(usr_param_cntr)))
			return (FAILURE);
		
		ret = da9052_pm_set_ldo(usr_param_cntr.ldo_buck_num,
							usr_param_cntr.flag);
			return(ret);
		break;
	case DA9052_PM_IOCTL_SET_BUCK:
		if (copy_from_user(&usr_param_cntr, (da9052_ldo_buck_control *)
						arg, sizeof(usr_param_cntr)))
			return (FAILURE);
		ret = da9052_pm_set_buck(usr_param_cntr.ldo_buck_num,
							usr_param_cntr.flag);
			return(ret);
		break;
	case DA9052_PM_IOCTL_SET_GPIO1415_RESET:
		if (copy_from_user(&buffer, (u8*) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_set_gpio1415_reset_function(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_SET_ONKEY_RESET:
		if (copy_from_user(&buffer, (u8 *) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_set_onkey_reset_function(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_SET_ACCDET:
		if (copy_from_user(&buffer, (u8 *) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_set_accdet(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_GET_FAULTLOG:
		ret = da9052_pm_get_faultlog(&buffer);
		DA9052_DEBUG("\nFault log in IOCTL : %x\n",buffer);
		if (copy_to_user((u8 *)arg,&buffer,sizeof(buffer)))
			return (FAILURE);
		return(ret);
		break;
	case DA9052_PM_IOCTL_ASSERT_SHUTDOWN:
		ret = da9052_pm_assert_shutdown();
			return(ret);
		break;
	case DA9052_PM_IOCTL_ASSERT_DEEPSLEEP:
		ret = da9052_pm_assert_deepsleep();
			return(ret);
		break;
	case DA9052_PM_IOCTL_VMEM_SW:
		if (copy_from_user(&buffer, (u8*) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_vmem_sw(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_VPERI_SW:
		if (copy_from_user(&buffer, (u8*) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_vperi_sw(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_SET_LOCK:
		if (copy_from_user(&buffer, (u8*) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_set_lock(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_GET_EVENT:
		/* Acquire semaphore to update event status */
		if (down_interruptible(&event_occurance_status_sem))
			return (FAILURE);
		/* Copy the status of event occurance into a temp variable */
		temp_event_occurance_status = event_occurance_status;
		/* Clear the event occurance status */
		event_occurance_status = 0;
		/* Release semaphore */
		up(&event_occurance_status_sem);

		/* Copy the status to user space */
		if(copy_to_user((u32 *)arg, &temp_event_occurance_status, 
							sizeof(u32)))
			return (FAILURE);
		return (SUCCESS);
		break;
	case DA9052_PM_IOCTL_REGISTER_EVENT:
		if (copy_from_user(&buffer, (u8*) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_register_event(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_UNREGISTER_EVENT:
		if (copy_from_user(&buffer, (u8*) arg, sizeof(buffer)))
			return (FAILURE);
		ret = da9052_pm_unregister_event(buffer);
			return(ret);
		break;
	case DA9052_PM_IOCTL_FUNC_CTRL_IN_POWERDOWN:
		if (copy_from_user(&pd_dis_ctrl, 
			(da9052_func_ctrl_in_powerdown*) arg, 
			sizeof(da9052_func_ctrl_in_powerdown)))
			return (FAILURE);
		ret = da9052_pm_function_ctrl_in_powerdown(pd_dis_ctrl.pd_func,
						pd_dis_ctrl.is_disable);
			return(ret);
		break;
	default:
		printk(KERN_DEBUG "Invalid ioctl command\n");
		return (INVALID_PM_IOCTL);
	}
	return (INVALID_PM_IOCTL);
}
/**
 * da9052_pm_fasync:	Register file descriptor asynchronous notification 
 *			for events
 *
 * @param s32 fd		File descriptor
 * @param strcut file *filp 	File structure
 * @param s32 on		fasync state
 * @return s32			Error status 0:SUCCESS, Non Zero: Error
 */
s32 da9052_pm_fasync (s32 fd, struct file *filp, s32 on)
{
	s32 ret;
 	DA9052_DEBUG ("In %s: %s\n",__FILE__, __FUNCTION__);
	ret = fasync_helper(fd, filp, on, &pm_fasync_queue);
  	return(ret);
}
/**
 * static struct file_operations da9052_pm_fops -
 * This structure definition has to be defined here as an exception.
 * @owner: 
 * @open : pointer to function providing "open" interface
 * @release: pointer to function providing "release" interface
 * @read: pointer to function providing "read" interface
 * @write: pointer to function providing "write" interface
 * @ioctl: pointer to function providing "ioctl" interface
 *
 */
static const struct file_operations da9052_pm_fops = {
	.owner		= THIS_MODULE,
	.open		= da9052_pm_open,
	.release	= da9052_pm_release,
	.ioctl		= da9052_pm_ioctl,
	.fasync		= da9052_pm_fasync
};

/**
 * da9052_pm_probe:	Called when a device gets attached to driver 
 *
 * @param struct platform_device *dev	platform device to be probe
 * @return s32				Error status 0:SUCCESS, Non Zero: Error
 */
static s32 __devinit da9052_pm_probe(struct platform_device *dev)
{
	s32 ret;
	
	/* Register the device */
	ret = register_chrdev(pm_major_number,
				DA9052_PM_DEVICE_NAME, &da9052_pm_fops);
	if (ret != 0) {
		DA9052_DEBUG("Unable to register %s\n",
						DA9052_PM_DEVICE_NAME);
		return (-EFAULT);	
	} else {
		DA9052_DEBUG("%s: Major number is: %d.\n",
				DA9052_PM_DEVICE_NAME, pm_major_number);

		/* Initialize the hardware */
		if (da9052_pm_data_init()) {
			/* Error in hw initialization */
			unregister_chrdev(pm_major_number,
							DA9052_PM_DEVICE_NAME);
			return (-EFAULT);
		}
		else
			return (SUCCESS);
	}
}


/**
 * da9052_pm_remove: Called when ditaching device from driver
 *
 * @param struct platform_device *dev	platform device to be removed
 * @return x32				Error status 0:SUCCESS, Non Zero: Error
 */
static s32 __devexit da9052_pm_remove(struct platform_device *dev)
{
	DA9052_DEBUG("Removing %s \n", DA9052_PM_DEVICE_NAME);
	return (SUCCESS);
}
/**
 * static struct platform_driver da9052_pm_driver -
 * This structure definition has to be defined here as an exception.
 * @probe: Probe function for this device. 
 * @suspend: Function to be called when suspending this device from platform
 * @remove: Function to be called when resuming this device from platform
 * @driver: Contains glue logic to bind platform device and plarform driver
 */
static struct platform_driver da9052_pm_driver = {
	.probe		= da9052_pm_probe,
	.remove		= __devexit_p(da9052_pm_remove),
	.suspend	= da9052_pm_suspend,
	.resume		= da9052_pm_resume,
	.driver		= {
		.name	= DA9052_PM_DEVICE_NAME,
		.owner	= THIS_MODULE,
	},
};

/**
 * da9052_pm_init: Initiales the driver
 * 
 * @param void
 * @return s32		Error status 0:SUCCESS, Non Zero: Error
 */
static s32 __init da9052_pm_init(void)
{
	s32 retval;
	printk(banner);

	DA9052_DEBUG("DA9052: init : %d.\n",DA9052_PM_DEVICE_MAJOR_NO);
	da9052_pm_platform_device = platform_device_alloc("da9052_pm", 0);
	if (!da9052_pm_platform_device)
		return (-ENOMEM);

	retval = platform_device_add(da9052_pm_platform_device);
	if (retval < 0) {
		platform_device_put(da9052_pm_platform_device);
		return (retval);
	}

	retval = platform_driver_register(&da9052_pm_driver);
	if (retval < 0)
		platform_device_unregister(da9052_pm_platform_device);

	return (retval);
}

/**
 * da9052_pm_exit: Closes the driver
 * 
 * @param void
 * @return void
 */
static void __exit da9052_pm_exit(void)
{
	printk("DA9052: Unregistering Power Manager device.\n");

	/* De-initializes */
	da9052_pm_data_deinit();

	/* Unregister the driver */
	unregister_chrdev(pm_major_number, DA9052_PM_DEVICE_NAME);
	platform_driver_unregister(&da9052_pm_driver);
	platform_device_unregister(da9052_pm_platform_device);
}

module_init(da9052_pm_init);
module_exit(da9052_pm_exit);

MODULE_AUTHOR("Dialog Semiconductor Ltd");
MODULE_DESCRIPTION("DA9052 Power Manager Device Driver");
MODULE_LICENSE("GPL");

/*--------------------------------------------------------------------------*/
/* Exports								    */
/*--------------------------------------------------------------------------*/

/* All functions accessible from other drivers */
EXPORT_SYMBOL(da9052_pm_configure_buck);
EXPORT_SYMBOL(da9052_pm_configure_ldo);
EXPORT_SYMBOL(da9052_pm_set_buckmerge);
EXPORT_SYMBOL(da9052_pm_go_buck);
EXPORT_SYMBOL(da9052_pm_go_ldo);
EXPORT_SYMBOL(da9052_pm_set_ldo);
EXPORT_SYMBOL(da9052_pm_set_buck);
EXPORT_SYMBOL(da9052_pm_set_gpio1415_reset_function);
EXPORT_SYMBOL(da9052_pm_set_onkey_reset_function);
EXPORT_SYMBOL(da9052_pm_set_accdet);
EXPORT_SYMBOL(da9052_pm_get_faultlog);
EXPORT_SYMBOL(da9052_pm_assert_shutdown);
EXPORT_SYMBOL(da9052_pm_assert_deepsleep);
EXPORT_SYMBOL(da9052_pm_vmem_sw);
EXPORT_SYMBOL(da9052_pm_vperi_sw);
EXPORT_SYMBOL(da9052_pm_set_lock);
EXPORT_SYMBOL(da9052_pm_function_ctrl_in_powerdown);
