/*
 * da9052_ssc.c: SSC (Synchronous Serial Communication) driver for DA9052
 *
 * This driver provides DA9052 specific wrappers around SPI and I2C host -
 * drivers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * 05/05/2009:	Initial version.
 *		Curent version only supports SPI and I2C communication. 
 *		Hooks to underlying host SPI driver are very specific to -
 *		SMDK-6410 host driver and some what non standard.
 *
 * 27/06/2009:	Added caching feature for DA9052 Non-volatile registers
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/device.h>
#include <linux/jiffies.h>
#include <linux/platform_device.h>

/* SPI specific include files */
#include <linux/spi/spi.h>

/* I2C specific include files */
#include <linux/i2c.h>

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_reg.h"
#include "../include/da9052_lib.h"
#include "../include/da9052_ssc.h"

/*--------------------------------------------------------------------------*/
/* Local Type Definitions                                                   */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Local Constant Definitions                                               */
/*--------------------------------------------------------------------------*/
static u8 banner[] __initdata = "DA9052 SSC Driver, v1.0\n";

/*--------------------------------------------------------------------------*/
/* Local Macro Definitions                                                  */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/

/* To prevent multiple opens */
u8 ssc_device_open = 0;

/* Major number for SSC device */
s32 ssc_major_number = DA9052_SSC_DEVICE_MAJOR_NO;

/* ssc lock */
struct semaphore	ssc_sem;

/* 
 * SSC cache for non-volatile registers.
 * Each entry has three parameters for each register of DA9052
 * (1) Type of register i.e. Volatile or Non-volatile
 * (2) Status of current cached value - Valid/Invalid
 * (3) Value of register in case if it is non-volatile
 *
 * Volatile Register: A regsiter whose value can be updated by hardware. This -
 * type of registers must be read every time from hardware and can't be cached
 *
 * Non-volatile Register: A register whose value can never be updated by -
 * hardware automatically. i.e. the value gets changed only after a write -
 * operation is performed. This type of registers can be cached. 
 */
da9052_ssc_cache_entry	ssc_cache[DA9052_REG_CNT];

/* Interface specific stuff */

#if DA9052_SSC_CONFIG_SPI
/* Curretn active page */
u32 spi_active_page = PAGECON_0;

/*
 * Value of R/W_POL bit of INTERFACE REG (R19) of DA9052. Here we have used it's
 * defaule value. If you change it through OTP or some other way, you need to 
 * set this variable accordingly
 */
u8 rw_pol = 1;

/* Handle to global spi_dev */
struct spi_device *spi_dev;
	
/* Buffers used for single register read/write functions */
u8* spi_rx_buf;
u8* spi_tx_buf;

#elif DA9052_SSC_CONFIG_I2C

/* Handle for i2c client */
struct i2c_client	*da9052_i2c_client;

#endif		/* DA9052_SSC_CONFIG_I2C */

/* Platform device structure for SSC */
static struct platform_device *da9052_ssc_platform_device;

/*--------------------------------------------------------------------------*/
/* Local Functions                                                          */
/*--------------------------------------------------------------------------*/

/**
 * da9052_init_ssc_cache: Initializes SSC regsiter cache
 * Here we define which DA9052 regiters are Volatile and which are not
 * Current definitions are according to 14th May 2009 data sheet of DA9052.
 * This also sets cache status as invalid for all entries to enable reading -
 * values from hardware for first read request.
 * @param
 * @return void
 */
static void da9052_init_ssc_cache(void)
{
	u8 cnt;
	
	/* First initialize all registers as Non-volatile */
	for(cnt = 0; cnt < DA9052_REG_CNT; cnt++){
		ssc_cache[cnt].type = NON_VOLATILE;
		ssc_cache[cnt].status = INVALID;
		ssc_cache[cnt].val = 0;
	}
	
	/* Now selectively set type for all Volatile registers */
	ssc_cache[DA9052_STATUSA_REG].type	= VOLATILE;	// Reg - 1
	ssc_cache[DA9052_STATUSB_REG].type	= VOLATILE;	// Reg - 2
	ssc_cache[DA9052_STATUSC_REG].type	= VOLATILE;	// Reg - 3
	ssc_cache[DA9052_STATUSD_REG].type	= VOLATILE;	// Reg - 4
	ssc_cache[DA9052_EVENTA_REG].type	= VOLATILE;	// Reg - 5
	ssc_cache[DA9052_EVENTB_REG].type	= VOLATILE;	// Reg - 6
	ssc_cache[DA9052_EVENTC_REG].type	= VOLATILE;	// Reg - 7
	ssc_cache[DA9052_EVENTD_REG].type	= VOLATILE;	// Reg - 8
	ssc_cache[DA9052_FAULTLOG_REG].type	= VOLATILE;	// Reg - 9

	ssc_cache[DA9052_CONTROLB_REG].type	= VOLATILE;	// Reg - 15

	ssc_cache[DA9052_CONTROLD_REG].type	= VOLATILE;	// Reg - 17

	ssc_cache[DA9052_SUPPLY_REG].type	= VOLATILE;	// Reg - 60

	ssc_cache[DA9052_CHGBUCK_REG].type	= VOLATILE;	// Reg - 62

	ssc_cache[DA9052_INPUTCONT_REG].type	= VOLATILE;	// Reg - 67
	ssc_cache[DA9052_CHGTIME_REG].type	= VOLATILE;	// Reg - 68

	ssc_cache[DA9052_BOOST_REG].type	= VOLATILE;	// Reg - 70

	ssc_cache[DA9052_ADCMAN_REG].type	= VOLATILE;	// Reg - 81

	ssc_cache[DA9052_ADCRESL_REG].type	= VOLATILE;	// Reg - 83
	ssc_cache[DA9052_ADCRESH_REG].type	= VOLATILE;	// Reg - 84
	ssc_cache[DA9052_VDDRES_REG].type	= VOLATILE;	// Reg - 85

	ssc_cache[DA9052_ICHGAV_REG].type	= VOLATILE;	// Reg - 87

	ssc_cache[DA9052_TBATRES_REG].type	= VOLATILE;	// Reg - 90

	ssc_cache[DA9052_ADCIN4RES_REG].type	= VOLATILE;	// Reg - 95

	ssc_cache[DA9052_ADCIN5RES_REG].type	= VOLATILE;	// Reg - 98

	ssc_cache[DA9052_ADCIN6RES_REG].type	= VOLATILE;	// Reg - 101

	ssc_cache[DA9052_TJUNCRES_REG].type	= VOLATILE;	// Reg - 104

	ssc_cache[DA9052_TSICONTB_REG].type	= VOLATILE;	// Reg - 106
	ssc_cache[DA9052_TSIXMSB_REG].type	= VOLATILE;	// Reg - 107
	ssc_cache[DA9052_TSIYMSB_REG].type	= VOLATILE;	// Reg - 108
	ssc_cache[DA9052_TSILSB_REG].type	= VOLATILE;	// Reg - 109
	ssc_cache[DA9052_TSIZMSB_REG].type	= VOLATILE;	// Reg - 110
	
	/* Uncomment follwoing lines to have full READ/WRITE test */
	//ssc_cache[102].type 			= VOLATILE;
	//ssc_cache[103].type 			= VOLATILE;
	
	ssc_cache[DA9052_COUNTS_REG].type	= VOLATILE;	// Reg - 111
	ssc_cache[DA9052_COUNTMI_REG].type	= VOLATILE;	// Reg - 112
	ssc_cache[DA9052_COUNTH_REG].type	= VOLATILE;	// Reg - 113
	ssc_cache[DA9052_COUNTD_REG].type	= VOLATILE;	// Reg - 114
	ssc_cache[DA9052_COUNTMO_REG].type	= VOLATILE;	// Reg - 115
	ssc_cache[DA9052_COUNTY_REG].type	= VOLATILE;	// Reg - 116
	ssc_cache[DA9052_ALARMMI_REG].type	= VOLATILE;	// Reg - 117

	ssc_cache[DA9052_SECONDA_REG].type	= VOLATILE;	// Reg - 122
	ssc_cache[DA9052_SECONDB_REG].type	= VOLATILE;	// Reg - 123
	ssc_cache[DA9052_SECONDC_REG].type	= VOLATILE;	// Reg - 124
	ssc_cache[DA9052_SECONDD_REG].type	= VOLATILE;	// Reg - 125
	
	/* Following addresses are not assigned to any register */
	ssc_cache[126].type = VOLATILE;
	ssc_cache[127].type = VOLATILE;
}
	
/*--------------------------------------------------------------------------*/
/* Global Functions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Bus independent functions to provide uniform interface to all drivers    */
/*--------------------------------------------------------------------------*/
/**
 * da9052_ssc_write: This is top most wrapper to bus specific write functions
 * @param *ssc_msg	pointer to ssc_msg structure
 * @return s32		status of write operation
 */
s32 da9052_ssc_write(da9052_ssc_msg *sscmsg)
{
	s32 ret;
	
	/* Reg addr should be a valid address on PAGE0 or PAGE1 */
	if((sscmsg->addr < DA9052_PAGE0_REG_START) || (sscmsg->addr > DA9052_PAGE1_REG_END) || 
		   ((sscmsg->addr > DA9052_PAGE0_REG_END) && (sscmsg->addr < DA9052_PAGE1_REG_START)))
		return (INVALID_REGISTER);
	
	/* Acquire ssc lock */
//	if (down_interruptible(&ssc_sem))
//		return (-EINTR);
	/*
	 * It is better to do un-interruptible sleep here. Otherwise we will get
	 * interttupted in case of high frequency events e.g. ADC THRESHOLD.
	 * As a result this SSC write operation will fail!
	 */
	down(&ssc_sem);
	
#if DA9052_SSC_CONFIG_SPI
	/* Call SPI specific function */
	ret = da9052_ssc_spi_write(sscmsg);
	
#elif DA9052_SSC_CONFIG_I2C
	/* Call I2C specific function */
	ret = da9052_ssc_i2c_write(sscmsg);
#endif
	
	/* Update local cache if required */
	if(ret == SUCCESS){
		/* Check if this register is Non-volatile*/
		if(ssc_cache[sscmsg->addr].type != VOLATILE){
			/* Update value */
			ssc_cache[sscmsg->addr].val = sscmsg->data;
			/* Make this cache entry valid */
			ssc_cache[sscmsg->addr].status = VALID;
		}
	}
	
	/* Release ssc lock */
	up(&ssc_sem);
	
	return (ret);
}

/**
 * da9052_ssc_read: This is top most wrapper to bus specific read functions
 * @param *ssc_msg	pointer to ssc_msg structure
 * @return s32		status of read operation
 */
s32 da9052_ssc_read(da9052_ssc_msg *sscmsg)
{
	s32 ret;

	/* Reg addr should be a valid address on PAGE0 or PAGE1 */
	if((sscmsg->addr < DA9052_PAGE0_REG_START) || (sscmsg->addr > DA9052_PAGE1_REG_END) || 
		   ((sscmsg->addr > DA9052_PAGE0_REG_END) && (sscmsg->addr < DA9052_PAGE1_REG_START)))
		return (INVALID_REGISTER);
	
	/* Acquire ssc lock */
//	if (down_interruptible(&ssc_sem))
//		return (-EINTR);
	/*
	 * It is better to do un-interruptible sleep here. Otherwise we will get
	 * interttupted in case of high frequency events e.g. ADC THRESHOLD.
	 * As a result this SSC read operation will fail!
	 */
	down(&ssc_sem);
	
	/* 
	 * Check if this is a Non-volatile register, if yes then return value -
	 * from cache instead of actual reading from hardware. Before reading -
	 * cache entry, make sure that the entry is valid
	 */
	if(ssc_cache[sscmsg->addr].type != VOLATILE){
		/* The read request is for Non-volatile register */
		/* Check if we have valid cached value for this */
		if(ssc_cache[sscmsg->addr].status == VALID){
			/* We have valid cached value, copy this value */
			sscmsg->data = ssc_cache[sscmsg->addr].val;
			
			/* Release ssc lock */
			up(&ssc_sem);
			return (SUCCESS);
		}
	}
	
#if DA9052_SSC_CONFIG_SPI
	/* Call SPI specific function */
	ret = da9052_ssc_spi_read(sscmsg);
	
#elif DA9052_SSC_CONFIG_I2C
	/* Put I2C specific function here */
	ret = da9052_ssc_i2c_read(sscmsg);
#endif

	/* Update local cache if required */
	if(ret == SUCCESS){
		/* Check if this register is Non-volatile*/
		if(ssc_cache[sscmsg->addr].type != VOLATILE){
			/* Update value */
			ssc_cache[sscmsg->addr].val = sscmsg->data;
			/* Make this cache entry valid */
			ssc_cache[sscmsg->addr].status = VALID;
		}
	}

	/* Release ssc lock */
	up(&ssc_sem);
	
	return (ret);
}


/**
 * da9052_ssc_write_many: Function to write multiple registers
 * @param *ssc_msg	pointer to ssc_msg structure array
 * @param *msg_no	Number of registers to be written
 * @return s32		status of write operation
 */
s32 da9052_ssc_write_many(da9052_ssc_msg *sscmsg, s32 msg_no)
{
	s32 ret;
	
#if DA9052_SSC_CONFIG_SPI
	s32 cnt;
	da9052_ssc_msg *msg;
#endif

	/* Check request size */
	if(msg_no > MAX_READ_WRITE_CNT)
		return (FAILURE);
	
#if DA9052_SSC_CONFIG_SPI
	msg = sscmsg;

	for(cnt = 0; cnt < msg_no; cnt++, msg++) {
		ret = da9052_ssc_write(msg);
		if(ret != SUCCESS)
		{
			DA9052_DEBUG("Error in %s", __FUNCTION__);
			return (FAILURE);
		}
	}
	
#elif DA9052_SSC_CONFIG_I2C
	ret = da9052_ssc_i2c_write_many(sscmsg, msg_no);
#endif
	return (ret);
}

/**
 * da9052_ssc_read_many: Function to read multiple registers
 * @param *ssc_msg	pointer to ssc_msg structure array
 * @param *msg_no	Number of registers to be read
 * @return s32		status of read operation
 */
s32 da9052_ssc_read_many(da9052_ssc_msg *sscmsg, s32 msg_no)
{
	s32 ret;
	
#if DA9052_SSC_CONFIG_SPI
	s32 cnt;
	da9052_ssc_msg *msg;
#endif

	/* Check request size */
	if(msg_no > MAX_READ_WRITE_CNT)
		return (FAILURE);
	
#if DA9052_SSC_CONFIG_SPI	
	msg = sscmsg;

	for(cnt = 0; cnt < msg_no; cnt++, msg++) {
		ret = da9052_ssc_read(msg);
		if(ret != SUCCESS) {
			DA9052_DEBUG("Error in %s", __FUNCTION__);
			return (FAILURE);
		}
	}
	
#elif DA9052_SSC_CONFIG_I2C
	ret = da9052_ssc_i2c_read_many(sscmsg, msg_no);
#endif
	return (ret);
}

#if DA9052_SSC_CONFIG_SPI

/*--------------------------------------------------------------------------*/
/* SPI Specific Functions                                                   */
/*--------------------------------------------------------------------------*/
/**
 * da9052_spi_set_page: Function to set current active page in DA9052
 * @param page	Page to set i.e. PAGECON_128 or PAGECON_0
 * @return int	Status of this operation
 */
static s32 da9052_spi_set_page(u8 page)
{
	da9052_ssc_msg sscmsg;
	struct spi_message message;
	struct spi_transfer xfer;
	s32 ret = 0;
	
	if((page != PAGECON_0) && ((page != PAGECON_128)))
		return (INVALID_PAGE);

	/* Current configuration is PAGE-0 and write request for PAGE-1 */
	sscmsg.addr = DA9052_PAGECON0_REG;	/* set register address */
	sscmsg.data = page;			/* set value */
	
	/* Check value of R/W_POL bit of INTERFACE register */
	if(!rw_pol){
		/* We need to set 0th bit for write operation */
		sscmsg.addr = ((sscmsg.addr << 1) | RW_POL);	
	}else{
		/* We need to reset 0th bit for write operation */
		sscmsg.addr = (sscmsg.addr << 1);
	}

	/* SMDK-6410 host SPI driver specific stuff */
	
	/* Build our spi message */
	spi_message_init(&message);
	memset(&xfer, 0, sizeof(xfer));
	
	xfer.len = 2;
	xfer.tx_buf = spi_tx_buf;
	xfer.rx_buf = spi_rx_buf;

	spi_tx_buf[0] = sscmsg.addr;
	spi_tx_buf[1] = sscmsg.data;

	spi_message_add_tail(&xfer, &message);

	/* Now, do the i/o */
	ret = spi_sync(spi_dev, &message);

	//printk(KERN_CRIT "Message completion code is - %d", message.status);
	DA9052_DEBUG("Return from Write is: %d \n", ret);
	
	if(ret == 0){
		/* Active Page set successfully */
		spi_active_page = page;
		return (SUCCESS);
	}
	else{
		/* Error in setting Active Page */
		return (FAILURE);
	}

}

/**
 * da9052_ssc_spi_write: Function to write a DA9052 register through SPI
 * @param *ssc_msg	pointer to da9052_ssc_msg structure
 * @return s32	status of write operation
 */
static s32 da9052_ssc_spi_write(da9052_ssc_msg *msg)
{
	struct spi_message message;
	struct spi_transfer xfer;
	s32 ret;
		
	/* We need a seperate copy of da9052_ssc_msg so that caller's copy remains intact */
	da9052_ssc_msg sscmsg;
	
	/* Copy callers data in to our local copy */
	sscmsg.addr = msg->addr;
	sscmsg.data = msg->data;
	
	DA9052_DEBUG("[%s] REG = 0x%x, DATA = 0x%x \n", __FUNCTION__, msg->addr, msg->data);
	
	if((sscmsg.addr > PAGE_0_END) && (spi_active_page == PAGECON_0)){
		/* Current configuration is PAGE-0 and write request for PAGE-1 */
		da9052_spi_set_page(PAGECON_128);
		/* Set register address accordindly */
		sscmsg.addr = (sscmsg.addr - PAGE_1_START);
	}
	else if((sscmsg.addr < PAGE_1_START) && (spi_active_page == PAGECON_128)){
		/* Current configuration is PAGE-1 and write request for PAGE-0 */
		da9052_spi_set_page(PAGECON_0);
	}
	else if(sscmsg.addr > PAGE_0_END){
		/* Current configuration is PAGE-1 and write request for PAGE-1 */
		/* Just need to adjust register address */
		sscmsg.addr = (sscmsg.addr - PAGE_1_START);
	}
		
	/* Check value of R/W_POL bit of INTERFACE register */
	if(!rw_pol){
		/* We need to set 0th bit for write operation */
		sscmsg.addr = ((sscmsg.addr << 1) | RW_POL);	
	}else{
		/* We need to reset 0th bit for write operation */
		sscmsg.addr = (sscmsg.addr << 1);
	}
	
	/* SMDK-6410 host SPI driver specific stuff */
	
	/* Build our spi message */
	spi_message_init(&message);
	memset(&xfer, 0, sizeof(xfer));
	
	xfer.len = 2;
	xfer.tx_buf = spi_tx_buf;
	xfer.rx_buf = spi_rx_buf;

	spi_tx_buf[0] = sscmsg.addr;
	spi_tx_buf[1] = sscmsg.data;

	spi_message_add_tail(&xfer, &message);

	/* Now, do the i/o */
	ret = spi_sync(spi_dev, &message);

	//printk(KERN_CRIT "Message completion code is - %d", message.status);
	DA9052_DEBUG("Return from Write is: %d \n", ret);
		
	if(ret == 0)
		return (SUCCESS);
	else
		return (FAILURE);
}

/**
 * da9052_ssc_spi_read:	Function to read a DA9052 register through SPI
 * @param *ssc_msg	pointer to da9052_ssc_msg structure
 * @return s32	status of write operation
 */
static s32 da9052_ssc_spi_read(da9052_ssc_msg *msg)
{
	struct spi_message message;
	struct spi_transfer xfer;
	s32 ret;
		
	/* We need a seperate copy of da9052_ssc_msg so that caller's copy remains intact */
	da9052_ssc_msg sscmsg;
	
	/* Copy callers data in to our local copy */
	sscmsg.addr = msg->addr;
	sscmsg.data = msg->data;
	
	if((sscmsg.addr > PAGE_0_END) && (spi_active_page == PAGECON_0)){
		/* Current configuration is PAGE-0 and read request for PAGE-1 */
		da9052_spi_set_page(PAGECON_128);
		/* Set register address accordindly */
		sscmsg.addr = (sscmsg.addr - PAGE_1_START);
	}
	else if((sscmsg.addr < PAGE_1_START) && (spi_active_page == PAGECON_128)){
		/* Current configuration is PAGE-1 and write request for PAGE-0 */
		da9052_spi_set_page(PAGECON_0);
	}
	else if(sscmsg.addr > PAGE_0_END){
		/* Current configuration is PAGE-1 and write request for PAGE-1 */
		/* Just need to adjust register address */
		sscmsg.addr = (sscmsg.addr - PAGE_1_START);
	}
	
	/* Check value of R/W_POL bit of INTERFACE register */
	if(rw_pol){
		/* We need to set 0th bit for read operation */
		sscmsg.addr = ((sscmsg.addr << 1) | RW_POL);	
	}else{
		/* We need to reset 0th bit for write operation */
		sscmsg.addr = (sscmsg.addr << 1);
	}
	
	/* SMDK-6410 host SPI driver specific stuff */
	
	/* Build our spi message */
	spi_message_init(&message);
	memset(&xfer, 0, sizeof(xfer));
	
	xfer.len = 2;
	xfer.tx_buf = spi_tx_buf;
	xfer.rx_buf = spi_rx_buf;

	spi_tx_buf[0] = sscmsg.addr;
	spi_tx_buf[1] = 0xff;
	
	spi_rx_buf[0] = 0;
	spi_rx_buf[1] = 0;

	spi_message_add_tail(&xfer, &message);

	/* Now, do the i/o */
	ret = spi_sync(spi_dev, &message);
	
	//printk(KERN_CRIT "Message completion code is - %d", message.status);
	DA9052_DEBUG("Return from Write is: %d \n", ret);
	
	if(ret == 0){
		/* Update read value in callers copy */
		msg->data = spi_rx_buf[1];
		return (SUCCESS);
	}
	else{
		return (FAILURE);
	}
}

#elif DA9052_SSC_CONFIG_I2C
/*--------------------------------------------------------------------------*/
/* I2C Specific Functions                                                   */
/*--------------------------------------------------------------------------*/

/**
 * da9052_ssc_i2c_write: Function to write a DA9052 register through I2C
 * @param *ssc_msg	pointer to da9052_ssc_msg structure
 * @return s32	status of write operation
 */
static s32 da9052_ssc_i2c_write(da9052_ssc_msg *msg)
{
	/*Get the da9052_i2c client details*/
	da9052_ssc_i2c_data *i2c = i2c_get_clientdata(da9052_i2c_client);
	struct i2c_msg i2cmsg;
	u8 buf[2] = {0};
	s32 ret = 0;
		
	/* Copy the ssc msg to local character buffer */
	buf[0] = msg->addr;
	buf[1] = msg->data;	
	
	/*Construct a i2c msg for a da9052 driver ssc message request */
	i2cmsg.addr  = i2c->da9052_slave_addr ;
	i2cmsg.len   = 2;
	i2cmsg.buf   = buf;
	
	/*To write the data on I2C set flag to zero */
	i2cmsg.flags = 0;
	
	/* Start the i2c transfer by calling host i2c driver function */
	ret = i2c_transfer(da9052_i2c_client->adapter, &i2cmsg, 1);
	
	if(ret < 0){
		DA9052_DEBUG("_%s:master_xfer Failed!!\n", __FUNCTION__);	
		return(FAILURE);
	}
	
	return (SUCCESS);
}

/**
 * da9052_ssc_i2c_read:	Function to read a DA9052 register through I2C
 * @param *ssc_msg	pointer to da9052_ssc_msg structure
 * @return s32	status of write operation
 */
static s32 da9052_ssc_i2c_read(da9052_ssc_msg *msg)
{
	/*Get the da9052_i2c client details*/
	da9052_ssc_i2c_data *i2c = i2c_get_clientdata(da9052_i2c_client);
	u8 buf[1] = {0};
	struct i2c_msg i2cmsg;
	s32 ret = 0;
	
	/* Copy SSC Msg to local character buffer */
	buf[0] = msg->addr;
	
	/*Construct a i2c msg for a da9052 driver ssc message request */
	i2cmsg.addr  = i2c->da9052_slave_addr ;
	i2cmsg.len   = 1;
	i2cmsg.buf   = buf;
	
	/*To write the data on I2C set flag to zero */
	i2cmsg.flags = 0;
	
	/* Start the i2c transfer by calling host i2c driver function */
	ret = i2c_transfer(da9052_i2c_client->adapter, &i2cmsg, 1);
	if(ret < 0){
		DA9052_DEBUG("1 - %s:master_xfer Failed!!\n", __FUNCTION__);
		return(FAILURE);
	}

	/* Read the data from da9052*/	
	/*Construct a i2c msg for a da9052 driver ssc message request */
	i2cmsg.addr  = i2c->da9052_slave_addr ;
	i2cmsg.len   = 1;
	i2cmsg.buf   = buf;
	
	/*To read the data on I2C set flag to I2C_M_RD */
	i2cmsg.flags = I2C_M_RD; 
	
	/* Start the i2c transfer by calling host i2c driver function */
	ret = i2c_transfer(da9052_i2c_client->adapter, &i2cmsg, 1);
	if(ret < 0){
		DA9052_DEBUG("2 - %s:master_xfer Failed!!\n", __FUNCTION__);
		return(FAILURE);
	}
	msg->data = buf[0];
	
	return (SUCCESS);
}

/**
 * da9052_ssc_i2c_write_many: Function to write multiple registers through I2C
 * @param *ssc_msg	pointer to ssc_msg structure array
 * @param *msg_no	Number of registers to be write
 * @return s32		status of write operation
 */
static s32 da9052_ssc_i2c_write_many(da9052_ssc_msg *sscmsg, s32 msg_no)
{
	struct i2c_msg i2cmsg;
	u8 data_buf[MAX_READ_WRITE_CNT+1];
	da9052_ssc_i2c_data *i2c;
	
	da9052_ssc_msg ctrlb_msg;
	da9052_ssc_msg *msg_queue = sscmsg;
	s32 ret = 0;
	
	/* Flag to check if requested registers are contiguous */
	u8 cont_data = TRUE;
	u8 cnt = 0;
	
	/* Check if requested regsiters are contiguous */
	for(cnt = 1; cnt < msg_no; cnt++){
		if((msg_queue[cnt].addr - msg_queue[cnt-1].addr) != 1){
			/* Difference is not 1, i.e. non-contiguous registers */
			cont_data = FALSE;
			break;
		}
	}
	
	if(cont_data == FALSE){
		/* Requested registers are non-contiguous */
		for(cnt = 0; cnt < msg_no; cnt++) {
			ret = da9052_ssc_write(&msg_queue[cnt]);
			if(ret != SUCCESS) {
				DA9052_DEBUG("Error in %s", __FUNCTION__);
				return (FAILURE);
			}
		}
		return (SUCCESS);
	}
	
	/* Requested registers are contiguous */
	
	/*
	* We want to perform PAGE WRITE via I2C
	* For PAGE WRITE sequence of I2C transactions is as below
	*
	* (slave_addr + reg_addr + data_1 + data_2 + ...)
	*/
	
	/* First read current WRITE MODE via CONTROL_B register of DA9052 */
	ctrlb_msg.addr = DA9052_CONTROLB_REG;
	ctrlb_msg.data = 0x0;
	ret = da9052_ssc_read(&ctrlb_msg);
	
	if(ret != SUCCESS){
		DA9052_DEBUG("Error in Reading DA9052_CONTROLB_REG in %s",__FUNCTION__);
		return (FAILURE);
	} 
	
	/* Check if PAGE WRITE mode is set */
	if(ctrlb_msg.data & DA9052_CONTROLB_WRITEMODE){
		/* REPEAT WRITE mode is configured */
		/* Now set DA9052 into PAGE WRITE mode */
		ctrlb_msg.data &= ~DA9052_CONTROLB_WRITEMODE;
		ret = da9052_ssc_write(&ctrlb_msg);
	
		if(ret != SUCCESS){
			DA9052_DEBUG("1-Error in Writing to DA9052_CONTROLB_REG in %s",__FUNCTION__);
			return (FAILURE);
		}
	}
	
	/* All set at DA9052 side, prepare for I2C transfer */
	
	/* Acquire ssc lock */
//	if (down_interruptible(&ssc_sem))
//		return (-EINTR);
	/*
	 * It is better to do un-interruptible sleep here. Otherwise we will get
	 * interttupted in case of high frequency events e.g. ADC THRESHOLD.
	 * As a result this SSC write operation will fail!
	 */
	down(&ssc_sem);

	i2c = i2c_get_clientdata(da9052_i2c_client);
	
	/* Put first register address */
	data_buf[0] = msg_queue[0].addr;
	
	for(cnt = 0; cnt < msg_no; cnt++){
		data_buf[cnt+1] = msg_queue[cnt].data;
	}
	
	/* Construct a i2c msg for PAGE WRITE */
	i2cmsg.addr  = i2c->da9052_slave_addr ;
	i2cmsg.len   = (msg_no + 1);	// First register address + all data
	i2cmsg.buf   = data_buf;
	
	/*To write the data on I2C set flag to zero */
	i2cmsg.flags = 0; 
	
	/* Start the i2c transfer by calling host i2c driver function */
	ret = i2c_transfer(da9052_i2c_client->adapter, &i2cmsg, 1);
	if(ret < 0){
		up(&ssc_sem);
		DA9052_DEBUG("1 - i2c_transfer function falied in [%s]!!!\n", __FUNCTION__);
		return(FAILURE);
	}
	
	/* Update local cache, if required */
	for(cnt = 0; cnt < msg_no; cnt++){
		/* Check if this register is Non-volatile*/
		if(ssc_cache[sscmsg[cnt].addr].type != VOLATILE){
			/* Update value */
			ssc_cache[sscmsg[cnt].addr].val = sscmsg[cnt].data;
			/* Make this cache entry valid */
			ssc_cache[sscmsg[cnt].addr].status = VALID;
		}
	}
	
	/* Release ssc lock before performing single ssc write */
	up(&ssc_sem);
	
	return (SUCCESS);
}

/**
 * da9052_ssc_i2c_read_many: Function to read multiple registers through I2C
 * @param *ssc_msg	pointer to ssc_msg structure array
 * @param *msg_no	Number of registers to be read
 * @return s32		status of read operation
 */
static s32 da9052_ssc_i2c_read_many(da9052_ssc_msg *sscmsg, s32 msg_no)
{
	struct i2c_msg i2cmsg;
	u8 data_buf[MAX_READ_WRITE_CNT];
	da9052_ssc_i2c_data *i2c;
	
	da9052_ssc_msg *msg_queue = sscmsg;
	s32 ret = 0;
	
	/* Flag to check if requested registers are contiguous */
	u8 cont_data = TRUE;
	u8 cnt = 0;

	/* Check if requested regsiters are contiguous */
	for(cnt = 1; cnt < msg_no; cnt++){
		if((msg_queue[cnt].addr - msg_queue[cnt-1].addr) != 1){
			/* Difference is not 1, i.e. non-contiguous registers */
			cont_data = FALSE;
			break;
		}
	}
	
	if(cont_data == FALSE){
		/* Requested registers are non-contiguous */
		for(cnt = 0; cnt < msg_no; cnt++) {
			ret = da9052_ssc_read(&msg_queue[cnt]);
			if(ret != SUCCESS) {
				DA9052_DEBUG("Error in %s", __FUNCTION__);
				return (FAILURE);
			}
		}
		return (SUCCESS);
	}
	
	/* Requested registers are contiguous */
	
	/* Acquire ssc lock */
//	if (down_interruptible(&ssc_sem))
//		return (-EINTR);
	/*
	 * It is better to do un-interruptible sleep here. Otherwise we will get
	 * interttupted in case of high frequency events e.g. ADC THRESHOLD.
	 * As a result this SSC write operation will fail!
	 */
	down(&ssc_sem);
	
	i2c = i2c_get_clientdata(da9052_i2c_client);
	
	/*
	 * We want to perform PAGE READ via I2C
	 * For PAGE READ sequence of I2C transactions is as below
	 *
	 * (slave_addr + reg_addr) + (slave_addr + data_1 + data_2 + ...)
	 */
	
	/* Copy address of first register */
	data_buf[0] = msg_queue[0].addr;
	
	/* Construct a i2c msg for first transaction of PAGE READ i.e. write */
	i2cmsg.addr  = i2c->da9052_slave_addr ;
	i2cmsg.len   = 1;
	i2cmsg.buf   = data_buf;
	
	/*To write the data on I2C set flag to zero */
	i2cmsg.flags = 0; 
	
	/* Start the i2c transfer by calling host i2c driver function */
	ret = i2c_transfer(da9052_i2c_client->adapter, &i2cmsg, 1);
	if(ret < 0){
		up(&ssc_sem);
		DA9052_DEBUG("1 - i2c_transfer function falied in [%s]!!!\n", __FUNCTION__);
		return(FAILURE);
	}

	/* Now Read the data from da9052 */
	
	/* Construct a i2c msg for second transaction of PAGE READ i.e. read */
	i2cmsg.addr  = i2c->da9052_slave_addr ;
	i2cmsg.len   = msg_no;
	i2cmsg.buf   = data_buf;
	
	/*To read the data on I2C set flag to I2C_M_RD */
	i2cmsg.flags = I2C_M_RD; 
	
	/* Start the i2c transfer by calling host i2c driver function */
	ret = i2c_transfer(da9052_i2c_client->adapter, &i2cmsg, 1);
	if(ret < 0){
		up(&ssc_sem);
		DA9052_DEBUG("2 - i2c_transfer function falied in [%s]!!!\n", __FUNCTION__);
		return(FAILURE);
	}
	
	/* Gather READ data */
	for(cnt = 0; cnt < msg_no; cnt++) {
		sscmsg[cnt].data = data_buf[cnt];
	}
	
	/* Update local cache, if required */
	for(cnt = 0; cnt < msg_no; cnt++){
		/* Check if this register is Non-volatile*/
		if(ssc_cache[sscmsg[cnt].addr].type != VOLATILE){
			/* Update value */
			ssc_cache[sscmsg[cnt].addr].val = sscmsg[cnt].data;
			/* Make this cache entry valid */
			ssc_cache[sscmsg[cnt].addr].status = VALID;
		}
	}
	
	up(&ssc_sem);
	return (SUCCESS);
}

#endif		/* DA9052_SSC_CONFIG_I2C */

/*--------------------------------------------------------------------------*/
/* Infrastructure Functions                                                 */
/*--------------------------------------------------------------------------*/
/**
 * da9052_ssc_open: Function that is called when some one opens SSC device node
 * 
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return int   Error Code, zero: no error
 */
s32 da9052_ssc_open(struct inode *inode, struct file *file)
{
	/* Check if device is already open */
	if(ssc_device_open) {
		printk( KERN_INFO "DA9052: SSC device already open.\n");
		return (-EBUSY);
	} else {
		ssc_device_open++;
		return (SUCCESS);
	}
}

/**
 * da9052_ssc_release:	Function that is called when some one closes SSC device node
 * 
 * @param *inode	pointer to device inode
 * @param *file	file pointer
 * @return int	Status of release operation
 */
s32 da9052_ssc_release(struct inode *inode, struct file *file)
{
	ssc_device_open--;
	printk( KERN_INFO "DA9052: SSC device closed.\n");
	return (SUCCESS);
}

/**
 * da9052_ssc_suspend: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @param state  pm state
 * @return s32 status of suspend operation
 */
static s32 da9052_ssc_suspend(struct platform_device *dev, pm_message_t state)
{
	/* Put your suspend related operations here */
	printk(KERN_CRIT "[%s] called! \n", __FUNCTION__);
	return (SUCCESS);
}

/**
 * da9052_ssc_resume: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @return s32 status of resume operation
 */
static s32 da9052_ssc_resume(struct platform_device *dev)
{
	/* Put your resume related operations here */
	printk(KERN_CRIT "[%s] called! \n", __FUNCTION__);
	return (SUCCESS);
}

/**
 * da9052_ssc_ioctl: Function that is called when some one calls ioctl() on SSC device node
 * 
 * @param *inode Inode number of device node
 * @param *file  File pointer
 * @param cmd	ioctl command to be executed
*  @arg	arguments for cmd
 * @return s32	status of executing cmd
 */
s32 da9052_ssc_ioctl(struct inode *inode, struct file *file, 
					unsigned int cmd, unsigned long arg)
{	
	s32 cnt, cnt_start, cnt_end;
	s32 ret = SUCCESS;
	
	da9052_ssc_msg *in_data, sscmsg, sscmsg_temp;
	
	in_data = (da9052_ssc_msg *) arg;

	DA9052_DEBUG("I am in ____%s____\n", __FUNCTION__);
	
	/* Copy parameters from user space */
	if (copy_from_user(&sscmsg, in_data, sizeof(da9052_ssc_msg)))
		return (-EFAULT);

	switch(cmd) {
		
		case DA9052_IOCTL_SSC_READ:
			/* Command to read a register */
			if(da9052_ssc_read(&sscmsg) != SUCCESS)
				return (FAILURE);	
			/* Read successful, pass data to user space */
			if(copy_to_user(in_data, &sscmsg, sizeof(da9052_ssc_msg)))
				return (-EFAULT);
			return (SUCCESS);
			
		case DA9052_IOCTL_SSC_WRITE:
			/* Command to write a register */
			DA9052_DEBUG("sscmsg->addr = 0x%x \n", sscmsg.addr);
			DA9052_DEBUG("sscmsg->data = 0x%x \n", sscmsg.data);
			return (da9052_ssc_write(&sscmsg));
			
		case DA9052_IOCTL_SSC_TEST:
			/*
			 * This is a READ/WRITE test for SPI. This can be used -
			 * to validate hardware interface between host and -
			 * DA9052 EVAL board. We have used to isolate many -
			 * grounding and noise related issues in SPI connection
			 *
			 * This test performs following operations
			 * (1) Write a value to a DA9052 register
			 * (2) Read back same register
			 * (3) Compare read value
			 * (4) If read value matches with write value, repeat test
			 * (5) If read value doesn't match, come out, giving an error
			 */
			
			/* Just for some profiling of SPI bus operations */
			cnt_start = jiffies;
			
			/* R102 and R103 registes are safe to use for test */
			sscmsg.addr = 102;
			sscmsg_temp.addr = 102;
			sscmsg_temp.data = 0;
			
			/* Start test */
			for(cnt = 0; cnt < 10240; cnt++){
				sscmsg.addr = 102;
				sscmsg.data = ((u8)cnt % 127);
				if(da9052_ssc_write(&sscmsg) != SUCCESS){
					printk(KERN_ERR "Error in WRITE\n");
					ret = FAILURE;
					break;
				}
				
				/*
				 * Perform a dummy write on other register
				 * This is requried because according to data -
				 * sheet of DA9052, a regsiter can't be read -
				 * immediately after performing write operation
				 * on it.
				 */
				sscmsg.addr = 103;
				sscmsg.data = 0xAA;
				if(da9052_ssc_write(&sscmsg) != SUCCESS){
					printk(KERN_ERR "Error in dummy WRITE\n");
					ret = FAILURE;
					break;
				}
				
				/* Read back first written register */
				if(da9052_ssc_read(&sscmsg_temp) != SUCCESS){
					printk(KERN_ERR "Error in READ\n");
					ret = FAILURE;
					break;
				}

				/* Compare read value */
				if(sscmsg_temp.data != ((u8)cnt % 127))
				{
					printk(KERN_ERR "READ/WRITE Error, data = %d, iteration = %d \n",
					sscmsg_temp.data, cnt);
					ret = FAILURE;
					break;
				}
			}
			cnt_end = jiffies;
			printk("Time taken for %d Read+Write cycles is %d ms\n", (cnt+1), 
			       jiffies_to_msecs(cnt_end - cnt_start));
			break;
			
		case DA9052_IOCTL_DUMP_SPI_REG:
			/* This is purely for debug purpose */
			//print_spi_reg(spi_dev);
			ret = FAILURE;
			break;
		default:
			break;
	}	
	return (ret);
}

#if DA9052_SSC_CONFIG_SPI

/**
 * da9052_spi_probe: Called when a device gets attached to driver 
 * 
 * @param struct spi_device *spi Pointer to spi device
 * @return int Error Code, zero: no error
 */
static int da9052_spi_probe(struct spi_device *spi)
{	
	/* Store handle to spi device */
	spi_dev = spi;
	
	/* Allocate memory for RX/TX bufferes used in single register read/write */
	spi_rx_buf = kmalloc(2, GFP_KERNEL | GFP_DMA);
	if (!spi_rx_buf)
		return (-ENOMEM);
	
	spi_tx_buf = kmalloc(2, GFP_KERNEL | GFP_DMA);
	if (!spi_tx_buf)
		return (-ENOMEM);
	
	/* Configure SPI */
	spi->mode = (SPI_MODE_0 | SPI_CPOL);
	spi->bits_per_word = 8;
	
	spi_setup(spi);
	
	return (0);
}

/**
 * da9052_spi_remove: Called when a device gets detached from driver 
 * 
 * @param struct spi_device *spi Pointer to I2C client
 * @return int Error Code, zero: no error
 */
static int da9052_spi_remove(struct spi_device *spi)
{
	printk("__[%s]__ called! \n",__FUNCTION__);
	kfree(spi_rx_buf);
	kfree(spi_tx_buf);
	
	return (0);
}

/**
 * da9052_spi_suspend: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @param state  pm state
 * @return s32 status of suspend operation
 */
static s32 da9052_spi_suspend(struct spi_device *dev, pm_message_t state)
{
	/* Put your suspend related operations here */
	printk(KERN_CRIT "[%s] called! \n", __FUNCTION__);
	return (SUCCESS);
}

/**
 * da9052_spi_resume: Power Management support function
 * 
 * @param *dev   pointer to platform device
 * @return s32 status of resume operation
 */
static s32 da9052_spi_resume(struct spi_device *dev)
{
	/* Put your resume related operations here */
	printk(KERN_CRIT "[%s] called! \n", __FUNCTION__);
	return (SUCCESS);
}

static struct spi_driver da9052_spi_driver = {
	.driver = {
		.name	 = DA9052_SSC_SPI_DEVICE_NAME,
		.bus	= &spi_bus_type,
		.owner	= THIS_MODULE,
	},
	.probe	 	= da9052_spi_probe,
	.remove 	= __devexit_p(da9052_spi_remove),
	.suspend	= da9052_spi_suspend,
	.resume		= da9052_spi_resume,
};

#elif DA9052_SSC_CONFIG_I2C

/**
 * da9052_i2c_probe: Called when a device gets attached to driver 
 * 
 * @param i2c_client Pointer to I2C client
 * @return int Error Code, zero: no error
 */
static int __devinit da9052_i2c_probe(struct i2c_client *client, const struct i2c_device_id *id)
{	
	/* Get the bus driver handler */
	struct i2c_adapter *adapter = to_i2c_adapter(client->dev.parent);
	da9052_ssc_i2c_data *clientdata = NULL;
	int err = 0;
		
	/* Check i2c bus driver supports byte data transfer */
	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE_DATA)){
		err = -ENODEV;
		printk("Error in %s:i2c_check_functionality \n",__FUNCTION__);
		return (err);
	}
	
	/* Allocate memory for i2c structure */
	clientdata = kzalloc(sizeof(da9052_ssc_i2c_data), GFP_KERNEL);
	if (!clientdata) {
		err = -ENOMEM;
		return (err);
	}
	
	/* Store handle to i2c client */
	da9052_i2c_client = client;
	
	/*Initalize i2c data structure here*/
	clientdata->adapter = adapter;
	
	/* host i2c driver looks only first 7 bits for the slave address */
	clientdata->da9052_slave_addr = DA9052_I2C_ADDR >> 1;
		
	/* Store the i2c client data */
	i2c_set_clientdata(client, clientdata);
	
	return (SUCCESS);
}

/**
 * da9052_i2c_remove: Called when a device gets detached from driver 
 * 
 * @param i2c_client Pointer to I2C client
 * @return int Error Code, zero: no error
 */
static int da9052_i2c_remove(struct i2c_client *client)
{
	da9052_ssc_i2c_data *clientdata = i2c_get_clientdata(client);
	
	printk(KERN_INFO "DA9052: Removing i2c device \n");
	kfree(clientdata);
	return (SUCCESS);
}

static struct i2c_device_id da9052_ssc_id[] = {
	{ DA9052_SSC_I2C_DEVICE_NAME, 0},
	{}
};

static struct i2c_driver da9052_i2c_driver =  {
	.id = 0,
	.driver = {
		.name   = DA9052_SSC_I2C_DEVICE_NAME,
		.owner  = THIS_MODULE,
	},
	.probe  = da9052_i2c_probe,
	.remove = da9052_i2c_remove,
	.id_table = da9052_ssc_id,
};

#endif		/* DA9052_SSC_CONFIG_I2C */

/**
 * static struct file_operations da9052_ssc_fops
 * This structure definition has to be defined here as an exception. 
 * @owner: member description
 * @open : member description
 * @release: member description
 * @ioctl: member description
 */
static const struct file_operations da9052_ssc_fops = {
        .owner          = THIS_MODULE,
        .open           = da9052_ssc_open,
        .release        = da9052_ssc_release,
        .ioctl          = da9052_ssc_ioctl
};

/**
 * da9052_ssc_probe: Called when a device gets attached to driver 
 * 
 * @param *dev   pointer to platform device
 * @return int   Error Code, zero: no error
 */
static s32 __devinit da9052_ssc_probe(struct platform_device *dev)
{
	s32 ret;	
	
	/* Register the device */
	ret = register_chrdev(ssc_major_number, 
			      DA9052_SSC_DEVICE_NAME, &da9052_ssc_fops);
	if(ret != 0) {
		printk(KERN_ERR "Unable to register %s\n", DA9052_SSC_DEVICE_NAME);
		return (-EFAULT);	
	} else {
		printk(KERN_INFO "%s: Major number is: %d.\n", 
		       DA9052_SSC_DEVICE_NAME, ssc_major_number);
		
#if DA9052_SSC_CONFIG_SPI
		printk(KERN_INFO "DA9052 SSC-SPI Driver\n");
		ret = spi_register_driver(&da9052_spi_driver);
		if(ret != 0) {
			printk(KERN_ERR "Unable to register %s\n", DA9052_SSC_SPI_DEVICE_NAME);
			unregister_chrdev(ssc_major_number, DA9052_SSC_DEVICE_NAME);
			return (-EFAULT);
		}

#elif DA9052_SSC_CONFIG_I2C
		ret = i2c_add_driver(&da9052_i2c_driver);
		if(ret != 0) {
			printk(KERN_ERR "Unable to register %s\n", DA9052_SSC_I2C_DEVICE_NAME);
			unregister_chrdev(ssc_major_number, DA9052_SSC_DEVICE_NAME);
			return (-EFAULT);	
		}
#endif
	}
	/* Initialize ssc lock */
	init_MUTEX(&ssc_sem);
	
	/* Initialize ssc cache */
	da9052_init_ssc_cache();
			
	return (SUCCESS);
}

/**
 * da9052_ssc_remove: Called when detaching device from driver
 * @param *dev   pointer to platform device
 * @return int   Error Code, zero: no error
 */
static int __devexit da9052_ssc_remove(struct platform_device *dev)
{
	DA9052_DEBUG(KERN_DEBUG "Removing %s \n", DA9052_SSC_DEVICE_NAME);
	return (SUCCESS);
}

/**
 * static struct platform_driver da9052_ssc_driver -
 * @probe: Probe function for this device. 
 * @remove: Function to be called when removing this device from platform
 * @suspend: Function to be called in suspend mode
 * @resume: Function to be called in resume mode
 * @driver: Contains glue logic to bind platform device and plarform driver
 */
static struct platform_driver da9052_ssc_driver = {
	.probe		= da9052_ssc_probe,
	.remove		= __devexit_p(da9052_ssc_remove),
	.suspend	= da9052_ssc_suspend,
	.resume		= da9052_ssc_resume,
	.driver		= {
		.name	= DA9052_SSC_DEVICE_NAME,
		.owner	= THIS_MODULE,
	},
};

/**
 * da9052_ssc_init: Function that gets called at the time of module loading
 * @param void
 * @return int status of module initialization
 */
static s32 __init da9052_ssc_init(void)
{
	s32 ret;
	
	printk(banner);
	
	da9052_ssc_platform_device = platform_device_alloc("da9052_ssc", 0);
	if (!da9052_ssc_platform_device)
		return (-ENOMEM);

	/* Add platform device to device hierarchy */
	ret = platform_device_add(da9052_ssc_platform_device);
	if (ret < 0) {
		/* Free all memory associated with a platform device */
		platform_device_put(da9052_ssc_platform_device);
		return (ret);
	}

	ret = platform_driver_register(&da9052_ssc_driver);
	if (ret < 0)
		/* Unregister the platform level device */
		platform_device_unregister(da9052_ssc_platform_device);

	return (ret);
}

/**
 * da9052_ssc_exit: Function that gets called at the time of module unloading
 * @param void
 * @return void
 */
static void __exit da9052_ssc_exit(void)
{
	printk(KERN_INFO "DA9052: Unregistering SSC device.\n");
	
	unregister_chrdev(ssc_major_number, DA9052_SSC_DEVICE_NAME);
	
#if DA9052_SSC_CONFIG_SPI
	spi_unregister_driver(&da9052_spi_driver);
#elif DA9052_SSC_CONFIG_I2C
	i2c_del_driver(&da9052_i2c_driver);
#endif

	platform_driver_unregister(&da9052_ssc_driver);
	/* Unregister the platform level device */
	platform_device_unregister(da9052_ssc_platform_device);
}

module_init(da9052_ssc_init);
module_exit(da9052_ssc_exit);

MODULE_AUTHOR("Dialog Semiconductor Ltd");
MODULE_DESCRIPTION("DA9052 SSC Device Driver");
MODULE_LICENSE("GPL");

/*--------------------------------------------------------------------------*/
/* Exports                                                                  */
/*--------------------------------------------------------------------------*/
EXPORT_SYMBOL(da9052_ssc_read);
EXPORT_SYMBOL(da9052_ssc_write);
EXPORT_SYMBOL(da9052_ssc_read_many);
EXPORT_SYMBOL(da9052_ssc_write_many);
