/*
 * da9052_tsi.c: Driver for Touch Screen interface on DA9052 chip.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (05/05/2009): Released with basic TSI functionality with out filter
 *
 * (12/05/2009): Released with following functions 
 *		 (1) window & Average filter implemented
 *		 (2) Calibration function has supported.
 *		 (3) EH interface functions are appropriately implmented and 
 *		     called
 *
 * (11/06/2009): Following are the major changes for this version:
 *		(1) Added thread to poll event handler(EH) periodically
 *		    for data.
 *		(2) The data read from EH is in the same format as that stored 
 *		    on DA9052 registers. This register format data or 
 *		    REG data is then converted to coordinate format.
 *		(3) This thread also implements pen-up detection mechanism.
 *		(4) This version also supports Manual measurement of TSI data.
 *		(5) Additional configuration functions have also been added 
 *		    to this release.
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/input.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/uaccess.h>
#include <linux/freezer.h>

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_reg.h"
#include "../include/da9052_lib.h"
#include "../include/da9052_tsi_filter.h"
#include "../include/da9052_tsi_calibrate.h"
#include "../include/da9052_eh.h"
#include "../include/da9052_gpio.h"
#include "../include/da9052_adc.h"	//for ADC_MAN_TSI & sampling interval
#include "../include/da9052_pm.h"
#include "../include/da9052_tsi_cfg.h"
#include "../include/da9052_tsi.h"

#include "da9052_tsi_filter.c"
#include "da9052_tsi_calibrate.c"


/*--------------------------------------------------------------------------*/
/* Local Constant Definitions                                               */
/*--------------------------------------------------------------------------*/
static u8 banner[] __initdata = "DA9052 TSI Device Driver, v1.0\n";

/*--------------------------------------------------------------------------*/
/* Local Macro Definitions                                                  */
/*--------------------------------------------------------------------------*/
/* TSI driver vrsion */
#define TSI_VERSION 0x0101

/* Bit position of TSI_DELAY bits in contral A register of DA9052 */
#define TSI_DELAY_BIT_SHIFT			6

/* Bit position of TSI_SKIP bits in contral A register of DA9052 */
#define TSI_SKIP_BIT_SHIFT			3

/* Number of TSI result registers */
#define NUM_TSI_RESULT_REGISTERS		4

/* Number of GPIO registers for TSI */
#define NUM_GPIO_TSI_REGISTERS			3

/* polling interval to check for manual conversion (in msecs) */
#define MANUAL_CONVERSION_SLEEP			15

/* time out count for manual conversion of TSI data */
#define MANUAL_CONVERSION_TIME_OUT_CNT		\
		((MANUAL_CONVERSION_TIME_OUT) / MANUAL_CONVERSION_SLEEP)

/* Supply voltage required for TSI conversion, in mVolt */
#define DA9052_TSI_SUPPLY_VOLTAGE		2500

/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/
/* store number of instances of the driver open */
static u8 tsi_device_open = 0;

/* Variable to store major number of DA9052 TSI device */
static s32 tsi_major_number = DA9052_TSI_MAJOR_NUM;

/* TSI Platform device for power management */
static struct platform_device *da9052_tsi_platform_device;

/* Time period (in msecs) for processing of REG FIFO */
u32 tsi_reg_data_poll_interval;

/* Lock for input channel of DA9052 adc manual multiplexer */
extern struct mutex da9052_adc_manconv_lock;


/*--------------------------------------------------------------------------*/
/* Local Function Prototype                                                 */
/*--------------------------------------------------------------------------*/
static ssize_t __init da9052_tsi_create_input_dev(struct input_dev** ip_dev, 
									u8 n);

static ssize_t read_da9052_reg(u8 reg_addr);

static ssize_t write_da9052_reg(u8 reg_addr, u8 data);

static void da9052_tsi_reg_pendwn_event(void);

static ssize_t da9052_tsi_config_pen_detect(u8 flag);

static ssize_t da9052_tsi_disable_irq(enum TSI_IRQ tsi_irq);

static ssize_t da9052_tsi_enable_irq(enum TSI_IRQ tsi_irq);

static ssize_t da9052_tsi_config_manual_coord(enum TSI_COORDINATE coord);

static ssize_t da9052_tsi_config_manual_mode(u8 coordinate);

static ssize_t da9052_tsi_config_auto_mode(u8 state);

static ssize_t da9052_tsi_config_gpio(void);

static ssize_t da9052_tsi_read_manual_data(s16* data, 
						enum TSI_COORDINATE coord);

static ssize_t da9052_tsi_config_power_supply(u8 state);

static da9052_tsi_info *get_tsi_drvdata(void);

void da9052_tsi_penup_event(void);

void da9052_tsi_pen_down_handler(u32 event);

static ssize_t da9052_tsi_reg_proc_thread (void *ptr);

static ssize_t da9052_tsi_resume(struct platform_device *dev);

static ssize_t da9052_tsi_suspend(struct platform_device *dev, 
							pm_message_t state);

/*--------------------------------------------------------------------------*/
/* Global Functions                                                         */
/*--------------------------------------------------------------------------*/
/**
 * da9052_tsi_config_measure_seq: 
 * Description: this function configures tsi measurement 
 *		sequence to XP or XYZP mode. Driver's status is not updated 
 *		in this call.
 *		
 * @param seq		TSI Measurement sequence. 0:XYP, 1:XP.
 * @return ssize_t      Returns Success or Failure
 */
ssize_t da9052_tsi_config_measure_seq(enum TSI_MEASURE_SEQ seq)
{
	ssize_t ret = 0;
	u8 data = 0;
	da9052_tsi_info *ts = get_tsi_drvdata();

	if (seq > 1)
		return (-EINVAL);

	/*Get the latest value from the device*/
	/* Read manual mode control A register */
	ret = read_da9052_reg(DA9052_TSICONTA_REG);
	if(ret < 0) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("read_da9052_reg Failed \n" );
			return (ret);
	}

	/* set state */
	data = (u8)ret;

	if (seq == XYZP_MODE)
		data = enable_xyzp_mode(data);
	else if (seq == XP_MODE)
		data = enable_xp_mode(data);
	else {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("Invalid Value passed \n" );
		return (-EINVAL);
	}

	/*Write value into register*/
	ret = write_da9052_reg(DA9052_TSICONTA_REG, data);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" write_da9052_reg Failed \n" );
		return (ret);
	}

	/* update driver's status */
	ts->tsi_conf.auto_cont.da9052_tsi_cont_a = data;

	return(SUCCESS);
}

/**
 * da9052_tsi_set_sampling_mode: 
 * Description: This is the function to select the 
 *		sampling intervel of ADC to measure the 
 *		TSI co-ordinates either 1ms or 10ms.
 *		
 * @param  mode		the mode to be configured, fast or economy mode. 
 * @return ssize_t	Returns Success or Failure
 */
ssize_t da9052_tsi_set_sampling_mode(u8 mode)
{
	u8 data = 0;
	ssize_t ret=0;
	da9052_tsi_info *ts = get_tsi_drvdata();

	/* Read ADC Control register to get the current configuration */
	ret = read_da9052_reg(DA9052_ADCCONT_REG );
	if(ret < 0) {
		DA9052_DEBUG("DA9052_TSI:%s:", __FUNCTION__);
		DA9052_DEBUG("read_da9052_reg Failed \n" );
		return (ret);
	}
	data = (u8)ret;

	/* configure ADC mode as per required sampling mode */
	if (mode == ECONOMY_MODE) {
		data = adc_mode_economy_mode(data);

	} else if (mode == FAST_MODE) {
		data = adc_mode_fast_mode(data);

	} else {
		DA9052_DEBUG("DA9052_TSI:%s:", __FUNCTION__);
		DA9052_DEBUG("Invalid interval passed \n" );
		return (-EINVAL);
	}

	/*Write into ADC control register*/
	ret = write_da9052_reg(DA9052_ADCCONT_REG, data );
	if(ret) {
		DA9052_DEBUG("DA9052_TSI:%s:", __FUNCTION__);
		DA9052_DEBUG("write_da9052_reg Failed \n" );
		return (ret);
	}

	/* configure driver as per required sampling mode */
	switch(mode)
	{
	case ECONOMY_MODE:
		tsi_reg_data_poll_interval = 
			TSI_ECO_MODE_REG_DATA_PROCESSING_INTERVAL;
		tsi_raw_data_poll_interval = 
			TSI_ECO_MODE_RAW_DATA_PROCESSING_INTERVAL;
		break;
	case FAST_MODE:
		tsi_reg_data_poll_interval = 
			TSI_FAST_MODE_REG_DATA_PROCESSING_INTERVAL;
		tsi_raw_data_poll_interval = 
			TSI_FAST_MODE_RAW_DATA_PROCESSING_INTERVAL;
		break;
	default:
		DA9052_DEBUG("DA9052_TSI:%s:", __FUNCTION__);
		DA9052_DEBUG("Invalid interval passed \n" );
		return (-EINVAL);
	}

	/* Calculate number of zero data count cycles to */
	/* declare penup in TS_PENUP_DETECT_INTERVEL */
	ts->tsi_penup_count = 
		(u32)TS_PENUP_DETECT_INTERVEL / tsi_reg_data_poll_interval; 

	return (SUCCESS);
}

/**
 * da9052_tsi_config_delay: 
 * Description: The TSI delay between measurments are 
 *	        configured from this function
 *		
 * @param delay   	Its a enum variable represents number of slots to be 
 *			delayed for each TSI new measurment
 * @return ssize_t	Returns Success or Failure
 */
ssize_t da9052_tsi_config_delay(enum TSI_DELAY delay)
{
	ssize_t ret = 0;
	u8 data = 0;
	da9052_tsi_info *ts = get_tsi_drvdata();

	if (delay > MAX_TSI_DELAY)
	{
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" invalid value for tsi delay!!!\n" );

		return (-EINVAL);
	}

	/* Get the latest value from the device */
	/* Read manual mode control A register */
	ret = read_da9052_reg(DA9052_TSICONTA_REG);
	if(ret < 0) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("read_da9052_reg Failed \n" );
			return (ret);
	}

	/* clear out 2 bits corresponding to TSI_DELAY */
	data = clear_bits( (u8)ret, DA9052_TSICONTA_TSIDELAY);

	/* store new delay value to register */
	data = set_bits(data, (delay << TSI_DELAY_BIT_SHIFT));

	/* Write value into register */
	ret = write_da9052_reg(DA9052_TSICONTA_REG, data);
	if(ret)
	{
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" write_da9052_reg Failed \n" );

		return (ret);
	}

	/* update s/w state */
	ts->tsi_conf.auto_cont.da9052_tsi_cont_a = data;

	return(SUCCESS);
}

/**
 * da9052_tsi_config_skip_slots: 
 * Description: The TSI number of slots to be skiped between
 *		TSI measurments are configured from this function
 *		
 * @param delay		Its a enum variable represents number of slots to be 
 *			skipped for each TSI new measurment
 * @return ssize_t      Returns Success or Failure
 */
ssize_t da9052_tsi_config_skip_slots(enum TSI_SLOT_SKIP skip)
{
	ssize_t ret = 0;
	u8 data = 0;
	da9052_tsi_info *ts = get_tsi_drvdata();

	if (skip > MAX_TSI_SKIP_SLOT)
	{
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" invalid value for tsi skip slots!!!\n" );
		return (-EINVAL);
	}

	/*Get the latest value from the device*/
	/* Read manual mode control register */
	ret = read_da9052_reg(DA9052_TSICONTA_REG);
	if(ret < 0) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("read_da9052_reg Failed \n" );
			return (ret);
	}

	/* clear out 3 bits corresponding to TSI_SKIP */
	data = clear_bits( (u8)ret, DA9052_TSICONTA_TSISKIP);

	/* store new skip value to register */
	data = set_bits(data, (skip << TSI_SKIP_BIT_SHIFT));

	/*Write value into register*/
	ret = write_da9052_reg(DA9052_TSICONTA_REG, data);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI:da9052_tsi_config_skip_slots:");
		DA9052_DEBUG(" write_da9052_reg Failed \n" );

		return (ret);
	}

	/* update s/w state */
	ts->tsi_conf.auto_cont.da9052_tsi_cont_a = data;

	return(SUCCESS);
}

/**
 * da9052_tsi_config_state:
 * Description: This function sets the measurement state
 *		of TSI device to one of the following:
 *	0. TSI_AUTO_MODE 		continuously measure coordinate data.
 * 	1. TSI_MANUAL_COORD_X		Manually Measure coordinate along X axis
 * 	2. TSI_MANUAL_COORD_Y		Manually Measure coordinate along Y axis
 * 	3. TSI_MANUAL_COORD_Z		Manually Measure coordinate along Z axis
 * 	4. TSI_MANUAL_SET		Measure one set of x, y & z coordinates
 * 	5. TSI_IDLE			TSI disabled
 *
 * @param  state	state to which TSI has to be configured.
 * @return ssize_t	Returns Success or Failure
 */
ssize_t da9052_tsi_config_state(enum TSI_STATE state)
{
	s32 ret;
	da9052_tsi_info *ts = get_tsi_drvdata();

	if (ts->tsi_conf.state == state)
		return (SUCCESS);

	switch (state) {
	case TSI_AUTO_MODE:
		/* Reset zero data counter & early data flag */
		ts->tsi_zero_data_cnt = 0;
		early_data_flag = TRUE;
		debounce_over = FALSE;
		win_reference_valid = FALSE;

		/* Clean REG & RAW FIFOs */ 
		clean_tsi_fifos();

		/* Enable auto TSI mode */
		ret = da9052_tsi_config_auto_mode(DISABLE);
		if (ret)
			return (ret);
		/* Disable TSI manual measurement mode */
		ret = da9052_tsi_config_manual_mode(DISABLE);
		if (ret)
			return (ret);

		/* Disable TSI power supply */
		ret = da9052_tsi_config_power_supply(DISABLE);
		if (ret)
			return (ret);

		/* enable TSI_PEN_DWN interrupt notification */
		ret = da9052_tsi_enable_irq(TSI_PEN_DWN);
		if (ret)
			return (ret);
		ts->tsi_conf.tsi_pendown_irq_mask = RESET;

		/* disable TSI_READY irq */
		ret = da9052_tsi_disable_irq(TSI_DATA_RDY);
		if (ret)
			return (ret);
		ts->tsi_conf.tsi_ready_irq_mask	  = SET;

		/* register with EH */
		da9052_tsi_reg_pendwn_event();

		/* Enable pen down detect */
		ret = da9052_tsi_config_pen_detect(ENABLE);
		if (ret)
			return (ret);
		break;


	case TSI_MANUAL_COORD_X:
		/* Reset pen down flag */
		ts->pen_dwn_event = RESET;

		/* Disable auto TSI mode */
		ret = da9052_tsi_config_auto_mode(DISABLE);
		if (ret)
			return (ret);

		/* configure the axis along which measurement's to be made */
		ret = da9052_tsi_config_manual_coord(X_COORDINATE);
		if (ret)
			return (ret);

		/* Enable TSI manual measurement mode */
		ret = da9052_tsi_config_manual_mode(ENABLE);
		if (ret)
			return (ret);

		/* In manual mode, TSI should NOT be in XP mode */
		da9052_tsi_config_measure_seq(XYZP_MODE);

		/* Enable TSI power supply */
		ret = da9052_tsi_config_power_supply(ENABLE);
		if (ret)
			return (ret);

		/* unregister from EH */
		if (ts->pd_reg_status) {
			da9052_eh_unregister_nb(&ts->pd_nb);
			ts->pd_reg_status = RESET;
		}

		/* Tell EH to stop sampling TSI data from h/w */
		da9052_tsi_stop_sampling();

		/* Enable pen down detect */
		ret = da9052_tsi_config_pen_detect(ENABLE);
		if (ret)
			return (ret);

		break;


	case TSI_MANUAL_COORD_Y:
		/* Reset pen down flag */
		ts->pen_dwn_event = RESET;

		/* Disable auto TSI mode */
		ret = da9052_tsi_config_auto_mode(DISABLE);
		if (ret)
			return (ret);

		/* configure the axis along which measurement's to be made */
		ret = da9052_tsi_config_manual_coord(Y_COORDINATE);
		if (ret)
			return (ret);

		/* Enable TSI manual measurement mode */
		ret = da9052_tsi_config_manual_mode(ENABLE);
		if (ret)
			return (ret);

		/* In manual mode, TSI should NOT be in XP mode */
		da9052_tsi_config_measure_seq(XYZP_MODE);

		/* Enable TSI power supply */
		ret = da9052_tsi_config_power_supply(ENABLE);
		if (ret)
			return (ret);

		/* unregister from EH */
		if (ts->pd_reg_status) {
			da9052_eh_unregister_nb(&ts->pd_nb);
			ts->pd_reg_status = RESET;
		}

		/* Tell EH to stop sampling TSI data from h/w */
		da9052_tsi_stop_sampling();

		/* Enable pen down detect */
		ret = da9052_tsi_config_pen_detect(ENABLE);
		if (ret)
			return (ret);
		break;


	case TSI_MANUAL_COORD_Z:
		/* Reset pen down flag */
		ts->pen_dwn_event = RESET;

		/* Disable auto TSI mode */
		ret = da9052_tsi_config_auto_mode(DISABLE);
		if (ret)
			return (ret);

		/* configure the axis along which measurement's to be made */
		ret = da9052_tsi_config_manual_coord(Z_COORDINATE);
		if (ret)
			return (ret);

		/* Enable TSI manual measurement mode */
		ret = da9052_tsi_config_manual_mode(ENABLE);
		if (ret)
			return (ret);

		/* In manual mode, TSI should NOT be in XP mode */
		da9052_tsi_config_measure_seq(XYZP_MODE);

		/* Enable TSI power supply */
		ret = da9052_tsi_config_power_supply(ENABLE);
		if (ret)
			return (ret);

		/* unregister from EH */
		if (ts->pd_reg_status) {
			da9052_eh_unregister_nb(&ts->pd_nb);
			ts->pd_reg_status = RESET;
		}

		/* Tell EH to stop sampling TSI data from h/w */
		da9052_tsi_stop_sampling();

		/* Enable pen down detect */
		ret = da9052_tsi_config_pen_detect(ENABLE);
		if (ret)
			return (ret);

		break;


	case TSI_MANUAL_SET:
		/* Reset pen down flag */
		ts->pen_dwn_event = RESET;

		/* Enable auto TSI mode */
		ret = da9052_tsi_config_auto_mode(ENABLE);
		if (ret)
			return (ret);

		/* Enable TSI manual measurement mode */
		ret = da9052_tsi_config_manual_mode(ENABLE);
		if (ret)
			return (ret);

		/* In manual mode, TSI should NOT be in XP mode */
		da9052_tsi_config_measure_seq(XYZP_MODE);

		/* Enable TSI power supply */
		ret = da9052_tsi_config_power_supply(ENABLE);
		if (ret)
			return (ret);

		/* unregister from EH */
		if (ts->pd_reg_status) {
			da9052_eh_unregister_nb(&ts->pd_nb);
			ts->pd_reg_status = RESET;
		}

		/* Tell EH to stop sampling TSI data from h/w */
		da9052_tsi_stop_sampling();

		/* Enable pen down detect */
		ret = da9052_tsi_config_pen_detect(ENABLE);
		if (ret)
			return (ret);
		break;


	case TSI_IDLE:
		/* Reset pen down flag */
		ts->pen_dwn_event = RESET;

		/* disable PEN_DOWN_DET mechanism of DA9052 */
		ret = da9052_tsi_config_pen_detect(DISABLE);
		if (ret)
			return (ret);

		/* Disable auto TSI mode */
		ret = da9052_tsi_config_auto_mode(DISABLE);
		if (ret)
			return (ret);

		/* Disable TSI manual measurement mode */
		ret = da9052_tsi_config_manual_mode(DISABLE);
		if (ret)
			return (ret);

		/* Disable TSI power supply */
		ret = da9052_tsi_config_power_supply(DISABLE);
		if (ret)
			return (ret);

		/* unregister from EH */
		if (ts->pd_reg_status) {
			da9052_eh_unregister_nb(&ts->pd_nb);
			ts->pd_reg_status = RESET;
		}
		break;


	default:
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" Invalid state passed");
		return (-EINVAL);
	}

	/* update device state */
	ts->tsi_conf.state = state;

	return (SUCCESS);
}

/**
 * da9052_tsi_get_manual_measurement: 
 * Description: This function is used to read manual conversion data.
 *		The following are the functionalities of this subroutine
 *		1. Verifies the mode of TSI driver.
 *		2. Enables TSI power supply.
 *		3. Configures the da9052 for Manual measurement
 *		4. Starts the manual conversion and reads the data.
 *		5. After conversion disables TSI power supply.
 *
 * @param tsi_data	Pointer to buffer to store manual data.
 * @return ssize_t   	Returns Success or Failure
 */
ssize_t da9052_tsi_get_manual_measurement(da9052_tsi_data* tsi_data)
{
	da9052_tsi_info *ts = get_tsi_drvdata();
	u32 time_out;
	s16 data;
	ssize_t ret;

	/* Initialize mutex required for ADC Manual read */
	//mutex_init(&da9052_adc_manconv_lock);

	/* If TSI state is either Auto or Idle mode */
	/* then return with error */
	if ((ts->tsi_conf.state == TSI_AUTO_MODE) || 
				(ts->tsi_conf.state == TSI_IDLE)) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" Configure TSI to either of the ");
		DA9052_DEBUG("three manual modes.\n");
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" Then call this API.\n" );
		return (-FAILURE);
	}

	if (!tsi_data)
		return (-EINVAL);

	data = 0;
	time_out = 0;

	/* Enable TSI power supply */
	da9052_tsi_config_power_supply(ENABLE);

	/* acquire lock for ADC multiplexer */
	mutex_lock(&da9052_adc_manconv_lock);

	/* configure TSI to manual mode, this starts manual conversion */
	if ((ret=da9052_tsi_config_manual_mode(ENABLE)) != SUCCESS)
		return (ret);

	/* wait till conversion is finished */
	do {
		/*Check whether manual conversion is over or not*/
		/* sleep between iterations */
		msleep(MANUAL_CONVERSION_SLEEP);

		/* wait max 5 sec for manual input */
		time_out++;

		/* read manual conversion register */
		data = read_da9052_reg(DA9052_TSICONTB_REG);
		if(data < 0) {
			DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
			DA9052_DEBUG(" Failed to verify manual conversion\n" );
			return (ret);
		}

	} while((data & DA9052_TSICONTB_TSIMAN) &&
				(time_out < MANUAL_CONVERSION_TIME_OUT_CNT));

	/* Release ADC multiplexer lock */
	mutex_unlock(&da9052_adc_manconv_lock);

	if (time_out == MANUAL_CONVERSION_TIME_OUT_CNT) {
		/* Polling timed out */
		da9052_tsi_config_power_supply(DISABLE);
		return (-FAILURE);
	}

	ts->tsi_conf.man_cont.tsi_cont_b.tsi_man = RESET;

	switch(ts->tsi_conf.state) {
	case TSI_MANUAL_COORD_X:
		da9052_tsi_read_manual_data(&tsi_data->x, X_COORDINATE);
		break;
	case TSI_MANUAL_COORD_Y:
		da9052_tsi_read_manual_data(&tsi_data->y, Y_COORDINATE);
		break;
	case TSI_MANUAL_COORD_Z:
		/* Note: Manual conversion result for z-coordinate */
		/* is stored in either X or Y result register by DA9052*/
		/* DA9052 TSI driver uses X result register to read */
		/* manual conversion result for z coordinate */
		da9052_tsi_read_manual_data(&tsi_data->z, X_COORDINATE);
		break;
	case TSI_MANUAL_SET:
		da9052_tsi_read_manual_data(&tsi_data->x, X_COORDINATE);
		da9052_tsi_read_manual_data(&tsi_data->y, Y_COORDINATE);
		da9052_tsi_read_manual_data(&tsi_data->z, Z_COORDINATE);
		break;
	default:
		da9052_tsi_config_power_supply(DISABLE);
		return (-FAILURE);
	}

	/* conversion over, switch off TSI power supply */
	da9052_tsi_config_power_supply(DISABLE);

	return(SUCCESS);
}

/**
 * da9052_tsi_get_drv_state: 
 * Description: This function returns the state of TSI driver.
 *
 * @param  void
 * @return TSI_STATE   	Returns the state of DA9052 TSI driver.
 */
enum TSI_STATE da9052_tsi_get_drv_state(void)
{
	da9052_tsi_info *ts = get_tsi_drvdata();
	return (ts->tsi_conf.state);
}

/*--------------------------------------------------------------------------*/
/* Local Functions                                                          */
/*--------------------------------------------------------------------------*/
/**
 * get_tsi_drvdata: 
 * Description:	The TSI Driver has a common structure called da9052_tsi_info
 *		which will be used across the driver code. 
 *		This function returns the address of the 
 *		da9052_tsi_info global object.
 *
 * @param  void
 * @return da9052_tsi_info *	Returns the global object address of  
 *				da9052_tsi_info 
 */
static da9052_tsi_info *get_tsi_drvdata(void)
{
	return &gda9052_tsi_info;
}

/**
 * write_da9052_reg: 
 * Description : This is a wrapper function to da9052_ssc_write to TSI driver
 *		 This function takes the da9052 register address and the data
 *		 to be written and populates the input details into
 *		 ssc_msg local sturcture and calls the da9052_ssc_write 
 *
 * @param  reg_addr 	DA9052 Register address to be written
 * @param  data 	Data to be written
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t write_da9052_reg(u8 reg_addr, u8 data)
{
	ssize_t ret=0;
	da9052_ssc_msg ssc_msg;
	
	/* Populate the SSC message structure */
	/* range checks on this data shall be performed by SSC */
	ssc_msg.addr =  reg_addr;
	ssc_msg.data =  data;
	ret = da9052_ssc_write(&ssc_msg);
	if(ret) {
		DA9052_DEBUG("%s: ",__FUNCTION__);
		DA9052_DEBUG("da9052_ssc_write Failed %d\n",ret );
	}

	return ret;
}

/**
 * read_da9052_reg: 
 * Description : This is a wrapper function to da9052_ssc_read to TSI driver
 *		 This function takes the da9052 register address to be read
 *		 and populates the input details into ssc_msg local sturcture
 *		 and calls the da9052_ssc_read
 *
 * @param  reg_addr 	DA9052 Register Address 
 * @return ssize_t   	On successful read, this function returns the register 
 *			value read, otherwise it returns failure.
 */
static ssize_t read_da9052_reg(u8 reg_addr)
{
	 ssize_t ret = 0;
	 da9052_ssc_msg ssc_msg;
	
	/* Populate the SSC message structure */
	/* range checks on this data shall be performed by SSC */
	ssc_msg.addr =  reg_addr;
	ssc_msg.data =  0;
	ret = da9052_ssc_read(&ssc_msg);
	if(ret)
	{
		DA9052_DEBUG("%s: ",__FUNCTION__);
		DA9052_DEBUG("da9052_ssc_read Failed => %d\n" ,ret);
		return (-ret);
	}
	return ssc_msg.data;
}

/**
 * da9052_tsi_reg_pendwn_event: 
 * Description:	The pendown handler function will be registered
 *		with EH Module
 *
 * @param  void			None
 * @return void
 */
static void da9052_tsi_reg_pendwn_event(void)
{
	ssize_t ret = 0;
	da9052_tsi_info  *ts = get_tsi_drvdata();
	
	/* If already registered with EH then return. Duplication is avoided */
	if(ts->pd_reg_status)
	{
		DA9052_DEBUG("%s: Pen down ",__FUNCTION__);
		DA9052_DEBUG("Registeration is already done \n");
		return;
	}

	/* Set the EH data structure to notify for Pen down event*/
	ts->pd_nb.eve_type = PEN_DOWN_EVE;
	ts->pd_nb.call_back = da9052_tsi_pen_down_handler;
	
	/* Register to the EH for the requested event */
	ret = da9052_eh_register_nb(&ts->pd_nb);
	
	if(ret)
	{
		DA9052_DEBUG("%s: EH Registeration",__FUNCTION__);
		DA9052_DEBUG(" Failed: ret = %d\n",ret );
		ts->pd_reg_status = RESET;
	
	}else{
		/* Update the eh registeration state */
		ts->pd_reg_status = SET;
	}

	return;
}

/**
 * da9052_tsi_create_input_dev:
 * Description:This function performs initialization of 
 *	       TSI driver dtata structures allocates requisite
 *
 * @param  ip_dev 	pointer to array of input devices.
 * @param  n	 	number of input devices.
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t __init da9052_tsi_create_input_dev(struct input_dev** ip_dev, 
									u8 n)
{
	u8 i;
	s32 ret;
	struct input_dev *dev = NULL;

	/* Input device ip_dev will be initialized here only */ 
	/* So the pointer will be NULL. Null pointer check is not done */	
	if (!n)
		return (-EINVAL);

	for (i = 0; i < n; i++) {
		dev = input_allocate_device();		
		if (!dev) {
			DA9052_DEBUG(KERN_ERR "%s:%s():memory allocation for \
					inputdevice failed\n", __FILE__, 
								__FUNCTION__);
			return (-ENOMEM);
		}

		ip_dev[i] = dev;
		switch(i) {
		case TSI_INPUT_DEVICE_OFF:	
			dev->name = DA9052_TSI_INPUT_DEV;
			dev->phys = "input(tsi)";
			break;
		default:
			break;
		}
	}
        dev->id.vendor = DA9052_VENDOR_ID;
        dev->id.product = DA9052_PRODUCT_ID;
        dev->id.bustype = BUS_RS232;
        dev->id.version = TSI_VERSION;
	dev->keybit[BIT_WORD(BTN_TOUCH)] = BIT_MASK(BTN_TOUCH);
	dev->evbit[0] = (BIT_MASK(EV_SYN) | 
					BIT_MASK(EV_KEY) | 
						BIT_MASK(EV_ABS));

	input_set_abs_params(dev, ABS_X, 0, 0x3FF, 0, 0);
        input_set_abs_params(dev, ABS_Y, 0, 0x3FF, 0, 0);
	input_set_abs_params(dev, ABS_PRESSURE, 0, 0x3FF, 0, 0);

        /* register with input device sub-system of kernel */
        ret = input_register_device(dev);
        if(ret) {
                DA9052_DEBUG(KERN_ERR "%s: Could ", __FUNCTION__);
		DA9052_DEBUG("not register input device(touchscreen)!\n");
                ret = -EIO;
                goto fail;
        }
	return (SUCCESS);

fail:
	/*Removes allocted input devices incase of any failure*/
	for(;i--!=0;)	
		input_free_device(ip_dev[i]);
	return(-FAILURE);	
}

/**
 * da9052_tsi_init_drv:
 * Description:This function performs initialization of 
 *	       TSI driver dtata structures allocates requisite
 *	       memory, and commits the TSI configuration to 
 *	       H/W control registers.
 * @param  void 	None
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t __init da9052_tsi_init_drv(void)
{
	u8 cnt = 0;
	ssize_t ret = 0;
	da9052_tsi_info  *ts = get_tsi_drvdata();

	if ((DA9052_GPIO_PIN_3 != DA9052_GPIO_CONFIG_TSI) ||
		(DA9052_GPIO_PIN_4 != DA9052_GPIO_CONFIG_TSI) ||
		(DA9052_GPIO_PIN_5 != DA9052_GPIO_CONFIG_TSI) ||
		(DA9052_GPIO_PIN_6 != DA9052_GPIO_CONFIG_TSI) ||
		(DA9052_GPIO_PIN_7 != DA9052_GPIO_CONFIG_TSI)) {
		printk(KERN_ERR"DA9052_TSI: Configure DA9052 GPIO ");
		printk(KERN_ERR"pins for TSI\n");
		return (-FAILURE);
	}

	/* Initialize GPIO pins for TSI functionality */
	ret = da9052_tsi_config_gpio();

	/* Initialize TSI state to idle */
	ret = da9052_tsi_config_state(TSI_IDLE);
	ts->tsi_conf.state = TSI_IDLE;

	/* configure TSI measurement sequence */
	da9052_tsi_config_measure_seq(TSI_MODE_VALUE);

	/* Initialize default skip slots */
	da9052_tsi_config_skip_slots(TSI_SLOT_SKIP_VALUE);

	/* Initialize default delay value */
	da9052_tsi_config_delay(TSI_DELAY_VALUE);

	/* sets the Sampling intervel ADC module */
	da9052_tsi_set_sampling_mode(DEFAULT_TSI_SAMPLING_MODE);

	/* Initialize pointer to TSI calibration configuration */
	ts->tsi_calib = get_calib_config();

	/* Allocate & initialize kernel Input device */
	ret = da9052_tsi_create_input_dev(ts->input_devs, NUM_INPUT_DEVS);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI: %s: ", __FUNCTION__);
		DA9052_DEBUG("da9052_tsi_create_input_dev Failed \n" );
		return (ret);
	}

	/* Initialize data FIFO pointers & locks */
	da9052_init_tsi_fifos();

	/* Initialize REG processing thread */
	init_completion(&tsi_reg_proc_thread.notifier);
	tsi_reg_proc_thread.state = ACTIVE;
	tsi_reg_proc_thread.pid = kernel_thread(da9052_tsi_reg_proc_thread,
						NULL, 
						CLONE_KERNEL | SIGCHLD);

	/* Initialize RAW processing thread */
	init_completion(&tsi_raw_proc_thread.notifier);
	tsi_raw_proc_thread.state = ACTIVE;
	tsi_raw_proc_thread.pid = kernel_thread(da9052_tsi_raw_proc_thread, 
						NULL, 
						CLONE_KERNEL | SIGCHLD);

	/* Set default state for TSI module */
	ret = da9052_tsi_config_state(DEFAULT_TSI_STATE);
	if (ret) {
		/* Failed to initialize TSI state */
		/* Remove input device allocation */
		for(cnt = 0; cnt < NUM_INPUT_DEVS; cnt++ ) {

			/* Allocated input devices are free at */
			/* initialization failure */
			if( ts->input_devs[cnt] != NULL )
				input_free_device(ts->input_devs[cnt]);
		}
	}

	return (SUCCESS);
}

/**
 * da9052_tsi_get_input_dev: 
 * Description:	This function returns pointer to input device.
 *
 * @param  off	offset of input device in array. 		
 * @return u32	Pointer to input device structure.
 */
u32 da9052_tsi_get_input_dev(u8 off)
{
	da9052_tsi_info   *ts = get_tsi_drvdata();

	if(off > NUM_INPUT_DEVS-1)
		return (-EINVAL);

	return ((u32)ts->input_devs[off]);
}

/**
 * da9052_tsi_config_pen_detect: 
 * Description: This functions enables or disables pen detect mechanism of 
 *		TSI.
 *
 * @param  flag		pen detect ENABLE / DISABLE setting.
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t da9052_tsi_config_pen_detect(u8 flag)
{
	u8 data;
	u32 ret;
	da9052_tsi_info  *ts = get_tsi_drvdata();

	/* read TSI control A register */
	ret = read_da9052_reg(DA9052_TSICONTA_REG);
	if(ret < 0) {
		DA9052_DEBUG("%s:", __FUNCTION__);
		DA9052_DEBUG(" read_da9052_reg	Failed\n" );
		return (ret);
	}

	/* update as per flag */
	if(flag == ENABLE)
		data = set_bits((u8)ret,DA9052_TSICONTA_PENDETEN);
	else if(flag == DISABLE)
		data = clear_bits((u8)ret,DA9052_TSICONTA_PENDETEN);
	else {
		DA9052_DEBUG("%s:", __FUNCTION__);
		DA9052_DEBUG(" Invalid flag passed \n" );
		return (-EINVAL);
	}

	/* send to SSC */
	write_da9052_reg(DA9052_TSICONTA_REG, data);

	/* update s/w structure */
	ts->tsi_conf.auto_cont.da9052_tsi_cont_a = data;
	return (SUCCESS);
}

/**
 * da9052_tsi_disable_irq: 
 * Description: Function to disable the given PEN DWN or TSI_READY 
 *		IRQ pins
 *
 * @enum tsi_irq     Type of IRQ to be masked is given as input 
 * @return ssize_t   Returns Success or Failure
 */
 static ssize_t da9052_tsi_disable_irq(enum TSI_IRQ tsi_irq)
 {
	u8 data =0;
	ssize_t ret =0;
	da9052_tsi_info  *ts = get_tsi_drvdata();

	ret = read_da9052_reg(DA9052_IRQMASKB_REG);
	if(ret < 0)
	{
		DA9052_DEBUG("DA9052_TSI:da9052_tsi_disable_irq:");
		DA9052_DEBUG("read_da9052_reg Failed \n" );
		return (ret);			
	}
	/*Get the latest IRQ configuration*/
	data = ret;
	switch(tsi_irq)
	{
		case TSI_PEN_DWN:
			data = mask_pendwn_irq(data);
		break;
		case TSI_DATA_RDY:
			data = mask_tsi_rdy_irq(data);
		break;
		default:
			DA9052_DEBUG("DA9052_TSI:da9052_tsi_disable_irq:");
			DA9052_DEBUG("Invalid IRQ passed \n" );
			return (-EINVAL);			
	}
	ret = write_da9052_reg(DA9052_IRQMASKB_REG, data);
	if(ret)
	{
		DA9052_DEBUG("DA9052_TSI:da9052_tsi_disable_irq:");
		DA9052_DEBUG("write_da9052_reg Failed \n" );
		return (ret);			
	}

	/* if SSC write doesn't fail, then update s/w structure */
	switch(tsi_irq)
	{
		case TSI_PEN_DWN:
			ts->tsi_conf.tsi_pendown_irq_mask = SET;
		break;
		case TSI_DATA_RDY:
			ts->tsi_conf.tsi_ready_irq_mask = SET;
		break;
	}

	return (SUCCESS); 
 }

/**
 * da9052_tsi_enable_irq: 
 * Description: Function to enable the PEN DWN and TSI_READY 
 *		IRQ pins
 *
 * @enum tsi_irq     Type of IRQ to be enabled is given as input 
 * @return ssize_t   Returns Success or Failure
 */
 static ssize_t da9052_tsi_enable_irq(enum TSI_IRQ tsi_irq)
 {
	u8 data =0;
	ssize_t ret =0;
	da9052_tsi_info  *ts = get_tsi_drvdata();

	ret = read_da9052_reg(DA9052_IRQMASKB_REG);
	if(ret < 0)
	{
		DA9052_DEBUG("DA9052_TSI:da9052_tsi_enable_irq:");
		DA9052_DEBUG("read_da9052_reg Failed \n" );
		return (ret);			
	}
	/*Get the latest IRQ configuration*/
	data = ret;
	switch(tsi_irq)
	{
		case TSI_PEN_DWN:
			data = unmask_pendwn_irq(data);
		break;
		case TSI_DATA_RDY:
			data = unmask_tsi_rdy_irq(data);
		break;	
		default:
			DA9052_DEBUG("DA9052_TSI:da9052_tsi_enable_irq:");
			DA9052_DEBUG("Invalid IRQ passed \n" );
			return (-EINVAL);			
	}
	ret = write_da9052_reg(DA9052_IRQMASKB_REG, data);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI:da9052_tsi_enable_irq:");
		DA9052_DEBUG("write_da9052_reg Failed \n" );
		return (ret);			
	}	

	switch(tsi_irq)
	{
		case TSI_PEN_DWN:
			ts->tsi_conf.tsi_pendown_irq_mask = RESET;
		break;
		case TSI_DATA_RDY:
			ts->tsi_conf.tsi_ready_irq_mask = RESET;
		break;
	}

	return (SUCCESS); 
 }

/**
 * da9052_tsi_config_gpio:
 * Description: This functions configures DA9052 GPIO pins for TSI 
 *		functionality.
 *
 * @param  void
 * @return ssize_t   Returns Success or Failure
 */
static ssize_t da9052_tsi_config_gpio(void)
{
	u8 idx = 0;
	ssize_t ret = 0;
	da9052_ssc_msg ssc_msg[NUM_GPIO_TSI_REGISTERS];

	/* Read GPIO pins */
	ssc_msg[idx++].addr  = DA9052_GPIO0203_REG;
	ssc_msg[idx++].addr  = DA9052_GPIO0405_REG;
	ssc_msg[idx++].addr  = DA9052_GPIO0607_REG;

	/* Perform a hardware read */
	ret = da9052_ssc_read_many(ssc_msg,idx);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("da9052_ssc_read_many Failed\n" );
		return (ret);
	}

	/* Set the Pin functionality to TSI */
	idx = 0;
	ssc_msg[idx].data = clear_bits(ssc_msg[idx].data, 
						DA9052_GPIO0203_GPIO3PIN);
	idx++;
	ssc_msg[idx].data = clear_bits(ssc_msg[idx].data,
		(DA9052_GPIO0405_GPIO4PIN | DA9052_GPIO0405_GPIO5PIN));
	idx++;
	ssc_msg[idx].data = clear_bits(ssc_msg[idx].data,
		(DA9052_GPIO0607_GPIO6PIN | DA9052_GPIO0607_GPIO7PIN));
	idx++;

	/* Perform a hardware write */
	ret = da9052_ssc_write_many(ssc_msg,idx);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("da9052_ssc_read_many Failed\n" );
		return (ret);
	}

	return (SUCCESS);
}

/**
 * da9052_tsi_config_power_supply: 
 * Description: This functions enables or disables power supply of DA9052 
 *		TSI.
 *
 * @param  state	power supply ENABLE / DISABLE setting.
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t da9052_tsi_config_power_supply(u8 state)
{
	da9052_ldo_config ldo_config;
	da9052_tsi_info *ts = get_tsi_drvdata();

	if (state != ENABLE && state != DISABLE) {
		DA9052_DEBUG("DA9052_TSI: %s: ", __FUNCTION__);
		DA9052_DEBUG("Invalid state Passed\n" );
		return (-EINVAL);
	}

	/* LDO configuration */
	ldo_config.ldo_volt = DA9052_TSI_SUPPLY_VOLTAGE;
	ldo_config.ldo_num = DA9052_TSI_REF_SOURCE;
	ldo_config.ldo_conf = RESET;

	/* configure TSI's LDO */
	if (da9052_pm_configure_ldo(ldo_config))
		return (-FAILURE);

	/* Enable or disable TSI supply */
	if (da9052_pm_set_ldo(DA9052_TSI_REF_SOURCE, state))
		return (-FAILURE);

	if (state == ENABLE)
		ts->tsi_conf.ldo9_en = SET;
	else
		ts->tsi_conf.ldo9_en = RESET;

	return (SUCCESS);
}

/**
 * da9052_tsi_config_manual_coord: 
 * Description: This function configures TSI control registers to measure 
 *		along desired coordinate axis.
 *
 * @enum  coord		coordinate axis along which manual coordinate is to 
 *			be measured.
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t da9052_tsi_config_manual_coord(enum TSI_COORDINATE coord)
{
	u8 data;
	s32 ret;
	da9052_tsi_info *ts = get_tsi_drvdata();

	switch(coord) {
	case X_COORDINATE:
		/*TSI Reference is selected*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_adc_ref = SET;
		/*Configure TSI_MUX to X+*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_mux = TSI_MUX_XPLUS;
		/*Configure TSI SEL to Y+ and Y- Closed*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_0 = RESET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_1 = RESET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_2 = SET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_3 = SET;

		data = ts->tsi_conf.man_cont.da9052_tsi_cont_b;
		break;

	case Y_COORDINATE:
		/*TSI Reference is selected*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_adc_ref = SET;
		/*Configure TSI_MUX to Y+*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_mux = TSI_MUX_YPLUS;
		/*Configure TSI SEL to X+ and X- Closed*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_0 = SET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_1 = SET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_2 = RESET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_3 = RESET;

		data = ts->tsi_conf.man_cont.da9052_tsi_cont_b;	
		break;

	case Z_COORDINATE:
		/*TSI Reference is selected*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_adc_ref = SET;
		/*Configure TSI_MUX to Y+*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_mux = TSI_MUX_XPLUS;
		/*Configure TSI SEL to X+ and X- Closed*/
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_0 = RESET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_1 = SET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_2 = SET;
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_sel_3 = RESET;

		data = ts->tsi_conf.man_cont.da9052_tsi_cont_b;
		break;

		default:
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("Invalid coordinate passed \n" );
		return (-EINVAL);
	}

	/* write to hardware */
	ret = write_da9052_reg(DA9052_TSICONTB_REG, data);
	if(ret)
	{
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("write_da9052_reg Failed \n" );
			return (ret);
	}

	return (SUCCESS);
}

/**
 * da9052_tsi_config_auto_mode: 
 * Description: Auto TSI enable or diabling is done by this function
 *		
 * @param state   	TSI auto mode Enable or Disable setting.
 * @return ssize_t      Returns Success or Failure
 */
static ssize_t da9052_tsi_config_auto_mode(u8 state)
{
	u8 data;
	s32 ret = 0;
	da9052_tsi_info *ts = get_tsi_drvdata();

	if (state != ENABLE && state != DISABLE)
		return (-EINVAL);

	/*Get the latest value from the device*/
	/* Read manual mode control A register */
	ret = read_da9052_reg(DA9052_TSICONTA_REG);
	if(ret < 0) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("read_da9052_reg Failed \n" );
			return (ret);
	}

	/* set state */
	data = (u8)ret;

	if (state == ENABLE)
		data = set_auto_tsi_en(data);
	else if (state == DISABLE)
		data = reset_auto_tsi_en(data);
	else {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("Invalid Parameter Passed \n" );
		return (-EINVAL);
	}

	/*Write value to register*/
	ret = write_da9052_reg(DA9052_TSICONTA_REG, data);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" Failed to configure Auto TSI mode\n" );
		return (ret);
	}

	ts->tsi_conf.auto_cont.da9052_tsi_cont_a = data;
	return(SUCCESS);
}

/**
 * da9052_tsi_config_manual_mode: 
 * Description: This function is used to enable or disable manual TSI mode.
 *		It performs following features:
 *		1. Changes state of TSI_MAN bit to the one desired.
 *		2. Configures TSI_MUX and TSI_SEL values 
 *
 * @param  state 	TSI manual mode Enable or Disable setting.
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t da9052_tsi_config_manual_mode(u8 state)
{
	u8 data=0;
	ssize_t ret=0;
	da9052_tsi_info *ts = get_tsi_drvdata();

	if (state != ENABLE && state != DISABLE) {
		DA9052_DEBUG("DA9052_TSI: %s: ", __FUNCTION__);
		DA9052_DEBUG("Invalid state Passed\n" );
		return (-EINVAL);
	}


	/* Read manual mode control register */
	ret = read_da9052_reg(DA9052_TSICONTB_REG);
	if(ret < 0) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("read_da9052_reg Failed \n" );
			return (ret);
	}

	/* set state */
	data = (u8)ret;
	if (state == DISABLE) {
		data = disable_tsi_manual_mode(data);
	} else {
		data = enable_tsi_manual_mode(data);
	}

	/* write to hardware */
	ret = write_da9052_reg(DA9052_TSICONTB_REG, data);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("write_da9052_reg Failed \n" );
			return (ret);
	}

	/* update software data structure */
	if (state == DISABLE) {
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_man = RESET;
	} else {
		ts->tsi_conf.man_cont.tsi_cont_b.tsi_man = SET;
	}

	/* make TSI as ADC channel input */
	data = 0;
	data = set_bits(data, ADC_TSI);

	/* Write to Hardware */
	ret = write_da9052_reg(DA9052_ADCMAN_REG, data);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("ADC write Failed \n" );
		return (ret);
	}

	return (SUCCESS);
}

/**
 * da9052_tsi_read_manual_data: 
 * Description: The TSI X,Y co-ordinates are read by this function
 *		based on the coordinates requested.
 *
 * @param tsi_data	Pointer to store the read data 
 * @param coord		coordinate axis along which samples are to be taken.
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t da9052_tsi_read_manual_data(s16* tsi_data, 
						enum TSI_COORDINATE coord)
{
	u8 idx = 0;
	u8 lsb;
	s16 data = 0;
	ssize_t ret = 0;
	da9052_ssc_msg ssc_msg[NUM_TSI_RESULT_REGISTERS];

	if (!tsi_data)
		return (-EINVAL);

	if (coord == X_COORDINATE) {
		/*Read X coordinate */
		ssc_msg[idx++].addr  = DA9052_TSIXMSB_REG;
	}

	else if (coord == Y_COORDINATE) {
		/*Read Y coordinate */
		ssc_msg[idx++].addr  = DA9052_TSIYMSB_REG;
	}

	else if (coord == Z_COORDINATE) {
		/*Read Z coordinate */
		ssc_msg[idx++].addr  = DA9052_TSIZMSB_REG;
	}
	
	else {
		DA9052_DEBUG("DA9052_TSI: %s", __FUNCTION__);
		DA9052_DEBUG("Invalid Parameter Passed\n");
		return (-EINVAL);
	}
		
	/*Read lsb value */
	ssc_msg[idx++].addr  = DA9052_TSILSB_REG;

	ret = da9052_ssc_read_many(ssc_msg,idx);
	if(ret) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG("da9052_ssc_read_many Failed\n" );
		return (ret);
	}


	/* Read data from ssc message */
	lsb =  ssc_msg[--idx].data;

	/*Construct 10bit value*/
	switch (coord) {
	case X_COORDINATE:
		data = (ssc_msg[--idx].data << X_MSB_SHIFT);
		data |= (lsb & X_LSB_MASK) >> X_LSB_SHIFT;
		break;

	case Y_COORDINATE:
		data = (ssc_msg[--idx].data << Y_MSB_SHIFT);
		data |= (lsb & Y_LSB_MASK) >> Y_LSB_SHIFT;
		break;

	case Z_COORDINATE:
		data = (ssc_msg[--idx].data << Z_MSB_SHIFT);
		data |= (lsb & X_LSB_MASK) >> Z_LSB_SHIFT;
		break;
	}

	(*tsi_data) = data;

	return (SUCCESS);
}

/**
 * da9052_tsi_get_calib_data: 
 * Description: This function reads calibration samples from user space & 
 *		sets the calibration factors accordingly.
 *
 * @param calib_arr	Pointer to array of display & corresponding touchscreen 
 *		 	point coordinates.
 * @return ssize_t   	Returns Success or Failure
 */
static ssize_t da9052_tsi_get_calib_data(da9052_tsi_data* calib_arr)
{
	u8 num;
	s32 ret;
	u8 *src, *dst;
	u8 bytes_remaining;
	da9052_tsi_data calib_array[NUM_OF_SAMPLE_TYPES][NUM_CALIB_SAMPLES];

	dst = (u8* ) &calib_array[0][0];
	src = (u8* ) calib_arr;

	/* Update calibration matrix */
	/* This matrix holds the displacement between */
	/* display screen  &  touch screen */

	/* calculate the size of data in bytes */
	num = sizeof(da9052_tsi_data) * NUM_OF_SAMPLE_TYPES * 
							NUM_CALIB_SAMPLES;

	do {
		/* copy data from user space in a loop */
		/* till required number of bytes are read */
		bytes_remaining = copy_from_user(dst, src, num);

		/* update source & destination pointers as */
		/* per remaining data */
		src = (u8* ) ((u32)src + (u32)(num - bytes_remaining));
		dst = (u8* ) ((u32)dst + (u32)(num - bytes_remaining));

		num = bytes_remaining;
	} while (bytes_remaining);

	/* call to initialize calibration factors */
	ret = da9052_tsi_set_calib_matrix(calib_array[DISPLAY_DATA_OFFSET],
						calib_array[TS_DATA_OFFSET]);
	return (ret);
}

/** da9052_tsi_get_reg_data: 
 * Description:	This function reads touchscreen coordinate values from EH FIFO. 
 *	       	EH API expects a linear buffer to write data in to.
 *		So this function takes care to request samples only 
 *		in linear segments of REG data FIFO.
 *
 * @param  void		N.A.
 * @return u32		Number of bytes copied from event handler.
 */
static u32 da9052_tsi_get_reg_data(void)
{
	u32 free_cnt, copy_cnt, cnt;

	/* Lock Reg data FIFO */
	if(down_interruptible(&tsi_reg_fifo.lock)) {
		return (SUCCESS);
	}

	/* Call EH function to copy data from h/w */
	/* It returns the number of bytes copied */
	/* Note: This function assumes a linear buffer */

	copy_cnt = 0;

	/* Has data in REG FIFO wrapped around the last index of array */
	if ((tsi_reg_fifo.head - tsi_reg_fifo.tail) > 1) {

		/* Data has wrapped around circular FIFO, */
		/* we have a linear free buffer space  */
		
		/* Get free space available in REG FIFO */
		free_cnt = get_reg_free_space_cnt();
		if (free_cnt > TSI_POLL_SAMPLE_CNT)
			free_cnt = TSI_POLL_SAMPLE_CNT;

		/* copy data from EH */
		cnt = da9052_tsi_get_data( 
					&tsi_reg_fifo.data[tsi_reg_fifo.tail], 
					free_cnt 
					);
		if (cnt > free_cnt) {
			DA9052_DEBUG("EH copied more data");
			return(-FAILURE);
		}
		
		copy_cnt = cnt;

		/* Update REG FIFO tail index as per copied data */
		while(cnt--)
			incr_with_wrap_reg_fifo(tsi_reg_fifo.tail);

	} else if ((tsi_reg_fifo.head - tsi_reg_fifo.tail) <= 0) {

		/* we have free space in 2 linear buffers. */
		
		/* Get free space available in first linear segment */
		free_cnt = (TSI_REG_DATA_BUF_SIZE - tsi_reg_fifo.tail);
		
		if (free_cnt > TSI_POLL_SAMPLE_CNT) {
			/* We have enough free space in first segment */
			free_cnt = TSI_POLL_SAMPLE_CNT;

			/* copy data from EH */
			cnt = da9052_tsi_get_data(
					&tsi_reg_fifo.data[tsi_reg_fifo.tail], 
					free_cnt 
					);
			if (cnt > free_cnt) {
				DA9052_DEBUG("EH copied more data");
				return(-FAILURE);
			}
			copy_cnt = cnt;

			/* Update REG FIFO tail index as per copied data */
			while(cnt--)
				incr_with_wrap_reg_fifo(tsi_reg_fifo.tail);
		} else {
		
			/* we have free space in 2 linear buffers. */
	
			if (free_cnt) {
				/* We have free space in first segment */
				/* copy data from EH */
				cnt = da9052_tsi_get_data(
						&tsi_reg_fifo.data[
							tsi_reg_fifo.tail], 
						free_cnt 
						);
				if (cnt > free_cnt) {
					DA9052_DEBUG("EH copied more data");
					return(-FAILURE);
				}
				copy_cnt = cnt;
	
				/* Update REG FIFO tail index */
				/* as per copied data */
				while(cnt--)
					incr_with_wrap_reg_fifo(
							tsi_reg_fifo.tail);
			}
			
			/* Get free space available in second linear segment */
			free_cnt = tsi_reg_fifo.head;
			
			if (free_cnt > TSI_POLL_SAMPLE_CNT - copy_cnt) {
				/* Available space is more than what we need */
				free_cnt = TSI_POLL_SAMPLE_CNT - copy_cnt;
			}

			if (free_cnt) {
				/* We have free space in second segment */
				/* copy data from EH */
				cnt = da9052_tsi_get_data( 
						&tsi_reg_fifo.data[
							tsi_reg_fifo.tail], 
						free_cnt 
						);
				if (cnt > free_cnt) {
					DA9052_DEBUG("EH copied more data");
					return(-FAILURE);
				}

				copy_cnt += cnt;

				/* Update REG FIFO tail index as */
				/* per copied data */
				while(cnt--)
					incr_with_wrap_reg_fifo(
							tsi_reg_fifo.tail);
			}
		}

	} else {
		/* REG FIFO is full, new data was not requested */
		copy_cnt = 0;
	}

	/* Release REG FIFO */
	up(&tsi_reg_fifo.lock);

	/* Return num of samples copied from EH. */
	return (copy_cnt);
}

/** da9052_tsi_reg_proc_thread: 
 * Description:	This function is executed as a thread. It polls EH 
 *		periodically & copies TSI data for processing. 
 *		While it's state is active it performs following operations:
 *		1. sleeps for TSI_REG_DATA_READ_INTERVAL msecs.
 *		2. copies data from EH.
 *		3. processes this data and.
 *		4. checks if Pen-Up has occured.
 *
 * @param  ptr		Not Used. Provided for compatibility with 
 *			prototype of thread.
 * @return ssize_t	Returns Success or Failure
 */
static ssize_t da9052_tsi_reg_proc_thread (void *ptr)
{
	u32 data_cnt;
	da9052_tsi_info *ts;

	set_freezable();

	while(tsi_reg_proc_thread.state == ACTIVE) {

		/* make our thread friendly to system suspend */
		try_to_freeze();

		/* set how we want to sleep & go for it */
		set_current_state(TASK_INTERRUPTIBLE);
		schedule_timeout(msecs_to_jiffies(tsi_reg_data_poll_interval));

		ts = get_tsi_drvdata();

		if(!ts->pen_dwn_event) {
			/* Data without pen down is invalid */
			continue;
		}

		/* In every timer elapse if pen down has already happened */
		/*  then poll EH for TSI data */
		data_cnt = da9052_tsi_get_reg_data();
		
		/* process the data read from EH 
		 * Note: This function should be called even if data read 
		 * from EH is zero so that any residual data 
		 * in the FIFO gets processed 
		 */
		da9052_tsi_process_reg_data();

		if (data_cnt) {
			/*Reset the penup timer count*/
			ts->tsi_zero_data_cnt = 0;
		} else {
			/* zero samples were read from EH, */
			/* increment the zero data counter */
			if ((++(ts->tsi_zero_data_cnt)) > 
						     ts->tsi_penup_count) {
				/* waited enough for any data from EH */
				/* declare pen up */
				ts->pen_dwn_event = RESET;
				da9052_tsi_penup_event();
			}
		}
	}

	/* Indicate termination of this thread */
	complete_and_exit(&tsi_reg_proc_thread.notifier, 0);
	return (SUCCESS);
}

/**
 * da9052_tsi_penup_event: 
 * Description:This function is called when PEN_UP is detected.
 * 	       it puts TSI to idle mode and disables power 
 *	       supplied to it.
 *	       This function informs to eh to stop the 
 *	       TSI data reading.
 *
 * @param  void
 * @return ssize_t   	Returns Success or Failure
 */
void da9052_tsi_penup_event(void)
{

	da9052_tsi_info *ts = get_tsi_drvdata();

	/* Disable auto TSI measurement mode */
	/* Enable TSI auto measurement mode */
	if (da9052_tsi_config_auto_mode(DISABLE))
		goto exit;

	ts->tsi_conf.auto_cont.tsi_cont_a.auto_tsi_en = RESET;


	/* disable touchscreen's power supply */
	if (da9052_tsi_config_power_supply(ENABLE))
		goto exit;

	ts->tsi_conf.ldo9_en = RESET;


	/* enable TSI_PEN_DWN interrupt notification */
	if (da9052_tsi_enable_irq(TSI_PEN_DWN))
		goto exit;

	ts->tsi_conf.tsi_pendown_irq_mask = RESET;


	/*Call EH function to terminate the data reading*/
	da9052_tsi_stop_sampling();

	/* Reset zero data counter & flags */
	ts->tsi_zero_data_cnt = 0;
	early_data_flag = TRUE;
	debounce_over = FALSE;
	win_reference_valid = FALSE;

exit:
	/* Clean REG & RAW FIFOs */ 
	clean_tsi_fifos();
	return;
}

/**
 * da9052_tsi_pen_down_handler
 * Description:	This function is called when pen touches the touchscreen very 
 * 	       	first time after driver's PEN_UP status.
 *		There are no return error statements, since it'll be 
 *		called from EH. So status can't be returned.
 *
 * @param  u32 event 	DA9052 Event B Register value
 * @return None
 *
 * This function is called if
 * 	- Pen_Down event has occured
 *	- But TS_READY event has not yet occured
 */
void da9052_tsi_pen_down_handler(u32 event)
{
	ssize_t ret =0;
	da9052_tsi_info *ts = get_tsi_drvdata();

	/* Get the input device handler to report Pen down to */
	/* input infrastructure */ 
	struct input_dev* ip_dev = 
		(struct input_dev*)da9052_tsi_get_input_dev(
						(u8)TSI_INPUT_DEVICE_OFF);

	DA9052_DEBUG("EH notified the pendown event 0x%x\n", event);

	/* If TSI state is not Auto mode */
	/* then return with error */
	if (ts->tsi_conf.state != TSI_AUTO_MODE) {
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" Configure TSI to auto mode.\n" );
		DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
		DA9052_DEBUG(" Then call this API.\n" );
		goto fail;
	}

	/* enable touchscreen's power supply */
	if (da9052_tsi_config_power_supply(ENABLE))
		goto fail;


	/* disable PEN_DOWN interrupt notification */
	if (da9052_tsi_disable_irq(TSI_PEN_DWN))
		goto fail;


	/* enable TSI_RDY interrupt notification */
	if (da9052_tsi_enable_irq(TSI_DATA_RDY))
		goto fail;


	/* Enable TSI auto measurement mode */
	if (da9052_tsi_config_auto_mode(ENABLE))
		goto fail;
	ts->tsi_conf.auto_cont.tsi_cont_a.auto_tsi_en = SET;


	/* Report Pen down to OS */
	input_report_abs(ip_dev, BTN_TOUCH, 1);
	input_sync(ip_dev);		
		
	/* update s/w data structure after reporting to OS */
	ts->tsi_rdy_event = (DA9052_EVENTB_ETSIREADY & (event>>8));
	ts->pen_dwn_event = (DA9052_EVENTB_EPENDOWN & (event>>8));

	/*Add EH function to start sampling*/
	da9052_tsi_start_sampling();
	
	goto success;


fail:
	/* Error occured, */
	/* workaround to get next pen down interrupt from EH */
	if (ts->pd_reg_status) {
		/* Unregister from EH */
		da9052_eh_unregister_nb(&ts->pd_nb);
		ts->pd_reg_status = RESET;

		/* register with EH */
		da9052_tsi_reg_pendwn_event();
	}

success:
	/* Dummy statement for label */
	ret = SUCCESS;
}

/**
 * da9052_tsi_open: 
 * Description:	This function is called when opening DA9052 TSI device.
 * 	       	It checks if device has already been opened. If yes, then 
 * 		it returns with error.
 *
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return int   Error Code, zero: no error
 */
static ssize_t da9052_tsi_open(struct inode *inode, struct file *file)
{
	/* Check if touchscreen device has already been opened */
	if(tsi_device_open) {
		printk(KERN_INFO"da9052: TouchScreen already opened \n");
		return(-EBUSY);
	} else {
		/* Touchscreen's opened the first time. */
		/* use tsi_device_open variable as */
		/* a flag to denote touchscreen's */
		/* Open / Close status */
		tsi_device_open++;
		return(SUCCESS);
	}
}

/**
 * da9052_tsi_release: 
 *Description: This function is called when closing DA9052 TSI device.
 *
 * @param *inode pointer to device inode
 * @param *file  file pointer
 * @return int   Error Code, zero: no error
 */
static ssize_t da9052_tsi_release(struct inode *inode, struct file *file)
{
	/* Resetting tsi_device_open flag so that */
	/* next user can  open the device  */
	tsi_device_open--;
	return(SUCCESS);
}

/**
 *da9052_tsi_remove: 
 *Description: This function is called when detatching DA9052 TSI
 *	       device from the driver.
 * @param void
 * @return void
 */
static int __devexit da9052_tsi_remove(struct platform_device *dev)
{

	DA9052_DEBUG(KERN_DEBUG "Removing %s \n", DA9052_TSI_DEVICE_NAME);
	return (SUCCESS);
}

/**
 * da9052_tsi_suspend:
 *Description:	Power Management support function. Puts TSI to idle mode.
 * 
 * @param *dev   pointer to platform device
 * @param state  pm state
 * @return s32 status of suspend operation
 */
static ssize_t da9052_tsi_suspend(struct platform_device *dev, 
							pm_message_t state)
{
	/* Put your suspend related operations here */
	printk(KERN_INFO "%s: called\n", __FUNCTION__);

	return (SUCCESS);
}

/**
 * da9052_tsi_resume: 
 *Description:	Power Management support function. Brings TSI to default state.
 * 
 * @param *dev   pointer to platform device
 * @return s32 status of resume operation
 */
static ssize_t da9052_tsi_resume(struct platform_device *dev)
{
	/* Put your resume related operations here */
	printk(KERN_INFO "%s: called\n", __FUNCTION__);

	return (SUCCESS);
}

/**
 * da9052_tsi_ioctl: 
 * Description:	It provides the user interface functionality to check the 
 *		da9052 TSI supported functions. 
 * @param *inode 	pointer to device inode.
 * @param *file 	file pointer.
 * @param cmd  		ioctl command to be executed.
 * @param arg  		any argument to be passed.
 * @return int   	operation status SUCCESS/FAILURE.
 */
static ssize_t da9052_tsi_ioctl(struct inode *inode, struct file *file,
					unsigned int cmd, unsigned long arg)
{
	ssize_t ret;
	da9052_tsi_ioctl_data usr;
	da9052_tsi_data tsi_data;
	enum TSI_STATE	tsi_state;
	da9052_tsi_info *ts = get_tsi_drvdata();
	
	ret = SUCCESS;

	/* Copy argument data from user space */
	if(copy_from_user(&usr, (da9052_tsi_ioctl_data *)arg, 
					sizeof(da9052_tsi_ioctl_data)))
		return (-EFAULT);

	switch(cmd) {
	case DA9052_TSI_SET_SAMPLING_MODE:
		ret = da9052_tsi_set_sampling_mode(usr.interval);
		break;

	case DA9052_TSI_CONFIG_DELAY:
		ret = da9052_tsi_config_delay(usr.delay);
		break;

	case DA9052_TSI_CONFIG_SLOT_SKIP:
		ret = da9052_tsi_config_skip_slots(usr.skip);
		break;

	case DA9052_TSI_MEASURE_MANUAL_DATA:
		/*Start Manual conversion*/
		ret = da9052_tsi_get_manual_measurement(&tsi_data);

		/* copy this sample to user space */
		if(copy_to_user(usr.tsi_data, &tsi_data, 
						sizeof(da9052_tsi_data))) {
			DA9052_DEBUG("DA9052_TSI: %s:", __FUNCTION__);
			DA9052_DEBUG(" Copy to user Failed\n" );
			return (-FAILURE);
		}
		break;

	case DA9052_TSI_MEASURE_SEQ:
		ret = da9052_tsi_config_measure_seq(usr.seq);
		break;

	case DA9052_TSI_SET_CALIB_MATRIX:
		ret = da9052_tsi_get_calib_data(usr.calib_arr);
		break;

	case DA9052_TSI_GET_TSI_CONFIG:
		if(copy_to_user(usr.tsi_conf,
					&ts->tsi_conf,
					sizeof(da9052_tsi_conf))) {
			ret = -EFAULT;
		}
		break;

	case DA9052_TSI_GET_CALIB_CONFIG:
		ts->tsi_calib = get_calib_config();
		if(copy_to_user(usr.calib,
					ts->tsi_calib,
					sizeof(calib_cfg_t))) {
			ret = -EFAULT;
		}
		break;

	case DA9052_TSI_SET_CALIB_CONFIG:
		ts->tsi_calib = get_calib_config();
		if(copy_from_user(ts->tsi_calib,
					usr.calib,
					sizeof(calib_cfg_t))) {
			ret = -EFAULT;
		}
		break;

	case DA9052_TSI_SET_DEVICE_STATE:
		/* set TSI device state to auto or manual mode */
		ret = da9052_tsi_config_state(usr.tsi_state);
		break;

	case DA9052_TSI_CONFIG_PEN_DET:
		/* enable / disable PEN_DOWN_DET mechanism of DA9052 */
		ret = da9052_tsi_config_pen_detect(usr.pen_det);
		break;

	case DA9052_TSI_REG_PENDWN_EH:
		/*Register pendwn event with EH*/
		da9052_tsi_reg_pendwn_event();
		break;

	case DA9052_TSI_UNREG_PENDWN_EH:
		/* Un-register from the EH for the requested event */
		da9052_eh_unregister_nb(&ts->pd_nb);
		/* Modify the status to un registered */ 
		ts->pd_reg_status = RESET;
		break;

	case DA9052_TSI_DISABLE_IRQ:
		ret = da9052_tsi_disable_irq(usr.tsi_irq);
		break;

	case DA9052_TSI_ENABLE_IRQ:
		ret = da9052_tsi_enable_irq(usr.tsi_irq);
		break;

	case DA9052_TSI_POWER_CONTROL:
		/* Enable / Disable TSI power supply */
		ret = da9052_tsi_config_power_supply(usr.pwr);
		break;

	case DA9052_TSI_SET_MANUAL_AXIS:
		/* configure the coordinate axis for manual measurement */
		ret = da9052_tsi_config_manual_coord(usr.axis);
		break;

	case DA9052_TSI_GET_DEVICE_STATE:
		/* set TSI device state to auto or manual mode */
		tsi_state = da9052_tsi_get_drv_state();
		if(copy_to_user(usr.get_tsi_state,
					&tsi_state,
					sizeof(enum TSI_STATE))) {
			ret = -EFAULT;
		}
		break;

	case DA9052_TSI_SET_GPIO_PINS:
		/* configure DA9052 GPIO pins for TSI */
		ret = da9052_tsi_config_gpio();
		break;

	default:
		/*Return faliure to IOCTL call*/
		ret = -FAILURE;
	}

	return (ret);
}

/**
 * static struct file_operations da9052_ts_fops -  
 * This structure definition is used by kernel to map it's generic operations
 * to device specific operations. 
 * @owner: 	Points to the owner of this module.
 * @open : 	function pointer to TSI's open method.
 * @release: 	function pointer to TSI's release device method.
 * @ioctl: 	function pointer to TSI's I/O control method.
 */
static const struct file_operations da9052_tsi_fops = {
	.owner		= THIS_MODULE,
	.open		= da9052_tsi_open,
	.release	= da9052_tsi_release,
	.ioctl		= da9052_tsi_ioctl
};

/**
 * da9052_tsi_probe:
 *Description: Called when a device gets attached to driver 
 *
 * @param dev	reference to platform device object
 * @return s32  Error Code, zero: no error
 */
static s32 __devinit da9052_tsi_probe(struct platform_device *dev)
{
	s32 ret;

	/* Register the TSI device driver & it's API's with kernel */
	ret = register_chrdev(tsi_major_number, DA9052_TSI_DEVICE_NAME, 
							&da9052_tsi_fops);
	if (ret != SUCCESS) {
		printk(KERN_ERR "Unable to register %s\n",
						DA9052_TSI_DEVICE_NAME);
		return (ret);	
	} else {
		printk(KERN_INFO "%s: Major number is: %d.\n", 
				DA9052_TSI_DEVICE_NAME, tsi_major_number);

		/* Perform DA9052 specific H/W initializations */
		if (da9052_tsi_init_drv()) {
			/* Error in hw initialization */
			unregister_chrdev(tsi_major_number, 
						DA9052_TSI_DEVICE_NAME);
			return (-EFAULT);
		}

		printk(KERN_INFO "TSI Drv Successfully Inserted %s\n", 
						DA9052_TSI_DEVICE_NAME);
		return (SUCCESS);
	}
}

/**
 * static struct platform_driver da9052_tsi_driver -
 * This structure definition has to be defined here as an exception.
 * @probe: Probe function for this device. 
 * @remove: Function to be called when removing this device from platform
 * @suspend: Function to be called in suspend mode
 * @resume: Function to be called in resume mode
 * @driver: Contains glue logic to bind platform device and plarform driver
 */
static struct platform_driver da9052_tsi_driver = {
	.probe		= da9052_tsi_probe,
	.remove		= __devexit_p(da9052_tsi_remove),
	.suspend	= da9052_tsi_suspend,
	.resume		= da9052_tsi_resume,
	.driver		= {
				.name	= DA9052_TSI_DEVICE_NAME,
				.owner	= THIS_MODULE,
			},
};

/**
 * da9052_tsi_init: 
 * Description:	This function registers DA9052 TSI platform 
 *	       	device with kernel.
 * 
 * @param void 	None
 * @return int  Returns Success or Failure
 */
static int __init da9052_tsi_init(void)
{
	int retval;
	printk(banner);

	/* Allocate memory for a platform device object for TSI */
	da9052_tsi_platform_device = platform_device_alloc("da9052_tsi", 0);
	if (!da9052_tsi_platform_device)
		return (-ENOMEM);

	/* Add TSI platform device to kernel's list of devices */
	retval = platform_device_add(da9052_tsi_platform_device);
	if (retval < 0) {
		platform_device_put(da9052_tsi_platform_device);
		return (retval);
	}

	/* Register the TSI as a platform device with the kernel */
	retval = platform_driver_register(&da9052_tsi_driver);
	if (retval < 0)
		platform_device_unregister(da9052_tsi_platform_device);

	return (retval);
}

/**
 * da9052_tsi_exit: 
 * Description: This function release the resources and unregisters 
 *		TSI driver from the platform device
 * @param void
 * @return void
 */
static void __exit da9052_tsi_exit(void)
{
	int i;
	da9052_tsi_info *ts = get_tsi_drvdata();
	ssize_t ret = 0;

	/* Disable the TSI on DA9052  */
	ret = da9052_tsi_config_state(TSI_IDLE);
	if(ret == SUCCESS) {
		DA9052_DEBUG(KERN_ERR "TSI Device Successfully Disabled \n");
	}

	/* Checks registration with EH for pendown is done or not */
	if(ts->pd_reg_status){
		/* Un-register from the EH for the requested event */	
		da9052_eh_unregister_nb(&ts->pd_nb);
		ts->pd_reg_status = RESET;
	}	

	/* stop processing thread */
	tsi_raw_proc_thread.state = INACTIVE;
	tsi_reg_proc_thread.state = INACTIVE;

	/* wait for threads to exit */
	wait_for_completion(&tsi_raw_proc_thread.notifier);
	wait_for_completion(&tsi_reg_proc_thread.notifier);

	/*Unregister the input device*/
	for (i=0; i < NUM_INPUT_DEVS; i++)
	{
		input_unregister_device(ts->input_devs[i]);
	}	
	
	/*Unregister the char device*/
	unregister_chrdev(tsi_major_number, DA9052_TSI_DEVICE_NAME);

	/*Unregister the platform device*/
	platform_driver_unregister(&da9052_tsi_driver);
	platform_device_unregister(da9052_tsi_platform_device);

	printk(KERN_ERR "TSI Driver %s Successfully Removed \n", 
					DA9052_TSI_DEVICE_NAME);
}


module_init(da9052_tsi_init);
module_exit(da9052_tsi_exit);

MODULE_AUTHOR("Dialog Semiconductor Ltd");
MODULE_DESCRIPTION("DA9052 Touch Screen Device Driver");
MODULE_LICENSE("GPL");
/*--------------------------------------------------------------------------*/
/* Exports                                                                  */
/*--------------------------------------------------------------------------*/
EXPORT_SYMBOL(da9052_tsi_set_sampling_mode);
EXPORT_SYMBOL(da9052_tsi_config_skip_slots);
EXPORT_SYMBOL(da9052_tsi_config_delay);
EXPORT_SYMBOL(da9052_tsi_config_measure_seq);

EXPORT_SYMBOL(da9052_tsi_config_state);
EXPORT_SYMBOL(da9052_tsi_get_manual_measurement);
EXPORT_SYMBOL(da9052_tsi_get_drv_state);
