/*
 * da9052_tsi_calibrate.c: 	File to hold calibration functins for TSI 
 *				driver.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (08/05/2009):	- Moved calibration functions to this file.
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_tsi.h"

/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/

/* This default init is for perfect alignment */
/* between  display & touch screen */
static Calib_xform_matrix_t xform = {
		.An = 1,
		.Bn = 0,
		.Cn = 0,
		.Dn = 0,
		.En = 1,
		.Fn = 0,
		.Divider = 1
};

static calib_cfg_t calib = {
	.calibrate_flag = TSI_USE_CALIBRATION,
};

/*--------------------------------------------------------------------------*/
/* Global Functions                                                         */
/*--------------------------------------------------------------------------*/
/**
 * get_calib_matrix:	returns pointer to calibration data.
 * @param void	
 * @return Calib_xform_matrix_t* 
 */
calib_cfg_t* get_calib_config(void)
{
	return (&calib);
}

/**
 *  Function: da9052_tsi_set_calib_matrix()
 *
 *  Description:	Calling this function with valid input data
 *			in the display and screen input arguments 
 *			causes the calibration factors between the
 *			screen and display points to be calculated,
 *			and the output argument - matrixPtr - to be 
 *			populated.
 *
 *			This function needs to be called only when new
 *			calibration factors are desired.
 *
 *
 *  @param  displayPtr (input) -	Pointer to an array of three 
 *					sample, reference points.
 *  @param  screenPtr (input) -		Pointer to the array of touch 
 *					screen points corresponding 
 *					to the reference display points.
 *
 * @return SUCCESS - 	the calibration matrix was correctly 
 *			calculated and its value is in the 
 *			output argument.
 *         -FAILURE - 	an error was detected and the function failed to 
 *			return a valid set of matrix values. The only 
 *			time this sample code returns -FAILURE is when 
 *			Divider is equal to 0.
 */
ssize_t da9052_tsi_set_calib_matrix(da9052_tsi_data *displayPtr,
                          da9052_tsi_data *screenPtr)
{

	int  retValue = SUCCESS ;

	xform.Divider = ((screenPtr[0].x - screenPtr[1].x) 
					* (screenPtr[1].y - screenPtr[2].y)) 
		 			- ((screenPtr[1].x - screenPtr[2].x) 
					* (screenPtr[0].y - screenPtr[1].y)); 

	if( xform.Divider == 0 )
	{
		retValue = -FAILURE ;
	}
	else
	{
		xform.An = ( (displayPtr[0].x - displayPtr[1].x) 
					* (screenPtr[1].y - screenPtr[2].y) ) 
			 		- ( (displayPtr[1].x - displayPtr[2].x) 
						* (screenPtr[0].y - screenPtr[1].y) ); 

		xform.Bn = ( (displayPtr[1].x - displayPtr[2].x) 
					* (screenPtr[0].x - screenPtr[1].x) ) 
			 		- ( (screenPtr[1].x - screenPtr[2].x) 
						* (displayPtr[0].x - displayPtr[1].x) );

		 xform.Cn = (displayPtr[0].x * xform.Divider) 
				 		- (screenPtr[0].x * xform.An) 
						- (screenPtr[0].y * xform.Bn);
		 
		xform.Dn = ( (displayPtr[0].y - displayPtr[1].y) 
					* (screenPtr[1].y - screenPtr[2].y) ) 
					- ( (displayPtr[1].y - displayPtr[2].y) 
						* (screenPtr[0].y - screenPtr[1].y) );

	        xform.En = ( (displayPtr[1].y - displayPtr[2].y) 
					* (screenPtr[0].x - screenPtr[1].x) ) 
						- ( (screenPtr[1].x - screenPtr[2].x) 
						* (displayPtr[0].y - displayPtr[1].y) );

		xform.Fn = (displayPtr[0].y * xform.Divider) 
						- (screenPtr[0].x * xform.Dn) 
						- (screenPtr[0].y * xform.En);
	}
 
    return( retValue ) ;
}


/**
 *  Function: da9052_tsi_get_calib_display_point()
 *
 *  Description: Given a valid set of calibration factors and a point
 *                value reported by the touch screen, this function
 *                calculates and returns the true (or closest to true)
 *                display point below the spot where the touch screen 
 *                was touched.
 * 
 *  @param displayPtr (output) - Pointer to the calculated (true) display point.
 *
 * @return 	SUCCESS - the display point was correctly calculated 
 *              	       and its value is in the output argument.
 * 		-FAILURE - an error was detected and the function
 *                      	   failed to return a valid point.
 */
ssize_t da9052_tsi_get_calib_display_point( da9052_tsi_data * displayPtr )
{
    int  retValue = TRUE;
    da9052_tsi_data screen_coord;
    
    screen_coord = *displayPtr;
    if( xform.Divider != 0 )
    {
	/*
	* Operation order is important since we are doing integer 
	*  math. We add all terms together before      
	*  dividing, so that the remainder is not rounded off     
	*  prematurely.                                         
	*/
	displayPtr->x = ( (xform.An * screen_coord.x) + 
                          (xform.Bn * screen_coord.y) + 
                           xform.Cn 
                        ) / xform.Divider ;

	displayPtr->y = ( (xform.Dn * screen_coord.x) + 
                          (xform.En * screen_coord.y) + 
                           xform.Fn 
                        ) / xform.Divider ;
    }
    else
        /* Error */
	retValue = FALSE;

#if DA9052_TSI_CALIB_DATA_PROFILING
	/* Print calibrated coordinate */
	printk("C\tX\t%4d\tY\t%4d\n",
					displayPtr->x,
					displayPtr->y);
#endif
    return( retValue ) ;
}

EXPORT_SYMBOL(da9052_tsi_get_calib_display_point);
EXPORT_SYMBOL(da9052_tsi_set_calib_matrix);
