/*
 * da9052_tsi_filter.c: Driver for Touch Screen interface filter
 *			on DA9052 chip.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (07/05/2009):	- Moved Average filter to this file.
 *			- Added window filter infrastructure.
 *
 * (25/05/2009):	- Added FIFO design to process data.
 *
 * (11/06/2009):	- Added thread to process touchscreen coordinates
 *			  read from event handler buffer. These unprocessed
 *			  or RAW coordinates are then passed through average
 *			  filter, calibration mechanism and reported OS.
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_tsi.h"

/*--------------------------------------------------------------------------*/
/* Local Macro Definitions                                                  */
/*--------------------------------------------------------------------------*/

/* This macro returns the absolute value of input value */
#define get_abs(x)	(x < 0 ? ((-1) * x) : (x))

/* Macro to check if coordinate lies within the configured window size */
#define WITHIN_WINDOW(x,y)	((get_abs(x) <= TSI_X_WINDOW_SIZE) && \
					(get_abs(y) <= TSI_Y_WINDOW_SIZE))


/* Macro define for first sample's index in FIFO */
#define FIRST_SAMPLE		0

/* Increment with wrap operation for ring buffer */
#define incr_with_wrap_raw_fifo(x)			\
		if(++x >= TSI_RAW_DATA_BUF_SIZE)	\
			x = 0

/* Decrement with wrap operation for ring buffer */
#define decr_with_wrap_raw_fifo(x)			\
		if(--x < 0)				\
			x = (TSI_RAW_DATA_BUF_SIZE-1)

/* Increment with wrap operation for ring buffer */
#define incr_with_wrap_reg_fifo(x)			\
		if(++x >= TSI_REG_DATA_BUF_SIZE)	\
			x = 0


/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/

/* Flag to indicate if processing of early data is over or not */
/* so RAW FIFO processing thread could process data */
u8 early_data_flag;

/* Flag to indicate if debouncing has finished or not */
u8 debounce_over;

/* Flag to indicate if reference sample for window filter is valid or not */
u8 win_reference_valid;

/* REG data processing thread */
tsi_thread_type 		tsi_reg_proc_thread;

/* FIFO to hold RAW data */
da9052_tsi_raw_fifo 		tsi_raw_fifo;

/* RAW data processing thread */
da9052_tsi_reg_fifo		tsi_reg_fifo;

/* FIFO to hold REG data */
tsi_thread_type 		tsi_raw_proc_thread;

/* Time period (in msecs) for processing of RAW FIFO */
u32 tsi_raw_data_poll_interval;


/*--------------------------------------------------------------------------*/
/* Local Function Prototypes                                                */
/*--------------------------------------------------------------------------*/
static u32 get_raw_data_cnt (void);

static void da9052_tsi_convert_reg_to_coord (da9052_tsi_data* raw_data);


static s32 da9052_tsi_raw_proc_thread (void *ptr);

static inline void clean_tsi_reg_fifo(void);

static inline void clean_tsi_raw_fifo(void);

#if (ENABLE_AVERAGE_FILTER)
static void da9052_tsi_avrg_filter(da9052_tsi_data* tsi_avg_data);

#endif

#if (ENABLE_TSI_DEBOUNCE)
static s32 da9052_tsi_calc_debounce_data(da9052_tsi_data* raw_data);

#endif

# if (ENABLE_WINDOW_FILTER)
static s32 diff_within_window(da9052_tsi_data* prev_raw_data, 
			      da9052_tsi_data* cur_raw_data);

static s32 da9052_tsi_window_filter(da9052_tsi_data* raw_data);

#endif
/*--------------------------------------------------------------------------*/
/* Global Functions                                                         */
/*--------------------------------------------------------------------------*/
/**
 * clean_tsi_fifos: 
 * Description: 	This function changes the FIFO state to empty by
 *			re-initializing head & tail pointers.
 *
 * @param  void 	
 * @return void
 */
void clean_tsi_fifos(void)
{
	clean_tsi_raw_fifo();
	clean_tsi_reg_fifo();
}

/**
 * da9052_init_tsi_fifos: 
 * Description: 	This function initializes the REG & RAW data
 *			processing FIFOs.
 *
 * @param  void 	
 * @return void
 */
void __init da9052_init_tsi_fifos (void)
{
	/* Initialize TSI FIFO locks */
	init_MUTEX(&tsi_raw_fifo.lock);
	init_MUTEX(&tsi_reg_fifo.lock);

	/* Initialize FIFO state variables */
	clean_tsi_raw_fifo();
	clean_tsi_reg_fifo();
}

/**
 * get_reg_data_cnt: 
 * Description: 	This function returns the number of raw coordinate 
 *			samples present in REG data processing FIFO.
 *
 * @param  void 	
 * @return u32   	Returns Success or Failure
 */
u32 get_reg_data_cnt (void)
{
	u8 reg_data_cnt;

	/* First find available data in REG FIFO */
	if (tsi_reg_fifo.head <= tsi_reg_fifo.tail) {
		/* no overflow occured */
		reg_data_cnt = (tsi_reg_fifo.tail - tsi_reg_fifo.head);
	} else {
		/* overflow occured, calculate valid data */
		reg_data_cnt = (tsi_reg_fifo.tail + (TSI_REG_DATA_BUF_SIZE - 
							tsi_reg_fifo.head));
	}

	return (reg_data_cnt);
}

/**
 * get_reg_free_space_cnt: 
 * Description: 	This function returns the free space present 
 *			in REG data processing FIFO.
 *
 * @param  void 	
 * @return u32   	Returns Success or Failure
 */
u32 get_reg_free_space_cnt(void)
{
	u32 free_cnt;

	/* First find available space in TSI FIFO */
	if (tsi_reg_fifo.head <= tsi_reg_fifo.tail) {
		/* no overflow occured */
		free_cnt = ((TSI_REG_DATA_BUF_SIZE - 1) - 
				(tsi_reg_fifo.tail - tsi_reg_fifo.head));
	} else {
		/* overflow occured, valid data is divided to two parts. */
		/* first count data at rear end */
		free_cnt = ((tsi_reg_fifo.head - tsi_reg_fifo.tail) - 1);
	}

	return (free_cnt);
}

/**
 * da9052_tsi_process_reg_data
 * Description:	This function processes REG data FIFO one sample at a time 
 *		- It pops a sample from REG data FIFO, converts it 
 *		  to coordinate format.
 *		- Checks if sample lies within TS dimensions.
 *		- Applies debouncing, if debouncing is enabled.
 *		- If window filter is enabled, it passes this sample 
 *		  through window filter.
 *		- If the sample's valid then moves it to RAW data FIFO.
 *
 * @param void
 * @return void
 */
void da9052_tsi_process_reg_data(void)
{
	s32 ret;
	da9052_tsi_data tmp_raw_data;
	u32 reg_data_cnt;

	/* Lock Reg data FIFO */
	if(down_interruptible(&tsi_reg_fifo.lock)) {
		return;
	}

	/* Find available data in REG FIFO */
	reg_data_cnt = get_reg_data_cnt();

	while (reg_data_cnt-- > 0) {

		/* we have sample to be processed */
		ret = SUCCESS;

		/* check if raw FIFO is full */
		if (get_raw_data_cnt() >= (TSI_RAW_DATA_BUF_SIZE - 1)) {
			/* FIFO is full */
			DA9052_DEBUG("%s: RAW data FIFO is full\n",
							__FUNCTION__);
			break;
		}

		/* convert & copy available sample from */
		/* REG format to co-ordinate format  */
		da9052_tsi_convert_reg_to_coord(&tmp_raw_data);

		/* Is data within touchscreen panel dimensions? */
		if ((tmp_raw_data.x < TS_X_MIN) ||
				   (tmp_raw_data.x > TS_X_MAX) ||
				   (tmp_raw_data.y < TS_Y_MIN) ||
				   (tmp_raw_data.y > TS_Y_MAX)) {
			/* Data beyond touchscreen boundries */
			/* reject this sample & continue with next one */
			DA9052_DEBUG("%s: ", __FUNCTION__);
			DA9052_DEBUG("sample beyond touchscreen panel ");
			DA9052_DEBUG("dimensions\n");
			continue;
		}

#if (ENABLE_TSI_DEBOUNCE)
		if(debounce_over == FALSE) {
			/* Debounce */
			ret = da9052_tsi_calc_debounce_data (&tmp_raw_data);
			if (ret != SUCCESS) {
				/* Don't start filtering till */
				/* debouncing is over */
				continue;
			}
		}
#endif

# if (ENABLE_WINDOW_FILTER)
		/* Apply Window Filter */
		ret = da9052_tsi_window_filter(&tmp_raw_data);
		if (ret != SUCCESS) {
			/* Sample was rejected by window filter */
			continue;
		}
#endif

		/* Early data processing is over now */
		early_data_flag = FALSE;

		/* We have valid data with us */

		/* Lock Raw data FIFO */
		if(down_interruptible(&tsi_raw_fifo.lock)) {
			/* RAW FIFO lock could not be acquired */
			DA9052_DEBUG("%s: Failed to ", __FUNCTION__);
			DA9052_DEBUG("acquire RAW FIFO Lock!\n");

			/* Release REG FIFO lock before error exit */
			up(&tsi_reg_fifo.lock);
			return;
		}

		/* update RAW data FIFO */
		tsi_raw_fifo.data[tsi_raw_fifo.tail] = tmp_raw_data;
		incr_with_wrap_raw_fifo(tsi_raw_fifo.tail);

		/* Release raw FIFO */
		up(&tsi_raw_fifo.lock);
	}


	/* Release reg FIFO */
	up(&tsi_reg_fifo.lock);

	return;
}

/*--------------------------------------------------------------------------*/
/* Local Functions                                                          */
/*--------------------------------------------------------------------------*/
/**
 * get_raw_data_cnt: 
 * Description: 	This function returns the number of raw coordinate 
 *			samples present in RAW data processing FIFO.
 *
 * @param  void 	
 * @return u32   	Returns Success or Failure
 */
static u32 get_raw_data_cnt (void)
{
	u32 raw_data_cnt;

	/* First find available data in RAW FIFO */
	if (tsi_raw_fifo.head <= tsi_raw_fifo.tail) {
		/* no overflow occured */
		raw_data_cnt = (tsi_raw_fifo.tail - tsi_raw_fifo.head);
	} else {
		/* overflow occured, calculate valid data */
		raw_data_cnt = (tsi_raw_fifo.tail + (TSI_RAW_DATA_BUF_SIZE - 
							tsi_raw_fifo.head));
	}

	return (raw_data_cnt);
}

#if 0

/* This function is not used in current implementation */
/* It can be uncommented if it has to be used */
/**
 * get_raw_free_space_cnt: 
 * Description: 	This function returns the free space present 
 *			in RAW data processing FIFO.
 *
 * @param  void 	
 * @return u32   	Returns Success or Failure
 */
static u32 get_raw_free_space_cnt(void)
{
	u32 free_cnt;

	/* First find available space in TSI FIFO */
	if (tsi_raw_fifo.head <= tsi_raw_fifo.tail) {
		/* no overflow occured */
		free_cnt = ((TSI_REG_DATA_BUF_SIZE - 1) - 
				(tsi_raw_fifo.tail - tsi_raw_fifo.head));
	} else {
		/* overflow occured, valid data is divided to two parts. */
		/* first count data at rear end */
		free_cnt = ((tsi_raw_fifo.head - tsi_raw_fifo.tail) - 1);
	}

	return (free_cnt);
}
#endif

/**
 * convert_reg_to_coord
 * Description: This function pops one sample in register format 
 *		from REG_FIFO converts it to co-ordinate format and stores 
 *		it to RAW FIFO.
 *
 * @param  raw_data	return pointer to store result.
 * @return void
 */
static void da9052_tsi_convert_reg_to_coord (da9052_tsi_data* raw_data)
{

	/* update raw data FIFO till free space is available */
	da9052_tsi_reg* src;
	da9052_tsi_data* dst = raw_data;

	src = &tsi_reg_fifo.data[tsi_reg_fifo.head];

	/* Convert data from REG format to co-ordinate format */
	dst->x = (src->x_msb << X_MSB_SHIFT);
	dst->x |= (src->lsb & X_LSB_MASK) >> X_LSB_SHIFT;

	dst->y = (src->y_msb << Y_MSB_SHIFT);
	dst->y |= (src->lsb & Y_LSB_MASK) >> Y_LSB_SHIFT;

	dst->z = (src->z_msb << Z_MSB_SHIFT);
	dst->z |=(src->lsb & Z_LSB_MASK) >> Z_LSB_SHIFT;

#if DA9052_TSI_RAW_DATA_PROFILING
	/* Print raw co-ordinate data */
	printk("R\tX\t%4d\tY\t%4d\tZ\t%4d\n",	
					(u16)dst->x,
					(u16)dst->y,
					(u16)dst->z);
#endif
	/* Wrap FIFO indices if they exceed FIFO length */
	incr_with_wrap_reg_fifo(tsi_reg_fifo.head);
}

#if (ENABLE_AVERAGE_FILTER)
/**
 * da9052_tsi_avrg_filter: 
 * Description:	This function pops out 'size' number of samples starting 
 *		from head of RAW data processing FIFO, calculates average &
 *		stores the result in out parameter 'tsi_avg_data'.
 *
 * @param tsi_avg_data	Return pointer for averaged out co-ordinate.
 * @param size		number of samples over which averaging is to 
 *			be performed.
 * @return void
 */
static void da9052_tsi_avrg_filter(da9052_tsi_data* tsi_avg_data)
{
	u8 cnt;

	/* acquire lock for RAW FIFO */
	if(down_interruptible(&tsi_raw_fifo.lock)) {
		printk("%s: No RAW Lock !\n", __FUNCTION__);
		return;
	}

	/* Assign first data */
	(*tsi_avg_data) = tsi_raw_fifo.data[tsi_raw_fifo.head];
	incr_with_wrap_raw_fifo(tsi_raw_fifo.head);

	/* Sum X,Y and Z data for average filter size */
	for (cnt=2; cnt <= TSI_AVERAGE_FILTER_SIZE; cnt++) {

		tsi_avg_data->x += tsi_raw_fifo.data[tsi_raw_fifo.head].x;
		tsi_avg_data->y += tsi_raw_fifo.data[tsi_raw_fifo.head].y;
		tsi_avg_data->z += tsi_raw_fifo.data[tsi_raw_fifo.head].z;

		incr_with_wrap_raw_fifo(tsi_raw_fifo.head);
	}

	/* Release lock for RAW FIFO */
	up(&tsi_raw_fifo.lock);

	/* Find the averaged X,Y and Z value by */ 
	/* dividing by number of co-ordinate samples */
	tsi_avg_data->x /= TSI_AVERAGE_FILTER_SIZE;
	tsi_avg_data->y /= TSI_AVERAGE_FILTER_SIZE;
	tsi_avg_data->z /= TSI_AVERAGE_FILTER_SIZE;

#if DA9052_TSI_AVG_FLT_DATA_PROFILING
	/* the refined/averaged X, Y & Z co-ordinates */
	printk("A\tX\t%4d\tY\t%4d\tZ\t%4d\n", 
	      			 (u16)tsi_avg_data->x,
				 (u16)tsi_avg_data->y,
				 (u16)tsi_avg_data->z);
#endif

	return;
}
#endif

/**
 * da9052_tsi_raw_proc_thread: 
 * Description:	This function is executed as a thread. It polls RAW data 
 *		FIFO periodically to check if data is available for processing. 
 *		As long as this thread's state is active, it performs 
 *		following operations:
 *		1. sleeps for TSI_RAW_DATA_PROCESSING_INTERVAL msecs.
 *		2. checks if RAW data FIFO has data to be processed.
 *		3. if average filter has been enabled, 
 *		   applies it to the RAW FIFO data.
 *		4. calibrates the final coordinate sample & checks it's range.
 *		5. reports this sample to the kernel.
 *
 * @param  ptr	Not Used. Provided for compatibility with prototype of thread.
 * @return s32	SUCCESS / FAILURE
 */
static s32 da9052_tsi_raw_proc_thread (void *ptr)
{
	da9052_tsi_data coord;
	u8 calib_ok, range_ok;
	calib_cfg_t *tsi_calib = get_calib_config();
	struct input_dev* ip_dev = (struct input_dev*)
				da9052_tsi_get_input_dev(
						(u8)TSI_INPUT_DEVICE_OFF);

	set_freezable();

	while(tsi_raw_proc_thread.state == ACTIVE) {

		/* make our thread friendly to system suspend */
		try_to_freeze();

		/* set how we want to sleep & go for it */
		set_current_state(TASK_INTERRUPTIBLE);
		schedule_timeout(msecs_to_jiffies(tsi_raw_data_poll_interval));

		/* We're awake */
		/* Check for availibility of data */
		if (early_data_flag || (get_raw_data_cnt() == 0)) {
			/* not enough data, continue the loop */
			continue;
		}

		/* Values checked for every iteration */
		calib_ok = TRUE;
		range_ok = TRUE;

#if (ENABLE_AVERAGE_FILTER)

		/* Average Filter */
		if (get_raw_data_cnt() < TSI_AVERAGE_FILTER_SIZE) {
			/* Not enough data to perform average */
			continue;
		}
		da9052_tsi_avrg_filter (&coord);

#else

		/* No filter's used */
		/* acquire lock for RAW FIFO */
		if(down_interruptible(&tsi_raw_fifo.lock)) {
			continue;
		}

		/* update RAW FIFO */
		coord = tsi_raw_fifo.data[tsi_raw_fifo.head];
		incr_with_wrap_raw_fifo(tsi_raw_fifo.head);

		/* Release lock for RAW FIFO */
		up(&tsi_raw_fifo.lock);

#endif

		/* All data processing over */
		/* Check if calibration is to be used. */
		if (tsi_calib->calibrate_flag) {
			calib_ok = da9052_tsi_get_calib_display_point(&coord);

			/* Is final data within display panel dimensions ? */
			if ((coord.x < DISPLAY_X_MIN) || 
				(coord.x > DISPLAY_X_MAX) || 
				(coord.y < DISPLAY_Y_MIN) || 
						(coord.y > DISPLAY_Y_MAX))
				range_ok = FALSE;
		}

		if (calib_ok && range_ok) {
			/* Report data to input dev infrastructure. */
			input_report_abs(ip_dev, ABS_X, coord.x);
			input_report_abs(ip_dev, ABS_Y, coord.y);
			input_report_abs(ip_dev, ABS_Z, coord.z);
			input_sync(ip_dev);

#if DA9052_TSI_OS_DATA_PROFILING
			printk("O\tX\t%4d\tY\t%4d\tZ\t%4d\n",  (u16)coord.x,
								(u16)coord.y,
								(u16)coord.z);
#endif
		} else {
			if (!calib_ok) {
				DA9052_DEBUG("%s: ", __FUNCTION__);
				DA9052_DEBUG("calibration Failed\n");
			}
			if (!range_ok) {
				DA9052_DEBUG("%s: ", __FUNCTION__);
				DA9052_DEBUG("sample beyond display ");
				DA9052_DEBUG("panel dimension\n");
			}
		}
	}

	/* Indicate termination of this thread */
	complete_and_exit(&tsi_raw_proc_thread.notifier, 0);
	return (SUCCESS);
}

#if (ENABLE_TSI_DEBOUNCE)
/**
 * da9052_tsi_calc_debounce_data: 
 * Description:	This function calculates debounce over initial set of 
 *		samples after change to a pen-down state. The number of 
 *		samples over which debounce is calculated is configurable.
 *
 * @param  raw_data	reference of data sample which has to be debounced.
 * @return s32		SUCCESS / FAILURE
 */
static s32 da9052_tsi_calc_debounce_data(da9052_tsi_data* raw_data)
{
#if (TSI_DEBOUNCE_DATA_CNT)
	u8 cnt;
	da9052_tsi_data	temp = {.x=0, .y=0, .z=0};

	/* store raw data to FIFO */
	tsi_raw_fifo.data[tsi_raw_fifo.tail] = (*raw_data);
	incr_with_wrap_raw_fifo(tsi_raw_fifo.tail);

	if (get_raw_data_cnt() <= TSI_DEBOUNCE_DATA_CNT) {
		/* Not enough data for debounce to process */
		return (-FAILURE);
	}

	/* Debounce: sum initial samples & average them */
	for (cnt = 1; cnt <= TSI_DEBOUNCE_DATA_CNT; cnt++) {
		temp.x += tsi_raw_fifo.data[tsi_raw_fifo.head].x;
		temp.y += tsi_raw_fifo.data[tsi_raw_fifo.head].y;
		temp.z += tsi_raw_fifo.data[tsi_raw_fifo.head].z;

		incr_with_wrap_raw_fifo(tsi_raw_fifo.head);
	}

	temp.x /= TSI_DEBOUNCE_DATA_CNT;
	temp.y /= TSI_DEBOUNCE_DATA_CNT;
	temp.z /= TSI_DEBOUNCE_DATA_CNT;

	/* clear out FIFO */
	tsi_raw_fifo.tail = tsi_raw_fifo.head;
	/* insert debounced sample */
	tsi_raw_fifo.data[tsi_raw_fifo.tail] = temp;
	incr_with_wrap_raw_fifo(tsi_raw_fifo.tail);

#if DA9052_TSI_PRINT_DEBOUNCED_DATA
	/* This print should be enabled in order */
	/* to observe the debounced value */
	printk("D: X: %d Y: %d Z: %d\n",
	       (u16)temp.x,
	       (u16)temp.y,
	       (u16)temp.z);
#endif

#endif //(TSI_DEBOUNCE_DATA_CNT) ends

	/* debouncing completed */
	debounce_over = TRUE;

	return (SUCCESS);
}
#endif

# if (ENABLE_WINDOW_FILTER)
/**
 * diff_within_window
 * Description:	This function takes as its input, pointers to two 
 *		TSI coordinates. It then checks if these lie
 *		within configured window filter size.
 *
 * @param prev_raw_data		pointer to previous valid coordinate.
 * @param cur_raw_data		pointer to current sample.
 * @return ssize_t		SUCCESS: current sample lies within window  
 *						range w.r.t. previous sample.
 *				-FAILURE: current sample lies beyond window  
 *						range w.r.t. previous sample.
 */
static s32 diff_within_window(da9052_tsi_data* prev_raw_data, 
					da9052_tsi_data* cur_raw_data)
{

	s32 ret = -FAILURE;
	s32 x, y;

	/* Find difference between current */
	/* and previous co-ordinate */
	x = ((cur_raw_data->x) - (prev_raw_data->x));
	y = ((cur_raw_data->y) - (prev_raw_data->y));

	if (WITHIN_WINDOW(x,y)) {

#if DA9052_TSI_WIN_FLT_DATA_PROFILING
		/* Print window filtered data */
		printk("W\tX\t%4d\tY\t%4d\tZ\t%4d\n",	
					(u16)cur_raw_data->x,
					(u16)cur_raw_data->y,
					(u16)cur_raw_data->z);
#endif
		/* sample lies within configured window size */
		ret = SUCCESS;
	}
	return (ret);
}

/**
 * da9052_tsi_window_filter
 * Description:	This function has the main window filter mechanism.
 *		It calculates & sets first sample for reference.
 *
 * @param raw_data	pointer to TSI coordinate data.
 * @return ssize_t	SUCCESS: current sample successfully passed through 
 *				window filter.
 *			-FAILURE: current sample rejected by window filter.
 */
static s32 da9052_tsi_window_filter(da9052_tsi_data* raw_data)
{
	u8 ref_found;
	u32 cur, next;
	s32 ret = -FAILURE;
	static da9052_tsi_data prev_raw_data;

	if (win_reference_valid == TRUE) {

#if DA9052_TSI_PRINT_PREVIOUS_DATA
		/* Debug print to be enabled if value previous */
		/* to current sample needs to be observed */
		printk("P: X: %d Y: %d Z: %d\n",
					(u16)prev_raw_data.x,
					(u16)prev_raw_data.y,
					(u16)prev_raw_data.z);
#endif
		/* Apply window filter on current sample */
		ret = diff_within_window(&prev_raw_data, raw_data);
		if (ret == SUCCESS) {
			/* update previous RAW data */
			prev_raw_data = (*raw_data);
		}

	} else {
		/* Reference sample for window filter */
		/* was not calculated  */

		/* store raw data to FIFO */
		tsi_raw_fifo.data[tsi_raw_fifo.tail] = (*raw_data);
		incr_with_wrap_raw_fifo(tsi_raw_fifo.tail);

		if (get_raw_data_cnt() == SAMPLE_CNT_FOR_WIN_REF) {

			/* We have enough samples to calculate */
			/* reference sample for window filter */
			ref_found = FALSE;

			next = cur = tsi_raw_fifo.head;
			incr_with_wrap_raw_fifo(next);

			while (next <= tsi_raw_fifo.tail) {
				/* Find if samples lie within */
				/* configured window size  */
				ret = diff_within_window(
						&tsi_raw_fifo.data[cur],
						&tsi_raw_fifo.data[next]
						);
				if (ret == SUCCESS) {
					/* Valid samples found from which */
					/* reference sample can be */
					/* calculated */
					ref_found = TRUE;
					break;
				}
				/* Increment indices */
				incr_with_wrap_raw_fifo(cur);
				incr_with_wrap_raw_fifo(next);

				/* If we haven't reached end-of-FIFO */
				/*  then loop again */
			}

			if (ref_found == FALSE) {
				/* Clear FIFO and wait  for  */
				/* next set of samples */
				tsi_raw_fifo.tail = tsi_raw_fifo.head;
			} else {
				/* Calculating reference sample */
				prev_raw_data = tsi_raw_fifo.data[cur];

				prev_raw_data.x += tsi_raw_fifo.data[next].x;
				prev_raw_data.y += tsi_raw_fifo.data[next].y;
				prev_raw_data.z += tsi_raw_fifo.data[next].z;

				prev_raw_data.x = prev_raw_data.x /2;
				prev_raw_data.y = prev_raw_data.y /2;
				prev_raw_data.z = prev_raw_data.z /2;

				/* Put reference sample in return pointer */
				(*raw_data) = prev_raw_data;

				/* Clear FIFO off remaining samples */
				tsi_raw_fifo.tail = tsi_raw_fifo.head;

				/* Release the flag & return SUCCESS */
				win_reference_valid = TRUE;
				ret = SUCCESS;
			}
		}
	}

	return (ret);
}
#endif //ENABLE_WINDOW_FILTER ends

/**
 * clean_tsi_reg_fifo: 
 * Description: 	This function changes the REG FIFO state to empty by
 *			re-initializing head & tail pointers to first sample 
 *			of the FIFO.
 *
 * @param  void 	
 * @return void
 */
static inline void clean_tsi_reg_fifo(void)
{
	/* clean buffer state variables */
	tsi_reg_fifo.head = FIRST_SAMPLE;
	tsi_reg_fifo.tail = FIRST_SAMPLE;
}

/**
 * clean_tsi_raw_fifo: 
 * Description: 	This function changes the RAW FIFO state to empty by
 *			re-initializing head & tail pointers to first sample 
 *			of the FIFO.
 *
 * @param  void 	
 * @return void
 */
static inline void clean_tsi_raw_fifo(void)
{
	/* clean buffer state variables */
	tsi_raw_fifo.head = FIRST_SAMPLE;
	tsi_raw_fifo.tail = FIRST_SAMPLE;
}

